/* API - Any Device - Any OS */
/* $Workfile: gsc_dma.h $ */
/* $Revision: 5 $ */
/* $Date: 10/27/07 10:06a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/gsc_dma.h $
 *
 * 5     10/27/07 10:06a Dnewbold
 * Ported to Linux.
 *
 * 4     3/01/07 5:24p Dnewbold
 * Linted and migrated to 32-bit and 64-bit Linux.
 *
 * 3     5/09/06 1:31p Dnewbold
 * Increased code modularity.
 *
 * 2     5/04/06 3:36p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 11:48a Dnewbold
*/

#ifndef __GSC_DMA_H__
#define __GSC_DMA_H__

#include "gsc_event.h"		/*lint !e537 */
#include "gsc_sem.h"		/*lint !e537 */

// See additional include(s) below.



/* #define	*****************************************************************/

#define	GSC_DMA_FLAG_BLDMA			0x01	// Blind DMA
#define	GSC_DMA_FLAG_DMDMA			0x02	// Demand Mode DMA
#define	GSC_DMA_FLAG_RX				0x04
#define	GSC_DMA_FLAG_TX				0x08


#define	GSC_DMA_FLAG_BLDMA_RX		(GSC_DMA_FLAG_BLDMA | GSC_DMA_FLAG_RX)
#define	GSC_DMA_FLAG_BLDMA_TX		(GSC_DMA_FLAG_BLDMA | GSC_DMA_FLAG_TX)
#define	GSC_DMA_FLAG_DMDMA_RX		(GSC_DMA_FLAG_DMDMA | GSC_DMA_FLAG_RX)
#define	GSC_DMA_FLAG_DMDMA_TX		(GSC_DMA_FLAG_DMDMA | GSC_DMA_FLAG_TX)

#define	GSC_DMA_TYPE_NONE			0
#define	GSC_DMA_TYPE_BLOCK			1
#define	GSC_DMA_TYPE_CHAINED		2



/* typedefs	*****************************************************************/

typedef enum
{
	GSC_DMA_PRI_CH_0,
	GSC_DMA_PRI_CH_1,
	GSC_DMA_PRI_SHARE
} gsc_dma_pri_t;

typedef struct
{
	// These setting might change per API settings.

	int				bus_width;		// In bits. 8, 16, 32
	int				demand_mode;	// Enable?
	gsc_dma_pri_t	priority;

	// These settings are typically fixed by the hardware design.

	int				bterm_input;	// Enable?
	int				dma_eot_pin;	// Enable?
	int				fifo_mode;		// Local address held constant?
	int				local_bursting;	// Enable?
	int				ready_input;	// Enable?
	int				stop_xfer_mode;	// Enable
	int				wait_states;	// 0 - 15
	int				wr_inval_mode;	// Enable?
} gsc_dma_info_t;

typedef struct
{
	U32				bytes;
	int				done_irq;
	U32				local_adrs;
	int				local_to_pci;
	U32				phys_adrs;		// For BLOCK mode DMA. Zero if not.
	UL				user_adrs;		// For SGL mode DMA. Zero if not.
} gsc_dma_xfer_t;

typedef struct
{
	U32				in_use;
	U8				index;
	U32				capabilities;
	U32				type;			// None, Block, Scatter-Gather
	gsc_event_t*	irq;
} gsc_dma_channel_t;

typedef struct
{
	gsc_device_t*		device;
	gsc_sem_t			sem;
	gsc_dma_channel_t	ch0;
	gsc_dma_channel_t	ch1;
} gsc_dma_t;



#include "gsc_io.h"		/*lint !e537 */



/* prototypes	*************************************************************/

U32		gsc_dma_abort(gsc_io_t* io);
U32		gsc_dma_channel_close(gsc_dma_t* dma, gsc_dma_channel_t** channel);
U32		gsc_dma_create(gsc_dma_t* dma, gsc_device_t* device);
void	gsc_dma_destroy(gsc_dma_t* dma);
U32		gsc_dma_done(gsc_io_t* gio);
U32		gsc_dma_init(gsc_dma_t* dma, U32 ch0, U32 ch1);
U32		gsc_dma_transfer(gsc_io_t* io, U32 LocToPci);

U32		gsc_dma_channel_open(
						gsc_dma_t*			dma,
						gsc_dma_channel_t**	channel,
						U32					type,
						U32					need,
						U32					(*fn_desc)(	gsc_io_t*		io,
														gsc_dma_info_t*	info),
						gsc_io_t*			io);



#endif
