/* API - Any Device - Any OS */
/* $Workfile: gsc_drv.h $ */
/* $Revision: 5 $ */
/* $Date: 10/27/07 10:06a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/gsc_drv.h $
 *
 * 5     10/27/07 10:06a Dnewbold
 * Ported to Linux.
 *
 * 4     3/01/07 5:25p Dnewbold
 * Linted and added/updated prototypes for Linux migration (32/64-bit).
 *
 * 3     5/09/06 1:31p Dnewbold
 * Increased code modularity.
 *
 * 2     5/05/06 10:35a Dnewbold
 *
 * 1     5/05/06 8:45a Dnewbold
*/

#ifndef __GSC_DRV_H__
#define __GSC_DRV_H__

#include "dev_drv.h"



/* prototypes	*************************************************************/

// These functions are implemented in gsc_os_drv.c.

U32	gsc_drv_close(gsc_device_t* device);
U32	gsc_drv_init(void);
U32	gsc_drv_open(gsc_device_t* device, U8 index, const char* name_upper);

U32	gsc_drv_bar_map(gsc_device_t* device, S8 bar, void *pva);
U32	gsc_drv_bar_size_get(gsc_device_t* device, S8 bar, U32* size);
U32	gsc_drv_bar_unmap(gsc_device_t* device, S8 bar, void* pva);

U32	gsc_drv_reg_io_mod(gsc_device_t* device, S8 bar, U16 offset, U8 size, U32 value, U32 mask);
U32	gsc_drv_reg_io_read(gsc_device_t* device, S8 bar, U16 offset, U8 size, U32* value);
U32	gsc_drv_reg_io_write(gsc_device_t* device, S8 bar, U16 offset, U8 size, U32 value);
U32	gsc_drv_reg_mem_mod(gsc_device_t* device, S8 bar, U16 offset, U8 size, U32 value, U32 mask);
U32	gsc_drv_reg_mem_read(gsc_device_t* device, S8 bar, U16 offset, U8 size, U32* value);
U32	gsc_drv_reg_mem_write(gsc_device_t* device, S8 bar, U16 offset, U8 size, U32 value);
U32	gsc_drv_reg_pci_read(gsc_device_t* device, U16 offset, U8 size, U32* value);

U32	gsc_drv_version_get(gsc_device_t* device, U8* api_ver, char* name, U8 size);
U32	gsc_drv_version_verify(gsc_device_t* device);
U32	gsc_drv_version_verify_dev_open(int* open);



#if DEV_USES_DMA

#include "gsc_dma.h"		/*lint !e537 */

U32	gsc_drv_dma_abort(gsc_device_t* device, U8 index);
U32	gsc_drv_dma_block_close(gsc_device_t* device, U8 index);
U32	gsc_drv_dma_block_open(gsc_device_t* device, U8 index, gsc_dma_info_t* pInfo);
U32	gsc_drv_dma_block_xfer(gsc_device_t* device, U8 index, gsc_dma_xfer_t* xfer);
U32	gsc_drv_dma_sgl_close(gsc_device_t* device, U8 index);
U32	gsc_drv_dma_sgl_open(gsc_device_t* device, U8 index, gsc_dma_info_t* pInfo);
U32	gsc_drv_dma_sgl_xfer(gsc_device_t* device, U8 index, gsc_dma_xfer_t* xfer);
U32	gsc_drv_dma_status(gsc_device_t* device, U8 index);

#endif



#if DEV_USES_IRQ

#include "gsc_irq.h"		/*lint !e537 */

U32		gsc_drv_irq_attach(gsc_device_t* device, gsc_irq_src_t* irq, os_event_t* event);
U32		gsc_drv_irq_disable(gsc_device_t* device);
U32		gsc_drv_irq_enable(gsc_device_t* device);
U32		gsc_drv_irq_status(gsc_device_t* device, gsc_irq_src_t* irq);
U32		gsc_drv_irq_master_event_create(gsc_irq_t* irq, gsc_device_t* device);
void	gsc_drv_irq_master_event_destroy(gsc_irq_t* irq);

#endif



#if DEV_USES_MEM

#include "gsc_mem.h"		/*lint !e537 */

U32	gsc_drv_mem_alloc(gsc_device_t* device, U32* size, unsigned long* phy_adrs);
U32	gsc_drv_mem_free(gsc_device_t* device, U32 size, U32 phy_adrs);
U32	gsc_drv_mem_map(gsc_device_t* device, U32 size, U32 phy_adrs, UL* usr_adrs);
U32	gsc_drv_mem_unmap(gsc_device_t* device, U32 size, U32 phy_adrs, UL usr_adrs);

#endif



#endif
