/* API - Any Device - Any OS */
/* $Workfile: gsc_sem.c $ */
/* $Revision: 1 $ */
/* $Date: 5/04/06 3:49p $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/gsc_sem.c $
 *
 * 1     5/04/06 3:49p Dnewbold
*/

#include "gsc_sem.h"



/*****************************************************************************
*
*	Function:	gsc_sem_create
*
*	Purpose:
*
*		Create a semaphore.
*
*	Arguments:
*
*		sem			The GSC specific semaphore data is stored here.
*
*		name		A name to associate with the semaphore.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_sem_create(gsc_sem_t* sem, const char* name)
{
	U32	status;

	status	= os_sem_create(&sem->sem, name);
	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_sem_destroy
*
*	Purpose:
*
*		Destroy a semaphore.
*
*	Arguments:
*
*		sem			The GSC specific semaphore data is stored here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_sem_destroy(gsc_sem_t* sem)
{
	U32	status;

	status	= os_sem_destroy(&sem->sem);
	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_sem_lock
*
*	Purpose:
*
*		Acquire a semaphore.
*
*	Arguments:
*
*		sem			The GSC specific semaphore data is stored here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_sem_lock(gsc_sem_t* sem)
{
	U32	status;

	status	= os_sem_lock(&sem->sem);
	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_sem_unlock
*
*	Purpose:
*
*		sem		The GSC specific semaphore data is stored here.
*
*	Arguments:
*
*		sem		The semaphore to acquire.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_sem_unlock(gsc_sem_t* sem)
{
	os_sem_unlock(&sem->sem);
}


