/* API - Any Device - Any OS */
/* $Workfile: gsc_version.c $ */
/* $Revision: 6 $ */
/* $Date: 4/02/07 4:54p $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/gsc_version.c $
 *
 * 6     4/02/07 4:54p Dnewbold
 * Ported to Linux.
 *
 * 5     3/01/07 5:07p Dnewbold
 * Linted. Updated for those environs that don't need to open a device to
 * get the driver version number.
 *
 * 4     5/09/06 1:31p Dnewbold
 * Increased code modularity.
 *
 * 3     5/05/06 10:35a Dnewbold
 *
 * 2     5/04/06 3:48p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 11:51a Dnewbold
*/

#include <string.h>

#include "gsc_version.h"

#include "gsc_drv.h"

#include "dev_version.h"



/*****************************************************************************
*
*	Function:	gsc_version_get
*
*	Purpose:
*
*		Provide version data about a specific software component.
*
*	Arguments:
*
*		device		The data used to access the device.
*
*		id			The item whose version number is desired.
*
*		version		The version number string is put here.
*
*		size		This is the size of the above buffer.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_version_get(gsc_device_t* device, U8 id, char* version, size_t size)
{
	U8	api;
	U32	status;

	version[0]	= 0;

	switch (id)
	{
		default:

			status	= (U32) GSC_INVALID_DATA;
			break;

		case GSC_VERSION_LIBRARY:

			strncpy(version, API_GSC_FILE_VERSION_STR, size);
			version[size - 1]	= 0;
			status				= (U32) GSC_SUCCESS;
			break;

		case GSC_VERSION_DRIVER:

			size	= (size > 255) ? 255 : size;
			status	= gsc_drv_version_get(	device,
											&api,
											version,
											(U8) size);
			break;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	gsc_version_verify
*
*	Purpose:
*
*		Verify that the versions of things we're working with are sufficient.
*		The driver version is checked only if we've got an open handle.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 gsc_version_verify(void)
{
	gsc_device_t	device;
	int				open;
	U32				status;

	gsc_drv_version_verify_dev_open(&open);

	if (open)
	{
		status	= gsc_open(0, 0, &device, NULL);

		if (status == (U32) GSC_SUCCESS)
		{
			status	= gsc_drv_version_verify(&device);
			gsc_close(&device);
		}
	}
	else
	{
		status	= gsc_drv_version_verify(NULL);
	}


	return(status);
}


