/* API - Any Device - Linux */
/* $Workfile: gsc_os_device.h $ */
/* $Revision: 2 $ */
/* $Date: 10/27/07 10:15a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/api/linux/gsc_os_device.h $
 *
 * 2     10/27/07 10:15a Dnewbold
 * Ported to Linux.
 *
 * 1     4/02/07 4:59p Dnewbold
*/

#ifndef __GSC_OS_DEVICE_H__
#define __GSC_OS_DEVICE_H__

typedef struct _os_device_t	os_device_t;

#include "gsc_common.h"
#include "gsc_os_event.h"
#include "gsc_os_key.h"
#include "gsc_os_sem.h"
#include "gsc_os_thread.h"



/* #defines	*****************************************************************/

#define	OS_DEVICE_HANDLE(dp)			((dp)->fd)
#define	OS_DEVICE_HANDLE_IS_VALID(dp)	(OS_DEVICE_HANDLE(dp))



/* #defines	*****************************************************************/

struct _os_device_t
{
	os_key_t	key;	// First field!
	int			fd;
	char		name[64];

	// Event waiting is handled by putting the thread to sleep until it is
	// triggered. The thread is put in an event list before going to sleep
	// so that the triggerring mechanism knows who to resume once an event
	// is triggerred.

	struct
	{
		os_sem_t			sem;	// Must lock to access list.
		int					quit;	// Set this to abort all processing.
		os_event_node_t*	head;
		os_event_node_t*	tail;
	} events;

	// A master thread is created to handle interrupt notification in the API.
	// The thread sits in a loop where it waits for an interrupt then triggers
	// the PCI event so that the common code can check to see which interrupt
	// occurred and then respond accordingly.

	struct
	{
		int			error;	// The thread sets this if there is an error.
		int			quit;	// Set before requesting that the thread abort.
		os_thread_t	thread;	// This is the master thread.
		os_event_t*	event;	// Trigger this when we get an interrupt.
	} master;

};



/* prototypes	*************************************************************/

U32	os_device_close(os_device_t* device);
U32	os_device_hc_base_index(int* hc, U8* base_index);
U32	os_device_name_set(os_device_t* device, const char* name);
U32	os_device_open(int index, const char* name, os_device_t* device);
U32	os_device_t_init(os_device_t* device);



#endif
