/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* Any Device - Driver - Linux */
/* $Workfile: close.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 10:31a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/driver/linux/close.c $
 *
 * 3     10/27/07 10:31a Dnewbold
 * Additional driver porting updates.
 *
 * 2     3/29/07 2:44p Dnewbold
 *
 * 1     5/10/06 1:02p Dnewbold
*/

#include "main.h"



/******************************************************************************
*
*	Function:	close_dev
*
*	Purpose:
*
*		Close the given device.
*
*	Arguments:
*
*		dev		The device to close.
*
*	Returned:
*
*		0		All went well.
*		< 0		There was a problem.
*
******************************************************************************/

int close_dev(dev_data_t* dev)
{
	gsc_drv_dma_t	dma;
	int				status;
	int				test;

	for (;;)	/* We'll use a loop for convenience. */
	{
		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We didn't get the semaphore. */
			status	= -ERESTARTSYS;
			break;
		}

		/* Close the device. */
		memset(&dma, 0, sizeof(dma));
		dma.index	= 0;
		dma_close_ioctl(dev, &dma);

		memset(&dma, 0, sizeof(dma));
		dma.index	= 1;
		dma_close_ioctl(dev, &dma);

		pid_remove(dev);
		status		= 0;
		MOD_DEC_USE_COUNT;

		if (dev->usage)
			dev->usage--;

		if (dev->usage == 0)
		{
			irq_release(dev);
			writel(0, dev->vaddr.plx_intcsr_32);
		}

		up(&dev->sem);
		status	= 0;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	close
*
*	Purpose:
*
*		Implement the release() procedure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*		filp	This the file structure.
*
*	Returned:
*
*		0		All went well.
*		< 0		There was a problem.
*
******************************************************************************/

int close(struct inode* inode, struct file* filp)
{
	dev_data_t*	dev;
	int			status;

	dev	= dev_locate(inode);

	if (dev == NULL)
	{
		/* The referenced device doesn't exist. */
		status	= -ENODEV;
	}
	else
	{
		/* Perform the work of closing the device. */
		status	= close_dev(dev);
	}

	return(status);
}



