/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* Any Device - Driver - Linux */
/* $Workfile: main.c $ */
/* $Revision: 4 $ */
/* $Date: 10/27/07 3:35p $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/driver/linux/main.c $
 *
 * 4     10/27/07 3:35p Dnewbold
 * Remove some dead and commented out code.
 *
 * 3     10/27/07 10:36a Dnewbold
 * Additional driver porting updates.
 *
 * 2     3/29/07 2:45p Dnewbold
 *
 * 1     5/10/06 1:01p Dnewbold
*/

#include <linux/delay.h>

#include "main.h"



/* variables	**************************************************************/

	dev_global_t	global;



/******************************************************************************
*
*	Function:	_dev_add
*
*	Purpose:
*
*		Add a device to our device list.
*
*	Arguments:
*
*		dev		The data for the device to add.
*
*	Returned:
*
*		>= 0	All went well. The index where the item was added.
*		-1		There was a problem.
*
******************************************************************************/

static int _dev_add(dev_data_t* dev)
{
	int	i;
	int	status;

	for (;;)	/* We'll use a loop for convenience. */
	{
		/* Find an available entry. */

		for (i = 0; i < ARRAY_ELEMENTS(global.dev_list); i++)
		{
			if (global.dev_list[i] == NULL)
				break;
		}

		if (i >= ARRAY_ELEMENTS(global.dev_list))
		{
			/* We don't have enough room to add the device. */
			status	= -1;
			printk(	"<1>%s: Too many devices were found (%d).\n"
					"%s: Rebuild the driver to increase support.\n",
					DRV_DEV_DEVICE_NAME_LOWER,
					(int) ARRAY_ELEMENTS(global.dev_list),
					DRV_DEV_DEVICE_NAME_LOWER);
			break;
		}

		global.dev_list[i]	= kmalloc(sizeof(dev_data_t), GFP_KERNEL);

		if (global.dev_list[i] == NULL)
		{
			/* We didn't get the needed memory. */
			status	= -1;
			printk(	"<1>%s: kmalloc() failure.\n",
					DRV_DEV_DEVICE_NAME_LOWER);
			break;
		}

		/* We got the needed memory. */
		memcpy(global.dev_list[i], dev, sizeof(dev_data_t));
		status	= i;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	_dev_data_t_init
*
*	Purpose:
*
*		Initialize the device data structure.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _dev_data_t_init(dev_data_t* dev)
{
	sema_init(&dev->sem, 1);
	sema_init(&dev->irq.sem, 1);
	spin_lock_init(&dev->irq.spinlock);

	dev->vaddr.plx_dmaarb_32	= reg_vaddr(dev, GSC_PLX_DMAARB);
	dev->vaddr.plx_dmacsr0_8	= reg_vaddr(dev, GSC_PLX_DMACSR0);
	dev->vaddr.plx_dmacsr1_8	= reg_vaddr(dev, GSC_PLX_DMACSR1);
	dev->vaddr.plx_dmathr_32	= reg_vaddr(dev, GSC_PLX_DMATHR);
	dev->vaddr.plx_intcsr_32	= reg_vaddr(dev, GSC_PLX_INTCSR);
	dev->vaddr.plx_l2pdbell_32	= reg_vaddr(dev, GSC_PLX_L2PDBELL);

	/* Initialize the DMA fields. */
	dma_channel_init(dev, &dev->dma.channel[0], 0);
	dma_channel_init(dev, &dev->dma.channel[1], 1);
}



/******************************************************************************
*
*	Function:	_dev_init
*
*	Purpose:
*
*		Initialize the device.
*
*	Arguments:
*
*		dev		The structure for the device to access.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _dev_init(dev_data_t* dev)
{
	writel(0, dev->vaddr.plx_intcsr_32);	// Disable all interrupts.
	dma_abort(&dev->dma.channel[0]);		// Abort DMA0.
	dma_abort(&dev->dma.channel[1]);		// Abort DMA1.
}



/******************************************************************************
*
*	Function:	cleanup_module
*
*	Purpose:
*
*		Clean things up when the kernel is about to unload the module.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void cleanup_module(void)
{
	int		i;

	for (i = 0; i < ARRAY_ELEMENTS(global.dev_list); i++)
	{
		if (global.dev_list[i])
		{
			global.dev_list[i]->usage	= 0;
			close_dev(global.dev_list[i]);
			dma_channel_reset(&global.dev_list[i]->dma.channel[0]);
			dma_channel_reset(&global.dev_list[i]->dma.channel[1]);
			bar_regions_release(&global.dev_list[i]->bars);
			kfree(global.dev_list[i]);
			global.dev_list[i]	= NULL;
		}
	}

	if (global.major_number >= 0)
	{
		unregister_chrdev(	global.major_number,
							DRV_DEV_DEVICE_NAME_LOWER);
		global.major_number	= -1;
	}

	proc_stop();

	if (global.driver_loaded)
	{
		global.driver_loaded	= 0;
		printk(	"%s driver version %s successfully removed.\n",
				DRV_DEV_DEVICE_NAME_UPPER,
				DRV_GSC_FILE_VERSION_STR);
	}

	global.dev_qty	= 0;
}



/******************************************************************************
*
*	Function:	dev_locate
*
*	Purpose:
*
*		Get a pointer to the device structure referenced by the given
*		i_node structure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*	Returned:
*
*		NULL	The respective device can't be accessed.
*		else	A pointer to the respective device.
*
******************************************************************************/

dev_data_t* dev_locate(struct inode* inode)
{
	dev_data_t*	dev;
	unsigned int	index	= MINOR(inode->i_rdev);

	if (index >= ARRAY_ELEMENTS(global.dev_list))
		dev	= NULL;
	else
		dev	= global.dev_list[index];

	return(dev);
}



/******************************************************************************
*
*	Function:	init_module
*
*	Purpose:
*
*		Initialize the driver upon loading.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

u32 _order_to_size(int order);
int _size_to_order(u32 size);

int init_module(void)
{
	dev_data_t		dev;
	int				i;
	struct pci_dev*	pci;
	int				status	= -ENODEV;

	memset(&global, 0, sizeof(global));
	sprintf(global.built, "%s, %s", __DATE__, __TIME__);
	global.major_number	= -1;

	/*
	*	Locate any and all applicable boards.
	*/

	PCI_DEVICE_LOOP(pci)
	{
		i	= dev_os_id_device(pci);

		if (i == 0)
			continue;

		i	= PCI_ENABLE_DEVICE(pci);

		if (i)
			continue;

		memset(&dev, 0, sizeof(dev));
		bar_read_all(pci, &dev.bars);
		i	= bar_regions_check(&dev.bars);

		if (i)
			continue;

		i	= bar_regions_request(&dev);

		if (i)
		{
			bar_regions_release(&dev.bars);
			continue;
		}

		dev.pci	= pci;

		/* Add the device to our list. */
		i	= _dev_add(&dev);

		if (i < 0)
		{
			bar_regions_release(&dev.bars);
		}
		else
		{
			/* Initialize the structure.  */
			_dev_data_t_init(global.dev_list[i]);
			_dev_init(global.dev_list[i]);
			global.dev_qty++;
		}
	}

	/*
	*	Get the major number dynamicly.
	*/

	if (global.dev_qty)
	{
		global.fops.open	= open;
		global.fops.release	= close;
		global.fops.ioctl	= ioctl;
		global.fops.mmap	= mmap;
		SET_MODULE_OWNER(&global.fops);
		mmap_setup();
		global.major_number	= register_chrdev(	0,
												DRV_DEV_DEVICE_NAME_LOWER,
												&global.fops);

		if (global.major_number < 0)
			cleanup_module();
	}

	/*
	*	Other startup work.
	*/

	i	= ioctl_init();

	if (i)
		cleanup_module();

	i	= proc_start();

	if (i)
		cleanup_module();

	/*
	*	Finish up.
	*/

	if (global.dev_qty)
	{
		status					= 0;
		global.driver_loaded	= 1;
		printk(	"%s driver version %s successfully inserted.\n",
				DRV_DEV_DEVICE_NAME_UPPER,
				DRV_GSC_FILE_VERSION_STR);
	}

	return(status);
}



