/* This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL). */
/* Any Device - Driver - Linux */
/* $Workfile: open.c $ */
/* $Revision: 2 $ */
/* $Date: 10/27/07 10:36a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/common/driver/linux/open.c $
 *
 * 2     10/27/07 10:36a Dnewbold
 * Additional driver porting updates.
 *
 * 1     5/10/06 1:02p Dnewbold
*/

#include "main.h"



/******************************************************************************
*
*	Function:	open
*
*	Purpose:
*
*		Implement the open() procedure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*		filp	This the file structure.
*
*	Returned:
*
*		0		All went well.
*		<0		An error code for the problem seen.
*
******************************************************************************/

int open(struct inode* inode, struct file* filp)
{
	#define	_RW_	(FMODE_READ | FMODE_WRITE)

	dev_data_t*	dev;
	int			status;
	int			test;

	for (;;)	/* We'll use a loop for convenience. */
	{
		if ((filp->f_mode & _RW_) != _RW_)
		{
			/* Read and write access are both required. */
			status	= -EINVAL;
			break;
		}

		dev	= dev_locate(inode);

		if (dev == NULL)
		{
			/* The referenced device doesn't exist. */
			status	= -ENODEV;
			break;
		}

		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We didn't get the semaphore. */
			status	= -ERESTARTSYS;
			break;
		}

		test	= pid_add(dev);

		if (test)
		{
			/* We couldn't add a new process. */
			status	= -EBUSY;
			up(&dev->sem);
			break;
		}

		/* Open the device. */
		if (dev->usage == 0)
		{
			status	= irq_request(dev);

			if (status)
			{
				/* We can't get the interrupt. */
				up(&dev->sem);
				break;
			}
		}

		MOD_INC_USE_COUNT;
		filp->private_data	= dev;
		dev->usage++;
		status		= 0;
		up(&dev->sem);
		break;
	}

	return(status);
}



