/* API - DMI32 - Any OS */
/* $Workfile: dev_access.h $ */
/* $Revision: 3 $ */
/* $Date: 3/01/07 8:21a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_access.h $
 *
 * 3     3/01/07 8:21a Dnewbold
 * Linted.
 *
 * 2     5/04/06 3:58p Dnewbold
 * Removed key validation code and increased modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#ifndef __DEV_ACCESS_H__
#define __DEV_ACCESS_H__

#include "gsc_irq.h"

#include "dev_io.h"		/*lint !e537 */
#include "gsc_sem.h"	/*lint !e537 */



/* typedefs	*****************************************************************/

#define BAR2_REMAP_ENABLE			0x00000001
#define BAR2_REMAP_DRAM_ADRS_MASK	0x7FFFFFFC
#define	BAR2_WINDOW_SIZE			4096UL		// Power of 2.
#define BAR2_REMAP_FW_REGS			(0x80000000 | BAR2_REMAP_ENABLE)



/* typedefs	*****************************************************************/

struct _dev_data_t
{
	gsc_device_t	device;
	gsc_sem_t		sem;
	gsc_bar_t		plx_regs;
	gsc_bar_t		gsc_regs;
	gsc_dma_t		dma;
	dev_io_t		tx;
	dev_io_t		rx;
	U32				strict_args;		// Enforce strict argument compliance.
	U32				strict_config;		// Enforce strict configuration compliance.
	U32				strict_dram;		// Restrict DRAM access per DRAM size.
	U32				dram_capacity_mb;	// DRAM capacity in Megabyte units.
	U32				dram_size_mb;		// DRAM size in Megabyte units.

	struct
	{
		gsc_sem_t	sem;		// Don't access without this lock.
		int			pci_bus;	// Is DRAM connected to the PCI bus?
		int			users;		// Count of those using current view.
	} dram;						// Access to DRAM is ...

	struct
	{
		gsc_irq_t		girq;		// GSC IRQ data
		gsc_event_t*	local;		// PLX: local interrupt event
		gsc_event_t*	locals[5];	// Local interrupt event handling.
	} irq;

	struct
	{
		U32			frr;		// Firmware Revision Register
		U32			fr;			// Features Register
	} cache;

	struct
	{
		U32			features_register;	// Features Register
		U32			irq_mbur;			// ICR: Multi-Buffer Under Run
		U32			lmb_register;		// Local Memory Bank Register
		U32			start_word_dir;		// BCR: Start Word Direction
	} features;							// Is the feature available?
};



/* prototypes	*************************************************************/

U32		dev_access_lock(void* handle, dev_data_t** dev);
U32		dev_access_dram(dev_data_t* dev, int acquire, int pci_bus);
void	dev_access_unlock(dev_data_t* dev);



#endif
