/* API - DMI32 - Any OS */
/* $Workfile: dev_config.c $ */
/* $Revision: 4 $ */
/* $Date: 10/27/07 10:43a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_config.c $
 *
 * 4     10/27/07 10:43a Dnewbold
 * Ported to Linux.
 *
 * 3     3/01/07 8:21a Dnewbold
 * Linted.
 *
 * 2     5/05/06 10:39a Dnewbold
 * Worked to increase modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include "dev_config.h"

#include "gsc_version.h"

#include "dev_access.h"
#include "dev_irq.h"
#include "dev_misc.h"
#include "dev_user_io.h"
#include "dev_xfer.h"



/* typedefs	*****************************************************************/

typedef struct
{
	U32	parm;
	U32	(*func)(dev_data_t* dev, U32 which, UL set, UL* get);
} parm_item_t;

typedef struct
{
	U32					group;
	const parm_item_t*	list;
	U32					size;
} parm_group_t;

typedef struct
{
	const parm_group_t*	list;
	U32					size;
} parm_set_t;



/* variables	*************************************************************/

static const parm_item_t	_io_list[]	=
{
	/* parm							func				*/
	{ DMI32_IO_ABORT,				io_abort			},
	{ DMI32_IO_ABORTED,				io_aborted			},
	{ DMI32_IO_BUFFER_POINTER,		io_buffer_pointer	},
	{ DMI32_IO_BUFFER_SIZE,			io_buffer_size		},
	{ DMI32_IO_CALLBACK_ARG,		io_callback_arg		},
	{ DMI32_IO_CALLBACK_FUNC,		io_callback_func	},
	{ DMI32_IO_MODE,				io_mode				},
	{ DMI32_IO_OFFSET,				io_offset			},
	{ DMI32_IO_OVERLAP_ENABLE,		io_overlap_enable	},
	{ DMI32_IO_STATUS,				io_status			},
	{ DMI32_IO_TIMEOUT,				io_timeout			}
};

static const parm_item_t	_irq_list[]	=
{
	/* parm						func					*/
	{ DMI32_IRQ_CALLBACK_ARG,	irq_callback_arg		},
	{ DMI32_IRQ_CALLBACK_FUNC,	irq_callback_func		},
	{ DMI32_IRQ_ENABLE,			irq_enable				},
	{ DMI32_IRQ_STATE,			irq_state				}
};

static const parm_item_t	_misc_list[]	=
{
	/* parm								func					*/
	{ DMI32_MISC_BOARD_JUMPERS,			misc_board_jumpers		},
	{ DMI32_MISC_DRAM_CAPACITY_MB,		misc_dram_capacity_mb	},
	{ DMI32_MISC_DRAM_SIZE_MB,			misc_dram_size_mb		},
	{ DMI32_MISC_FEATURES,				misc_features			},
	{ DMI32_MISC_MAP_GSC_REGS,			misc_map_gsc_regs		},
	{ DMI32_MISC_MAP_PLX_REGS,			misc_map_plx_regs		},
	{ DMI32_MISC_STRICT_ARGUMENTS,		misc_strict_arguments	},
	{ DMI32_MISC_STRICT_CONFIG,			misc_strict_config		},
	{ DMI32_MISC_STRICT_DRAM_LIMITS,	misc_strict_dram_limits	}
};

static const parm_item_t	_user_io_list[]	=
{
	/* parm								func				*/
	{ DMI32_USER_IO_DIRECTION,			user_io_direction	},
	{ DMI32_USER_IO_INPUT,				user_io_input		},
	{ DMI32_USER_IO_OUTPUT,				user_io_output		}
};

static const parm_item_t	_xfer_list[]	=
{
	/* parm								func						*/
	{ DMI32_XFER_CLKIN_TEST,			xfer_clkin_test				},
	{ DMI32_XFER_DIRECTION,				xfer_direction				},
	{ DMI32_XFER_ENABLE,				xfer_enable					},
	{ DMI32_XFER_INTERNAL_CLOCK_SPEED,	xfer_internal_clock_speed	},
	{ DMI32_XFER_MODE,					xfer_mode					},
	{ DMI32_XFER_OFFSET,				xfer_offset					},
	{ DMI32_XFER_SIZE,					xfer_size					},
	{ DMI32_XFER_START_WORD,			xfer_start_word				},
	{ DMI32_XFER_START_WORD_DIR,		xfer_start_word_dir			},
	{ DMI32_XFER_STATE,					xfer_state					},
	{ DMI32_XFER_STATE_RX,				xfer_state_rx				},
	{ DMI32_XFER_STATE_TX,				xfer_state_tx				},
	{ DMI32_XFER_TEST_MODE,				xfer_test_mode				}
};

static const parm_group_t	_groups[]	=
{
	/* group						list			size							*/
	{ DMI32_CONFIG_GROUP_IO,		_io_list,		ARRAY_ELEMENTS(_io_list)		},
	{ DMI32_CONFIG_GROUP_IRQ,		_irq_list,		ARRAY_ELEMENTS(_irq_list)		},
	{ DMI32_CONFIG_GROUP_MISC,		_misc_list,		ARRAY_ELEMENTS(_misc_list)		},
	{ DMI32_CONFIG_GROUP_USER_IO,	_user_io_list,	ARRAY_ELEMENTS(_user_io_list)	},
	{ DMI32_CONFIG_GROUP_XFER,		_xfer_list,		ARRAY_ELEMENTS(_xfer_list)		}
};

static const parm_set_t	_parm_set	=
{
	/* list	*/	_groups,
	/* size	*/	ARRAY_ELEMENTS(_groups)
};



/*****************************************************************************
*
*	Function:	config_init
*
*	Purpose:
*
*		Verify that out data set is valid.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 config_init(void)
{
	int	i;
	int	j;
	U32	parm;
	U32	status	= (U32) GSC_SUCCESS;

	for (i = 0; i < (int) _parm_set.size; i++)
	{
		for (j = 0; j < (int) _parm_set.list[i].size; j++)
		{
			parm	= DMI32_CONFIG_ENCODE(_parm_set.list[i].group, j);

			if (parm != _parm_set.list[i].list[j].parm)
			{
				status			= (U32) GSC_INIT_FAILURE;
				api_status_arg	= DMI32_CONFIG_ENCODE(i, j);
			}
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	config_which_validate
*
*	Purpose:
*
*		Verify the given configuration "which" value against the given mask of
*		permissible bits.
*
*	Arguments:
*
*		dev			The device to act on.
*
*		which		The value to validate.
*
*		mask		The set of permissible bits.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 config_which_validate(dev_data_t* dev, U32 which, U32 mask)
{
	U32	status;

	if ((which & ~mask) == 0)
	{
		status	= (U32) GSC_SUCCESS;
	}
	else if (dev->strict_args)
	{
		status	= (U32) GSC_INVALID_DATA;
	}
	else
	{
		status	= (U32) GSC_SUCCESS;	// Ignore the extra bits.
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_config
*
*	Purpose:
*
*		Process a parameter configuration request per the given structure.
*
*	Arguments:
*
*		handle		The handle of the channel to close.
*
*		parm		The parameter to be acted upon.
*
*		which		A field of bits specifying the parameter specific
*					subcomponents to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_config(
	void*			handle,
	U32				parm,
	U32				which,
	unsigned long	set,
	unsigned long*	get)
{
	dev_data_t*			dev;
	const parm_group_t*	g_ptr	= NULL;
	U32					group;
	const parm_item_t*	i_ptr	= NULL;
	U32					index;
	U32					status	= (U32) GSC_SUCCESS;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		group	= DMI32_CONFIG_GROUP_DECODE(parm);
		index	= DMI32_CONFIG_INDEX_DECODE(parm);

		for (;;)	/* A convenience loop. */
		{
			if (group >= _parm_set.size)
			{
				status	= (U32) GSC_INVALID_DATA;
				break;
			}

			g_ptr	= _parm_set.list + group;

			if (index >= g_ptr->size)
			{
				status	= (U32) GSC_INVALID_DATA;
				break;
			}

			i_ptr	= g_ptr->list + index;

			if (i_ptr->parm != parm)
			{
				status	= (U32) GSC_INVALID_DATA;
				break;
			}

			if (i_ptr->func == NULL)
			{
				status	= (U32) GSC_UNSUPPORTED_FUNCTION;
				break;
			}

			status	= (i_ptr->func)(dev, which, set, get);
			break;
		}

		dev_access_unlock(dev);
	}

	return(status);
}


