/* API - DMI32 - Any OS */
/* $Workfile: dev_version.c $ */
/* $Revision: 3 $ */
/* $Date: 3/01/07 8:33a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dev_version.c $
 *
 * 3     3/01/07 8:33a Dnewbold
 * Linted.
 *
 * 2     5/05/06 10:39a Dnewbold
 * Worked to increase modularity.
 *
 * 1     8/22/05 3:40p Dnewbold
*/

#include "dev_version.h"

#include "gsc_version.h"

#include "dev_access.h"



/*****************************************************************************
*
*	Function:	dev_version_verify
*
*	Purpose:
*
*		Verify the version data for various components.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 dev_version_verify(void)
{
	U32	status;

	status	= gsc_version_verify();
	return(status);
}



/*****************************************************************************
*
*	Function:	dmi32_version_get
*
*	Purpose:
*
*		Close an open communications channel.
*
*	Arguments:
*
*		handle		The handle of the device to be used to access the driver.
*
*		id			The item whose version number is desired.
*
*		version		The version number string is put here.
*
*		size		This is the size of the above buffer.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT dmi32_version_get(void* handle, U8 id, char* version, size_t size)
{
	dev_data_t*	dev;
	U32			status;

	for (;;)	/* A convenience loop. */
	{
		if (version == NULL)
		{
			status	= (U32) GSC_NULL_PARAM;
			break;
		}

		if (size <= 1)
		{
			status	= (U32) GSC_INVALID_SIZE;
			break;
		}

		if (handle == NULL)
		{
			status	= gsc_version_get(NULL, id, version, size);
			break;
		}

		status	= dev_access_lock(handle, &dev);

		if (status == (U32) GSC_SUCCESS)
		{
			status	= gsc_version_get(&dev->device, id, version, size);
			dev_access_unlock(dev);
		}

		break;
	}

	return(status);
}


