/* $Revision: 3 $ */
/* $Date: 10/27/07 1:51p $ */
/*
*	This header defines the application interface to the DMI32 library.
*/
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/api/dmi32_api.h $
 *
 * 3     10/27/07 1:51p Dnewbold
 * Updated the API version number due to Linux porting.
 *
 * 2     10/27/07 10:49a Dnewbold
 * Ported to Linux.
 *
 * 1     8/22/05 3:40p Dnewbold
 *
 * 3     7/06/04 1:26p Dnewbold
 * Updated per changes in common code.
 *
 * 2     8/25/03 1:30p Dnewbold
 * Updated the version number.
 *
 * 1     6/17/03 8:30a Dnewbold
*/

#ifndef __DMI32_API_H__
#define __DMI32_API_H__

#include "gsc_common.h"
#include "gsc_pci9080.h"

#ifdef __cplusplus
extern "C" {
#endif



/* #defines	*****************************************************************/

#define	DMI32_API_VERSION		4
/* 4	Ported to Linux. */
/* 3	Ported to the SDK. */
/* 2	Each detected local IRQ is disabled when serviced. */
/* 1	Initial release. */

#define	DMI32_VENDOR_ID			0x10B5	/* PLX Technologies */
#define	DMI32_SUBVENDOR_ID		0x10B5	/* PLX assigned subsystem device id */

#define	DMI32_DEVICE_ID			0x9080	/* PCI9080 PCI interface chip */
#define	DMI32_SUBSYSTEM_ID		0x2707	/* DMI32 (32-bit PCI interface) */



#define	DMI32_REG_ENCODE(s,o)	GSC_REG_ENCODE(GSC_REG_GSC,(s),(o))

/* GSC specific registers. */
#define	DMI32_FRR				DMI32_REG_ENCODE(4, 0x00) // Firmware Revision
#define	DMI32_BCR				DMI32_REG_ENCODE(4, 0x04) // Board Control
#define	DMI32_BSR				DMI32_REG_ENCODE(4, 0x08) // Board Status
#define	DMI32_TBBAR				DMI32_REG_ENCODE(4, 0x0C) // Trans Buf Base Address
#define	DMI32_TBSR				DMI32_REG_ENCODE(4, 0x10) // Trans Buf Size
#define	DMI32_ICR				DMI32_REG_ENCODE(4, 0x14) // Interrupt Control
#define	DMI32_ISR				DMI32_REG_ENCODE(4, 0x18) // Interrupt Status
#define	DMI32_TSWR				DMI32_REG_ENCODE(4, 0x1C) // Trans Start Word
#define	DMI32_LMBR				DMI32_REG_ENCODE(4, 0x20) // Local Mem Bank Register
#define	DMI32_FR				DMI32_REG_ENCODE(4, 0x3C) // Features



/* Register field masks. */
/* GSC: Firmware Revision Register */
#define	DMI32_FRR_REVISION_MASK				0x000000FF
#define	DMI32_FRR_DEVICE_ID_MASK			0x0000FF00
#define	DMI32_FRR_BOARD_ID_MASK				0x00FF0000
#define	DMI32_FRR_MISC_MASK					0xFF000000

#define	DMI32_FRR_FR_DECODE(r)				GSC_FIELD_DECODE((r),7,0)
#define	DMI32_FRR_DID_DECODE(r)				GSC_FIELD_DECODE((r),15,8)
#define	DMI32_FRR_BID_DECODE(r)				GSC_FIELD_DECODE((r),23,16)
#define	DMI32_FRR_MISC_DECODE(r)			GSC_FIELD_DECODE((r),31,24)

#define	DMI32_FRR_MISC_FR					0x01
#define	DMI32_FRR_FR						0x01000000L

/* GSC: Board Control Register */
#define	DMI32_BCR_BOARD_RESET		0x00000001	// Needs 1us to complete.
#define	DMI32_BCR_CLKIN_TEST		0x00000002	// Needs 5us to complete.
#define	DMI32_BCR_XFER_DIR			0x00000004	// 0 = Rx, 1 = Tx
#define	DMI32_BCR_USER_IO_A_DIR		0x00000008	// 0 = input, 1 = output
#define	DMI32_BCR_USER_IO_B_DIR		0x00000010	// 0 = input, 1 = output
#define	DMI32_BCR_XFER_MODE			0x00000020	// 0 = single, 1 = multi
#define	DMI32_BCR_TEST_MODE			0x00000040	// 0 = normal, 1 = test
#define	DMI32_BCR_XFER_ENABLE		0x00000080	// 0 = disable, 1 = enable
#define	DMI32_BCR_USER_IO_A_OUTPUT	0x00007F00	// 7-bit mask
#define	DMI32_BCR_USER_IO_B_OUTPUT	0x003F8000	// 7-bit mask
#define	DMI32_BCR_INT_CLOCK_SPEED	0x00400000	// 0 = /1, 1 = /2
#define	DMI32_BCR_START_WORD_DIR	0x00800000	// 0 = Tx, 1 = Rx

/* GSC: Board Status Register */
#define	DMI32_BSR_CLKIN_STATUS		0x00000003	// 0 = not detected, 1 = detected
#define	DMI32_BSR_CLKIN_STATUS_NONE	0x00000002	// A clock was not detected.
#define	DMI32_BSR_CLKIN_STATUS_OK	0x00000003	// A clock was detected.
#define	DMI32_BSR_JP0				0x00000010	// 0 = present, 1 = removed
#define	DMI32_BSR_JP1				0x00000020	// 0 = present, 1 = removed
#define	DMI32_BSR_TX_IN_PROGRESS	0x00000040	// = 0 inactive, 1 = active
#define	DMI32_BSR_RX_IN_PROGRESS	0x00000080	// = 0 inactive, 1 = active
#define	DMI32_BSR_PORT_A_INPUT		0x00007F00	// 7-bit mask
#define	DMI32_BSR_PORT_B_INPUT		0x003F8000	// 7-bit mask
#define	DMI32_BSR_CLKIN_COUNT		0xFFC00000	// 10-bit mask
#define	DMI32_BSR_CLKIN_SHIFT		22

/* GSC: Features Register */
#define	DMI32_FR_4GB				0x00000001	// A bunch of changes with this.

/* GSC: Interrupt Control Register */
#define	DMI32_ICR_DEFAULT			0x00000000	// Written during initialization
#define	DMI32_ICR_BS				0x00000001	// Buffer Start
#define	DMI32_ICR_BE				0x00000002	// Buffer End
#define	DMI32_ICR_UIOA				0x00000004	// User I/O A Input
#define	DMI32_ICR_UIOB				0x00000008	// User I/O B Input
#define	DMI32_ICR_MBUR				0x00000010	// Multi-Buffer Under Run

/* GSC: Interrupt Status Register */
#define	DMI32_ISR_BS				0x00000001	// Buffer Start
#define	DMI32_ISR_BE				0x00000002	// Buffer End
#define	DMI32_ISR_UIOA				0x00000004	// User I/O A Input
#define	DMI32_ISR_UIOB				0x00000008	// User I/O B Input
#define	DMI32_ISR_MBUR				0x00000010	// Multi-Buffer Under Run



// Configuration parameters: Common Components
#define	DMI32_CONFIG_GET(h,p,w,g)			dmi32_config((h),(p),(w),GSC_NO_CHANGE,(g))
#define	DMI32_CONFIG_SET(h,p,w,s)			dmi32_config((h),(p),(w),(s),NULL)
#define	DMI32_CONFIG_SET_GET(h,p,w,s,g)		dmi32_config((h),(p),(w),(s),(g))

#define	DMI32_CONFIG_ENCODE(g,i)			(GSC_FIELD_ENCODE((g),31,16) | \
											GSC_FIELD_ENCODE((i),15,0))
#define	DMI32_CONFIG_GROUP_DECODE(v)		GSC_FIELD_DECODE((v),31,16)
#define	DMI32_CONFIG_INDEX_DECODE(v)		GSC_FIELD_DECODE((v),15,0)

#define	DMI32_CONFIG_GROUP_IO				0
#define	DMI32_CONFIG_GROUP_IRQ				1
#define	DMI32_CONFIG_GROUP_MISC				2
#define	DMI32_CONFIG_GROUP_USER_IO			3
#define	DMI32_CONFIG_GROUP_XFER				4



// Configuration parameters: WHICH values
// User I/O A/B Bit Selection (may be OR'd for SET or GET operations)
#define	DMI32_WHICH_USER_IO_A		0x01
#define	DMI32_WHICH_USER_IO_B		0x02
#define	DMI32_WHICH_USER_IO_A_B		0x03
#define	DMI32_WHICH_USER_IO_CURRENT	0x04

// Tx/Rx Bit Selection (may be OR'd for SET or GET operations)
#define	DMI32_WHICH_RX				0x01
#define	DMI32_WHICH_TX				0x02
#define	DMI32_WHICH_TX_RX			0x03

// IRQ Selection (these are index values, not image bits)
#define	DMI32_WHICH_IRQ_BS			0x0001	// Buffer Start
#define	DMI32_WHICH_IRQ_BE			0x0002	// Buffer End
#define	DMI32_WHICH_IRQ_UIOA		0x0004	// User I/O A
#define	DMI32_WHICH_IRQ_UIOB		0x0008	// User I/O B
#define	DMI32_WHICH_IRQ_MBUR		0x0010	// Multi-Buffer Under Run

#define	DMI32_WHICH_IRQ_ALL			0x001F



// Configuration parameters: I/O Parameters
#define	DMI32_IO_ENCODE(i)							DMI32_CONFIG_ENCODE(DMI32_CONFIG_GROUP_IO, (i))

#define	DMI32_IO_ABORT								DMI32_IO_ENCODE( 0)	// Tx, Rx
#define	DMI32_IO_ABORTED							DMI32_IO_ENCODE( 1)	// Tx, Rx, GET only
#define	DMI32_IO_BUFFER_POINTER						DMI32_IO_ENCODE( 2)	// Tx, Rx, GET only
#define	DMI32_IO_BUFFER_SIZE						DMI32_IO_ENCODE( 3)	// Tx, Rx
#define	DMI32_IO_CALLBACK_ARG						DMI32_IO_ENCODE( 4)	// Tx, Rx
#define	DMI32_IO_CALLBACK_FUNC						DMI32_IO_ENCODE( 5)	// Tx, Rx
#define	DMI32_IO_MODE								DMI32_IO_ENCODE( 6)	// Tx, Rx
#define	DMI32_IO_OFFSET								DMI32_IO_ENCODE( 7)	// Tx, Rx
#define	DMI32_IO_OVERLAP_ENABLE						DMI32_IO_ENCODE( 8)	// Tx, Rx
#define	DMI32_IO_STATUS								DMI32_IO_ENCODE( 9)	// Tx, Rx, GET only
#define	DMI32_IO_TIMEOUT							DMI32_IO_ENCODE(10)	// Tx, Rx

#define	DMI32_IO_ABORT_NO							0
#define	DMI32_IO_ABORT_YES							1
#define	DMI32_IO_ABORT_DEFAULT						DMI32_IO_ABORT_NO
#define	DMI32_IO_ABORT__GET(h,w,g)					DMI32_CONFIG_GET((h),DMI32_IO_ABORT,(w),(g))
#define	DMI32_IO_ABORT__SET(h,w,s)					DMI32_CONFIG_SET((h),DMI32_IO_ABORT,(w),(s))
#define	DMI32_IO_ABORT__YES(h,w)					DMI32_IO_ABORT__SET((h),(w),DMI32_IO_ABORT_YES)
#define	DMI32_IO_ABORT__RX_GET(h,g)					DMI32_IO_ABORT__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_ABORT__RX_SET(h,s)					DMI32_IO_ABORT__SET((h),DMI32_WHICH_RX,(s))
#define	DMI32_IO_ABORT__RX_YES(h)					DMI32_IO_ABORT__RX_SET((h),DMI32_IO_ABORT_YES)
#define	DMI32_IO_ABORT__TX_GET(h,g)					DMI32_IO_ABORT__GET((h),DMI32_WHICH_TX,(g))
#define	DMI32_IO_ABORT__TX_SET(h,s)					DMI32_IO_ABORT__SET((h),DMI32_WHICH_TX,(s))
#define	DMI32_IO_ABORT__TX_YES(h)					DMI32_IO_ABORT__TX_SET((h),DMI32_IO_ABORT_YES)

#define	DMI32_IO_ABORTED_NO							0
#define	DMI32_IO_ABORTED_YES						1
#define	DMI32_IO_ABORTED__GET(h,w,g)				DMI32_CONFIG_GET((h),DMI32_IO_ABORTED,(w),(g))
#define	DMI32_IO_ABORTED__RX_GET(h,g)				DMI32_IO_ABORTED__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_ABORTED__TX_GET(h,g)				DMI32_IO_ABORTED__GET((h),DMI32_WHICH_TX,(g))

#define	DMI32_IO_BUFFER_POINTER__GET(h,w,g)			DMI32_CONFIG_GET((h),DMI32_IO_BUFFER_POINTER,(w),(g))
#define	DMI32_IO_BUFFER_POINTER__RX_GET(h,g)		DMI32_IO_BUFFER_POINTER__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_BUFFER_POINTER__TX_GET(h,g)		DMI32_IO_BUFFER_POINTER__GET((h),DMI32_WHICH_TX,(g))

#define	DMI32_IO_BUFFER_SIZE_DEFAULT				0
#define	DMI32_IO_BUFFER_SIZE_MASK					0x0FFFFFFC
#define	DMI32_IO_BUFFER_SIZE__GET(h,w,g)			DMI32_CONFIG_GET((h),DMI32_IO_BUFFER_SIZE,(w),(g))
#define	DMI32_IO_BUFFER_SIZE__SET(h,w,s,g)			DMI32_CONFIG_SET_GET((h),DMI32_IO_BUFFER_SIZE,(w),(s),(g))
#define	DMI32_IO_BUFFER_SIZE__RESET(h,w)			DMI32_IO_BUFFER_SIZE__SET((h),(w),0,NULL)
#define	DMI32_IO_BUFFER_SIZE__RX_GET(h,g)			DMI32_IO_BUFFER_SIZE__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_BUFFER_SIZE__RX_SET(h,s,g)			DMI32_CONFIG_SET_GET((h),DMI32_IO_BUFFER_SIZE,DMI32_WHICH_RX,(s),(g))
#define	DMI32_IO_BUFFER_SIZE__RX_RESET(h)			DMI32_IO_BUFFER_SIZE__RX_SET(h,DMI32_IO_BUFFER_SIZE_DEFAULT,NULL)
#define	DMI32_IO_BUFFER_SIZE__TX_GET(h,g)			DMI32_IO_BUFFER_SIZE__GET((h),DMI32_WHICH_TX,(g))
#define	DMI32_IO_BUFFER_SIZE__TX_SET(h,s,g)			DMI32_CONFIG_SET_GET((h),DMI32_IO_BUFFER_SIZE,DMI32_WHICH_TX,(s),(g))
#define	DMI32_IO_BUFFER_SIZE__TX_RESET(h)			DMI32_IO_BUFFER_SIZE__TX_SET(h,DMI32_IO_BUFFER_SIZE_DEFAULT,NULL)

#define	DMI32_IO_CALLBACK_ARG_DEFAULT				0
#define	DMI32_IO_CALLBACK_ARG__GET(h,w,g)			DMI32_CONFIG_GET((h),DMI32_IO_CALLBACK_ARG,(w),(g))
#define	DMI32_IO_CALLBACK_ARG__SET(h,w,s)			DMI32_CONFIG_SET((h),DMI32_IO_CALLBACK_ARG,(w),(s))
#define	DMI32_IO_CALLBACK_ARG__RESET(h,w)			DMI32_CONFIG_SET((h),DMI32_IO_CALLBACK_ARG,(w),DMI32_IO_CALLBACK_ARG_DEFAULT)
#define	DMI32_IO_CALLBACK_ARG__RX_GET(h,g)			DMI32_IO_CALLBACK_ARG__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_CALLBACK_ARG__RX_SET(h,s)			DMI32_IO_CALLBACK_ARG__SET((h),DMI32_WHICH_RX,(s))
#define	DMI32_IO_CALLBACK_ARG__RX_RESET(h)			DMI32_IO_CALLBACK_ARG__SET((h),DMI32_WHICH_RX,DMI32_IO_CALLBACK_ARG_DEFAULT)
#define	DMI32_IO_CALLBACK_ARG__TX_GET(h,g)			DMI32_IO_CALLBACK_ARG__GET((h),DMI32_WHICH_TX,(g))
#define	DMI32_IO_CALLBACK_ARG__TX_SET(h,s)			DMI32_IO_CALLBACK_ARG__SET((h),DMI32_WHICH_TX,(s))
#define	DMI32_IO_CALLBACK_ARG__TX_RESET(h)			DMI32_IO_CALLBACK_ARG__SET((h),DMI32_WHICH_TX,DMI32_IO_CALLBACK_ARG_DEFAULT)

#define	DMI32_IO_CALLBACK_FUNC_DEFAULT				0
#define	DMI32_IO_CALLBACK_FUNC__GET(h,w,g)			DMI32_CONFIG_GET((h),DMI32_IO_CALLBACK_FUNC,(w),(g))
#define	DMI32_IO_CALLBACK_FUNC__SET(h,w,s)			DMI32_CONFIG_SET((h),DMI32_IO_CALLBACK_FUNC,(w),(s))
#define	DMI32_IO_CALLBACK_FUNC__RESET(h,w)			DMI32_CONFIG_SET((h),DMI32_IO_CALLBACK_FUNC,(w),DMI32_IO_CALLBACK_FUNC_DEFAULT)
#define	DMI32_IO_CALLBACK_FUNC__RX_GET(h,g)			DMI32_IO_CALLBACK_FUNC__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_CALLBACK_FUNC__RX_SET(h,s)			DMI32_IO_CALLBACK_FUNC__SET((h),DMI32_WHICH_RX,(s))
#define	DMI32_IO_CALLBACK_FUNC__RX_RESET(h)			DMI32_IO_CALLBACK_FUNC__SET((h),DMI32_WHICH_RX,DMI32_IO_CALLBACK_FUNC_DEFAULT)
#define	DMI32_IO_CALLBACK_FUNC__TX_GET(h,g)			DMI32_IO_CALLBACK_FUNC__GET((h),DMI32_WHICH_TX,(g))
#define	DMI32_IO_CALLBACK_FUNC__TX_SET(h,s)			DMI32_IO_CALLBACK_FUNC__SET((h),DMI32_WHICH_TX,(s))
#define	DMI32_IO_CALLBACK_FUNC__TX_RESET(h)			DMI32_IO_CALLBACK_FUNC__SET((h),DMI32_WHICH_TX,DMI32_IO_CALLBACK_FUNC_DEFAULT)

#define	DMI32_IO_MODE_PIO							0
#define	DMI32_IO_MODE_DMA							1
#define	DMI32_IO_MODE_DEFAULT						DMI32_IO_MODE_DMA
#define	DMI32_IO_MODE__GET(h,w,g)					DMI32_CONFIG_GET((h),DMI32_IO_MODE,(w),(g))
#define	DMI32_IO_MODE__SET(h,w,s)					DMI32_CONFIG_SET((h),DMI32_IO_MODE,(w),(s))
#define	DMI32_IO_MODE__RESET(h,w)					DMI32_CONFIG_SET((h),DMI32_IO_MODE,(w),DMI32_IO_MODE_DEFAULT)
#define	DMI32_IO_MODE__PIO(h,w)						DMI32_IO_MODE__SET((h),(w),DMI32_IO_MODE_PIO)
#define	DMI32_IO_MODE__DMA(h,w)						DMI32_IO_MODE__SET((h),(w),DMI32_IO_MODE_DMA)
#define	DMI32_IO_MODE__RX_GET(h,g)					DMI32_IO_MODE__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_MODE__RX_SET(h,s)					DMI32_IO_MODE__SET((h),DMI32_WHICH_RX,(s))
#define	DMI32_IO_MODE__RX_RESET(h)					DMI32_IO_MODE__SET((h),DMI32_WHICH_RX,DMI32_IO_MODE_DEFAULT)
#define	DMI32_IO_MODE__RX_PIO(h)					DMI32_IO_MODE__RX_SET((h),DMI32_IO_MODE_PIO)
#define	DMI32_IO_MODE__RX_DMA(h)					DMI32_IO_MODE__RX_SET((h),DMI32_IO_MODE_DMA)
#define	DMI32_IO_MODE__TX_GET(h,g)					DMI32_IO_MODE__GET((h),DMI32_WHICH_TX,(g))
#define	DMI32_IO_MODE__TX_SET(h,s)					DMI32_IO_MODE__SET((h),DMI32_WHICH_TX,(s))
#define	DMI32_IO_MODE__TX_RESET(h)					DMI32_IO_MODE__SET((h),DMI32_WHICH_TX,DMI32_IO_MODE_DEFAULT)
#define	DMI32_IO_MODE__TX_PIO(h)					DMI32_IO_MODE__TX_SET((h),DMI32_IO_MODE_PIO)
#define	DMI32_IO_MODE__TX_DMA(h)					DMI32_IO_MODE__TX_SET((h),DMI32_IO_MODE_DMA)

#define	DMI32_IO_OFFSET_MIN							DMI32_XFER_OFFSET_MIN
#define	DMI32_IO_OFFSET_MAX							DMI32_XFER_OFFSET_MAX
#define	DMI32_IO_OFFSET_MASK						DMI32_XFER_OFFSET_MASK
#define	DMI32_IO_OFFSET_DEFAULT						DMI32_IO_OFFSET_MIN
#define	DMI32_IO_OFFSET__GET(h,w,g)					DMI32_CONFIG_GET((h),DMI32_IO_OFFSET,(w),(g))
#define	DMI32_IO_OFFSET__SET(h,w,s)					DMI32_CONFIG_SET((h),DMI32_IO_OFFSET,(w),(s))
#define	DMI32_IO_OFFSET__RESET(h,w)					DMI32_CONFIG_SET((h),DMI32_IO_OFFSET,(w),DMI32_IO_OFFSET_DEFAULT)
#define	DMI32_IO_OFFSET__RX_GET(h,g)				DMI32_IO_OFFSET__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_OFFSET__RX_SET(h,s)				DMI32_IO_OFFSET__SET((h),DMI32_WHICH_RX,(s))
#define	DMI32_IO_OFFSET__RX_RESET(h)				DMI32_IO_OFFSET__SET((h),DMI32_WHICH_RX,DMI32_IO_OFFSET_DEFAULT)
#define	DMI32_IO_OFFSET__TX_GET(h,g)				DMI32_IO_OFFSET__GET((h),DMI32_WHICH_TX,(g))
#define	DMI32_IO_OFFSET__TX_SET(h,s)				DMI32_IO_OFFSET__SET((h),DMI32_WHICH_TX,(s))
#define	DMI32_IO_OFFSET__TX_RESET(h)				DMI32_IO_OFFSET__SET((h),DMI32_WHICH_TX,DMI32_IO_OFFSET_DEFAULT)

#define	DMI32_IO_OVERLAP_ENABLE_NO					0
#define	DMI32_IO_OVERLAP_ENABLE_YES					1
#define	DMI32_IO_OVERLAP_ENABLE_DEFAULT				DMI32_IO_OVERLAP_ENABLE_NO
#define	DMI32_IO_OVERLAP_ENABLE__GET(h,w,g)			DMI32_CONFIG_GET((h),DMI32_IO_OVERLAP_ENABLE,(w),(g))
#define	DMI32_IO_OVERLAP_ENABLE__SET(h,w,s)			DMI32_CONFIG_SET((h),DMI32_IO_OVERLAP_ENABLE,(w),(s))
#define	DMI32_IO_OVERLAP_ENABLE__RESET(h,w)			DMI32_CONFIG_SET((h),DMI32_IO_OVERLAP_ENABLE,(w),DMI32_IO_OVERLAP_ENABLE_DEFAULT)
#define	DMI32_IO_OVERLAP_ENABLE__NO(h,w)			DMI32_IO_OVERLAP_ENABLE__SET((h),(w),DMI32_IO_OVERLAP_ENABLE_NO)
#define	DMI32_IO_OVERLAP_ENABLE__YES(h,w)			DMI32_IO_OVERLAP_ENABLE__SET((h),(w),DMI32_IO_OVERLAP_ENABLE_YES)
#define	DMI32_IO_OVERLAP_ENABLE__RX_GET(h,g)		DMI32_IO_OVERLAP_ENABLE__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_OVERLAP_ENABLE__RX_SET(h,s)		DMI32_IO_OVERLAP_ENABLE__SET((h),DMI32_WHICH_RX,(s))
#define	DMI32_IO_OVERLAP_ENABLE__RX_RESET(h)		DMI32_IO_OVERLAP_ENABLE__SET((h),DMI32_WHICH_RX,DMI32_IO_OVERLAP_ENABLE_DEFAULT)
#define	DMI32_IO_OVERLAP_ENABLE__RX_NO(h)			DMI32_IO_OVERLAP_ENABLE__RX_SET((h),DMI32_IO_OVERLAP_ENABLE_NO)
#define	DMI32_IO_OVERLAP_ENABLE__RX_YES(h)			DMI32_IO_OVERLAP_ENABLE__RX_SET((h),DMI32_IO_OVERLAP_ENABLE_YES)
#define	DMI32_IO_OVERLAP_ENABLE__TX_GET(h,g)		DMI32_IO_OVERLAP_ENABLE__GET((h),DMI32_WHICH_TX,(g))
#define	DMI32_IO_OVERLAP_ENABLE__TX_SET(h,s)		DMI32_IO_OVERLAP_ENABLE__SET((h),DMI32_WHICH_TX,(s))
#define	DMI32_IO_OVERLAP_ENABLE__TX_RESET(h)		DMI32_IO_OVERLAP_ENABLE__SET((h),DMI32_WHICH_TX,DMI32_IO_OVERLAP_ENABLE_DEFAULT)
#define	DMI32_IO_OVERLAP_ENABLE__TX_NO(h)			DMI32_IO_OVERLAP_ENABLE__TX_SET((h),DMI32_IO_OVERLAP_ENABLE_NO)
#define	DMI32_IO_OVERLAP_ENABLE__TX_YES(h)			DMI32_IO_OVERLAP_ENABLE__TX_SET((h),DMI32_IO_OVERLAP_ENABLE_YES)

// Refer to the GSC_IO_STATUS_XXX definitions.
#define	DMI32_IO_STATUS__GET(h,w,g)					DMI32_CONFIG_GET((h),DMI32_IO_STATUS,(w),(g))
#define	DMI32_IO_STATUS__RX_GET(h,g)				DMI32_IO_STATUS__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_STATUS__TX_GET(h,g)				DMI32_IO_STATUS__GET((h),DMI32_WHICH_TX,(g))

#define	DMI32_IO_TIMEOUT_NO_WAIT					0
#define	DMI32_IO_TIMEOUT_MAX						3600	// 1 hour
#define	DMI32_IO_TIMEOUT_DEFAULT					10		// seconds
#define	DMI32_IO_TIMEOUT__GET(h,w,g)				DMI32_CONFIG_GET((h),DMI32_IO_TIMEOUT,(w),(g))
#define	DMI32_IO_TIMEOUT__SET(h,w,s)				DMI32_CONFIG_SET((h),DMI32_IO_TIMEOUT,(w),(s))
#define	DMI32_IO_TIMEOUT__RESET(h,w)				DMI32_IO_TIMEOUT__SET((h),(w),DMI32_IO_TIMEOUT_DEFAULT)
#define	DMI32_IO_TIMEOUT__NO_WAIT(h,w)				DMI32_IO_TIMEOUT__SET((h),(w),DMI32_IO_TIMEOUT_NO_WAIT)
#define	DMI32_IO_TIMEOUT__RX_GET(h,g)				DMI32_IO_TIMEOUT__GET((h),DMI32_WHICH_RX,(g))
#define	DMI32_IO_TIMEOUT__RX_SET(h,s)				DMI32_IO_TIMEOUT__SET((h),DMI32_WHICH_RX,(s))
#define	DMI32_IO_TIMEOUT__RX_RESET(h)				DMI32_IO_TIMEOUT__RX_SET((h),DMI32_IO_TIMEOUT_DEFAULT)
#define	DMI32_IO_TIMEOUT__RX_NO_WAIT(h)				DMI32_IO_TIMEOUT__RX_SET((h),DMI32_IO_TIMEOUT_NO_WAIT)
#define	DMI32_IO_TIMEOUT__TX_GET(h,g)				DMI32_IO_TIMEOUT__GET((h),DMI32_WHICH_TX,(g))
#define	DMI32_IO_TIMEOUT__TX_SET(h,s)				DMI32_IO_TIMEOUT__SET((h),DMI32_WHICH_TX,(s))
#define	DMI32_IO_TIMEOUT__TX_RESET(h)				DMI32_IO_TIMEOUT__TX_SET((h),DMI32_IO_TIMEOUT_DEFAULT)
#define	DMI32_IO_TIMEOUT__TX_NO_WAIT(h)				DMI32_IO_TIMEOUT__TX_SET((h),DMI32_IO_TIMEOUT_NO_WAIT)



// Configuration parameters: Interrupt Notification Parameters
#define	DMI32_IRQ_ENCODE(i)						DMI32_CONFIG_ENCODE(DMI32_CONFIG_GROUP_IRQ, (i))

#define	DMI32_IRQ_CALLBACK_ARG					DMI32_IRQ_ENCODE(0)	// which: IRQ#
#define	DMI32_IRQ_CALLBACK_FUNC					DMI32_IRQ_ENCODE(1)	// which: IRQ#
#define	DMI32_IRQ_ENABLE						DMI32_IRQ_ENCODE(2)	// which: IRQ#
#define	DMI32_IRQ_STATE							DMI32_IRQ_ENCODE(3)	// which: IRQ#

#define	DMI32_IRQ_CALLBACK_ARG_DEFAULT			0
#define	DMI32_IRQ_CALLBACK_ARG__GET(h,w,g)		DMI32_CONFIG_GET((h),DMI32_IRQ_CALLBACK_ARG,(w),(g))
#define	DMI32_IRQ_CALLBACK_ARG__SET(h,w,s)		DMI32_CONFIG_SET((h),DMI32_IRQ_CALLBACK_ARG,(w),(s))
#define	DMI32_IRQ_CALLBACK_ARG__RESET(h,w)		DMI32_IRQ_CALLBACK_ARG__SET((h),(w),DMI32_IRQ_CALLBACK_ARG_DEFAULT)
#define	DMI32_IRQ_CALLBACK_ARG__BS_GET(h,g)		DMI32_IRQ_CALLBACK_ARG__GET((h),DMI32_WHICH_IRQ_BS,(g))
#define	DMI32_IRQ_CALLBACK_ARG__BS_SET(h,s)		DMI32_IRQ_CALLBACK_ARG__SET((h),DMI32_WHICH_IRQ_BS,(s))
#define	DMI32_IRQ_CALLBACK_ARG__BS_RESET(h)		DMI32_IRQ_CALLBACK_ARG__BS_SET((h),DMI32_IRQ_CALLBACK_ARG_DEFAULT)
#define	DMI32_IRQ_CALLBACK_ARG__BE_GET(h,g)		DMI32_IRQ_CALLBACK_ARG__GET((h),DMI32_WHICH_IRQ_BE,(g))
#define	DMI32_IRQ_CALLBACK_ARG__BE_SET(h,s)		DMI32_IRQ_CALLBACK_ARG__SET((h),DMI32_WHICH_IRQ_BE,(s))
#define	DMI32_IRQ_CALLBACK_ARG__BE_RESET(h)		DMI32_IRQ_CALLBACK_ARG__BE_GET((h),DMI32_IRQ_CALLBACK_ARG_DEFAULT)
#define	DMI32_IRQ_CALLBACK_ARG__UIOA_GET(h,g)	DMI32_IRQ_CALLBACK_ARG__GET((h),DMI32_WHICH_IRQ_UIOA,(g))
#define	DMI32_IRQ_CALLBACK_ARG__UIOA_SET(h,s)	DMI32_IRQ_CALLBACK_ARG__SET((h),DMI32_WHICH_IRQ_UIOA,(s))
#define	DMI32_IRQ_CALLBACK_ARG__UIOA_RESET(h)	DMI32_IRQ_CALLBACK_ARG__UIOA_SET((h),DMI32_IRQ_CALLBACK_ARG_DEFAULT)
#define	DMI32_IRQ_CALLBACK_ARG__UIOB_GET(h,g)	DMI32_IRQ_CALLBACK_ARG__GET((h),DMI32_WHICH_IRQ_UIOB,(g))
#define	DMI32_IRQ_CALLBACK_ARG__UIOB_SET(h,s)	DMI32_IRQ_CALLBACK_ARG__SET((h),DMI32_WHICH_IRQ_UIOB,(s))
#define	DMI32_IRQ_CALLBACK_ARG__UIOB_RESET(h)	DMI32_IRQ_CALLBACK_ARG__UIOB_SET((h),DMI32_IRQ_CALLBACK_ARG_DEFAULT)
#define	DMI32_IRQ_CALLBACK_ARG__MBUR_GET(h,g)	DMI32_IRQ_CALLBACK_ARG__GET((h),DMI32_WHICH_IRQ_MBUR,(g))
#define	DMI32_IRQ_CALLBACK_ARG__MBUR_SET(h,s)	DMI32_IRQ_CALLBACK_ARG__SET((h),DMI32_WHICH_IRQ_MBUR,(s))
#define	DMI32_IRQ_CALLBACK_ARG__MBUR_RESET(h)	DMI32_IRQ_CALLBACK_ARG__MBUR_SET((h),DMI32_IRQ_CALLBACK_ARG_DEFAULT)

#define	DMI32_IRQ_CALLBACK_FUNC_DEFAULT			0
#define	DMI32_IRQ_CALLBACK_FUNC__GET(h,w,g)		DMI32_CONFIG_GET((h),DMI32_IRQ_CALLBACK_FUNC,(w),(g))
#define	DMI32_IRQ_CALLBACK_FUNC__SET(h,w,s)		DMI32_CONFIG_SET((h),DMI32_IRQ_CALLBACK_FUNC,(w),(s))
#define	DMI32_IRQ_CALLBACK_FUNC__RESET(h,w)		DMI32_IRQ_CALLBACK_FUNC__SET((h),(w),DMI32_IRQ_CALLBACK_FUNC_DEFAULT)
#define	DMI32_IRQ_CALLBACK_FUNC__BS_GET(h,g)	DMI32_IRQ_CALLBACK_FUNC__GET((h),DMI32_WHICH_IRQ_BS,(g))
#define	DMI32_IRQ_CALLBACK_FUNC__BS_SET(h,s)	DMI32_IRQ_CALLBACK_FUNC__SET((h),DMI32_WHICH_IRQ_BS,(s))
#define	DMI32_IRQ_CALLBACK_FUNC__BS_RESET(h)	DMI32_IRQ_CALLBACK_FUNC__BS_SET((h),DMI32_IRQ_CALLBACK_FUNC_DEFAULT)
#define	DMI32_IRQ_CALLBACK_FUNC__BE_GET(h,g)	DMI32_IRQ_CALLBACK_FUNC__GET((h),DMI32_WHICH_IRQ_BE,(g))
#define	DMI32_IRQ_CALLBACK_FUNC__BE_SET(h,s)	DMI32_IRQ_CALLBACK_FUNC__SET((h),DMI32_WHICH_IRQ_BE,(s))
#define	DMI32_IRQ_CALLBACK_FUNC__BE_RESET(h)	DMI32_IRQ_CALLBACK_FUNC__BE_SET((h),DMI32_IRQ_CALLBACK_FUNC_DEFAULT)
#define	DMI32_IRQ_CALLBACK_FUNC__UIOA_GET(h,g)	DMI32_IRQ_CALLBACK_FUNC__GET((h),DMI32_WHICH_IRQ_UIOA,(g))
#define	DMI32_IRQ_CALLBACK_FUNC__UIOA_SET(h,s)	DMI32_IRQ_CALLBACK_FUNC__SET((h),DMI32_WHICH_IRQ_UIOA,(s))
#define	DMI32_IRQ_CALLBACK_FUNC__UIOA_RESET(h)	DMI32_IRQ_CALLBACK_FUNC__UIOA_SET((h),DMI32_IRQ_CALLBACK_FUNC_DEFAULT)
#define	DMI32_IRQ_CALLBACK_FUNC__UIOB_GET(h,g)	DMI32_IRQ_CALLBACK_FUNC__GET((h),DMI32_WHICH_IRQ_UIOB,(g))
#define	DMI32_IRQ_CALLBACK_FUNC__UIOB_SET(h,s)	DMI32_IRQ_CALLBACK_FUNC__SET((h),DMI32_WHICH_IRQ_UIOB,(s))
#define	DMI32_IRQ_CALLBACK_FUNC__UIOB_RESET(h)	DMI32_IRQ_CALLBACK_FUNC__UIOB_SET((h),DMI32_IRQ_CALLBACK_FUNC_DEFAULT)
#define	DMI32_IRQ_CALLBACK_FUNC__MBUR_GET(h,g)	DMI32_IRQ_CALLBACK_FUNC__GET((h),DMI32_WHICH_IRQ_MBUR,(g))
#define	DMI32_IRQ_CALLBACK_FUNC__MBUR_SET(h,s)	DMI32_IRQ_CALLBACK_FUNC__SET((h),DMI32_WHICH_IRQ_MBUR,(s))
#define	DMI32_IRQ_CALLBACK_FUNC__MBUR_RESET(h)	DMI32_IRQ_CALLBACK_FUNC__MBUR_SET((h),DMI32_IRQ_CALLBACK_FUNC_DEFAULT)

#define	DMI32_IRQ_ENABLE_NO						0
#define	DMI32_IRQ_ENABLE_YES					1
#define	DMI32_IRQ_ENABLE_DEFAULT				DMI32_IRQ_ENABLE_NO
#define	DMI32_IRQ_ENABLE__GET(h,w,g)		   	DMI32_CONFIG_GET((h),DMI32_IRQ_ENABLE,(w),(g))
#define	DMI32_IRQ_ENABLE__SET(h,w,s)		   	DMI32_CONFIG_SET((h),DMI32_IRQ_ENABLE,(w),(s))
#define	DMI32_IRQ_ENABLE__RESET(h,w)		   	DMI32_IRQ_ENABLE__SET((h),(w),DMI32_IRQ_ENABLE_NO)
#define	DMI32_IRQ_ENABLE__NO(h,w)		   		DMI32_IRQ_ENABLE__SET((h),(w),DMI32_IRQ_ENABLE_NO)
#define	DMI32_IRQ_ENABLE__YES(h,w)				DMI32_IRQ_ENABLE__SET((h),(w),DMI32_IRQ_ENABLE_YES)
#define	DMI32_IRQ_ENABLE__BS_GET(h,g)		   	DMI32_IRQ_ENABLE__GET((h),DMI32_WHICH_IRQ_BS,(g))
#define	DMI32_IRQ_ENABLE__BS_SET(h,s)		   	DMI32_IRQ_ENABLE__SET((h),DMI32_WHICH_IRQ_BS,(s))
#define	DMI32_IRQ_ENABLE__BS_RESET(h)		   	DMI32_IRQ_ENABLE__BS_SET((h),DMI32_IRQ_ENABLE_DEFAULT)
#define	DMI32_IRQ_ENABLE__BS_NO(h)		   		DMI32_IRQ_ENABLE__BS_SET((h),DMI32_IRQ_ENABLE_NO)
#define	DMI32_IRQ_ENABLE__BS_YES(h)				DMI32_IRQ_ENABLE__BS_SET((h),DMI32_IRQ_ENABLE_YES)
#define	DMI32_IRQ_ENABLE__BE_GET(h,g)		   	DMI32_IRQ_ENABLE__GET((h),DMI32_WHICH_IRQ_BE,(g))
#define	DMI32_IRQ_ENABLE__BE_SET(h,s)		   	DMI32_IRQ_ENABLE__SET((h),DMI32_WHICH_IRQ_BE,(s))
#define	DMI32_IRQ_ENABLE__BE_RESET(h)		   	DMI32_IRQ_ENABLE__BE_SET((h),DMI32_IRQ_ENABLE_DEFAULT)
#define	DMI32_IRQ_ENABLE__BE_NO(h)		   		DMI32_IRQ_ENABLE__BE_SET((h),DMI32_IRQ_ENABLE_NO)
#define	DMI32_IRQ_ENABLE__BE_YES(h)				DMI32_IRQ_ENABLE__BE_SET((h),DMI32_IRQ_ENABLE_YES)
#define	DMI32_IRQ_ENABLE__UIOA_GET(h,g)			DMI32_IRQ_ENABLE__GET((h),DMI32_WHICH_IRQ_UIOA,(g))
#define	DMI32_IRQ_ENABLE__UIOA_SET(h,s)			DMI32_IRQ_ENABLE__SET((h),DMI32_WHICH_IRQ_UIOA,(s))
#define	DMI32_IRQ_ENABLE__UIOA_RESET(h)		   	DMI32_IRQ_ENABLE__UIOA_SET((h),DMI32_IRQ_ENABLE_DEFAULT)
#define	DMI32_IRQ_ENABLE__UIOA_NO(h)		   	DMI32_IRQ_ENABLE__UIOA_SET((h),DMI32_IRQ_ENABLE_NO)
#define	DMI32_IRQ_ENABLE__UIOA_YES(h)			DMI32_IRQ_ENABLE__UIOA_SET((h),DMI32_IRQ_ENABLE_YES)
#define	DMI32_IRQ_ENABLE__UIOB_GET(h,g)			DMI32_IRQ_ENABLE__GET((h),DMI32_WHICH_IRQ_UIOB,(g))
#define	DMI32_IRQ_ENABLE__UIOB_SET(h,s)			DMI32_IRQ_ENABLE__SET((h),DMI32_WHICH_IRQ_UIOB,(s))
#define	DMI32_IRQ_ENABLE__UIOB_RESET(h)		   	DMI32_IRQ_ENABLE__UIOB_SET((h),DMI32_IRQ_ENABLE_DEFAULT)
#define	DMI32_IRQ_ENABLE__UIOB_NO(h)		   	DMI32_IRQ_ENABLE__UIOB_SET((h),DMI32_IRQ_ENABLE_NO)
#define	DMI32_IRQ_ENABLE__UIOB_YES(h)			DMI32_IRQ_ENABLE__UIOB_SET((h),DMI32_IRQ_ENABLE_YES)
#define	DMI32_IRQ_ENABLE__MBUR_GET(h,g)			DMI32_IRQ_ENABLE__GET((h),DMI32_WHICH_IRQ_MBUR,(g))
#define	DMI32_IRQ_ENABLE__MBUR_SET(h,s)			DMI32_IRQ_ENABLE__SET((h),DMI32_WHICH_IRQ_MBUR,(s))
#define	DMI32_IRQ_ENABLE__MBUR_RESET(h)		   	DMI32_IRQ_ENABLE__MBUR_SET((h),DMI32_IRQ_ENABLE_DEFAULT)
#define	DMI32_IRQ_ENABLE__MBUR_NO(h)		   	DMI32_IRQ_ENABLE__MBUR_SET((h),DMI32_IRQ_ENABLE_NO)
#define	DMI32_IRQ_ENABLE__MBUR_YES(h)			DMI32_IRQ_ENABLE__MBUR_SET((h),DMI32_IRQ_ENABLE_YES)

#define	DMI32_IRQ_STATE_INACTIVE				0
#define	DMI32_IRQ_STATE_ACTIVE					1
#define	DMI32_IRQ_STATE__GET(h,w,g)		   		DMI32_CONFIG_GET((h),DMI32_IRQ_ENABLE,(w),(g))
#define	DMI32_IRQ_STATE__BS_GET(h,g)		   	DMI32_IRQ_STATE__GET((h),DMI32_WHICH_IRQ_BS,(g))
#define	DMI32_IRQ_STATE__BE_GET(h,g)		   	DMI32_IRQ_STATE__GET((h),DMI32_WHICH_IRQ_BE,(g))
#define	DMI32_IRQ_STATE__UIOA_GET(h,g)			DMI32_IRQ_STATE__GET((h),DMI32_WHICH_IRQ_UIOA,(g))
#define	DMI32_IRQ_STATE__UIOB_GET(h,g)			DMI32_IRQ_STATE__GET((h),DMI32_WHICH_IRQ_UIOB,(g))
#define	DMI32_IRQ_STATE__MBUR_GET(h,g)			DMI32_IRQ_STATE__GET((h),DMI32_WHICH_IRQ_MBUR,(g))



// Configuration parameters: Misc Parameters
#define	DMI32_MISC_ENCODE(i)							DMI32_CONFIG_ENCODE(DMI32_CONFIG_GROUP_MISC, (i))

#define	DMI32_MISC_BOARD_JUMPERS						DMI32_MISC_ENCODE(0)	// GET only
#define	DMI32_MISC_DRAM_CAPACITY_MB						DMI32_MISC_ENCODE(1)
#define	DMI32_MISC_DRAM_SIZE_MB							DMI32_MISC_ENCODE(2)
#define	DMI32_MISC_FEATURES								DMI32_MISC_ENCODE(3)
#define	DMI32_MISC_MAP_GSC_REGS							DMI32_MISC_ENCODE(4)
#define	DMI32_MISC_MAP_PLX_REGS							DMI32_MISC_ENCODE(5)
#define	DMI32_MISC_STRICT_ARGUMENTS						DMI32_MISC_ENCODE(6)
#define	DMI32_MISC_STRICT_CONFIG						DMI32_MISC_ENCODE(7)
#define	DMI32_MISC_STRICT_DRAM_LIMITS					DMI32_MISC_ENCODE(8)

#define	DMI32_MISC_BOARD_JUMPERS__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_MISC_BOARD_JUMPERS,0,(g))

#define	DMI32_MISC_DRAM_CAPACITY_MB__GET(h,g)			DMI32_CONFIG_GET((h),DMI32_MISC_DRAM_CAPACITY_MB,0,(g))

#define	DMI32_MISC_DRAM_SIZE_MB__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_MISC_DRAM_SIZE_MB,0,(g))

#define	DMI32_MISC_FEATURES_ABSENT						0
#define	DMI32_MISC_FEATURES_PRESENT						1
#define	DMI32_MISC_FEATURES_FR							0	// SET field: Is Features Register present?
#define	DMI32_MISC_FEATURES_IRQ_MBUR					1	// SET field: Is ICR Multi-Buffer Under Run present?
#define	DMI32_MISC_FEATURES_LMBR						2	// SET field: Is Local Memory Bank Register present?
#define	DMI32_MISC_FEATURES_SW_DIR						3	// SET field: Is Start Word Direction supported?
#define	DMI32_MISC_FEATURES__GET(h,s,g)					DMI32_CONFIG_SET_GET((h),DMI32_MISC_FEATURES,0,(s),(g))
#define	DMI32_MISC_FEATURES__FR(h,g)					DMI32_MISC_FEATURES__GET((h),DMI32_MISC_FEATURES_FR,(g))
#define	DMI32_MISC_FEATURES__LMBR(h,g)					DMI32_MISC_FEATURES__GET((h),DMI32_MISC_FEATURES_LMBR,(g))
#define	DMI32_MISC_FEATURES__IRQ_MBUR(h,g)				DMI32_MISC_FEATURES__GET((h),DMI32_MISC_FEATURES_IRQ_MBUR,(g))
#define	DMI32_MISC_FEATURES__SW_DIR(h,g)				DMI32_MISC_FEATURES__GET((h),DMI32_MISC_FEATURES_SW_DIR,(g))

#define	DMI32_MISC_MAP_GSC_REGS_DISABLE					0
#define	DMI32_MISC_MAP_GSC_REGS_ENABLE					1
#define	DMI32_MISC_MAP_GSC_REGS_DEFAULT					DMI32_MISC_MAP_GSC_REGS_ENABLE
#define	DMI32_MISC_MAP_GSC_REGS__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_MISC_MAP_GSC_REGS,0,(g))
#define	DMI32_MISC_MAP_GSC_REGS__SET(h,s)				DMI32_CONFIG_SET((h),DMI32_MISC_MAP_GSC_REGS,0,(s))
#define	DMI32_MISC_MAP_GSC_REGS__RESET(h)				DMI32_MISC_MAP_GSC_REGS__SET((h),DMI32_MISC_MAP_GSC_REGS_DEFAULT)
#define	DMI32_MISC_MAP_GSC_REGS__ENABLE(h)				DMI32_MISC_MAP_GSC_REGS__SET((h),DMI32_MISC_MAP_GSC_REGS_ENABLE)

#define	DMI32_MISC_MAP_PLX_REGS_DISABLE					0
#define	DMI32_MISC_MAP_PLX_REGS_ENABLE					1
#define	DMI32_MISC_MAP_PLX_REGS_DEFAULT					DMI32_MISC_MAP_PLX_REGS_ENABLE
#define	DMI32_MISC_MAP_PLX_REGS__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_MISC_MAP_PLX_REGS,0,(g))
#define	DMI32_MISC_MAP_PLX_REGS__SET(h,s)				DMI32_CONFIG_SET((h),DMI32_MISC_MAP_PLX_REGS,0,(s))
#define	DMI32_MISC_MAP_PLX_REGS__RESET(h)				DMI32_MISC_MAP_PLX_REGS__SET((h),DMI32_MISC_MAP_PLX_REGS_DEFAULT)
#define	DMI32_MISC_MAP_PLX_REGS__ENABLE(h)				DMI32_MISC_MAP_PLX_REGS__SET((h),DMI32_MISC_MAP_PLX_REGS_ENABLE)

#define	DMI32_MISC_STRICT_ARGUMENTS_DISABLE				0
#define	DMI32_MISC_STRICT_ARGUMENTS_ENABLE				1
#define	DMI32_MISC_STRICT_ARGUMENTS_DEFAULT				DMI32_MISC_STRICT_ARGUMENTS_DISABLE
#define	DMI32_MISC_STRICT_ARGUMENTS__GET(h,g)			DMI32_CONFIG_GET((h),DMI32_MISC_STRICT_ARGUMENTS,0,(g))
#define	DMI32_MISC_STRICT_ARGUMENTS__SET(h,s)			DMI32_CONFIG_SET((h),DMI32_MISC_STRICT_ARGUMENTS,0,(s))
#define	DMI32_MISC_STRICT_ARGUMENTS__RESET(h)			DMI32_MISC_STRICT_ARGUMENTS__SET((h),DMI32_MISC_STRICT_ARGUMENTS_DEFAULT)
#define	DMI32_MISC_STRICT_ARGUMENTS__NO(h)				DMI32_MISC_STRICT_ARGUMENTS__SET((h),DMI32_MISC_STRICT_ARGUMENTS_DISABLE)
#define	DMI32_MISC_STRICT_ARGUMENTS__YES(h)				DMI32_MISC_STRICT_ARGUMENTS__SET((h),DMI32_MISC_STRICT_ARGUMENTS_ENABLE)

#define	DMI32_MISC_STRICT_CONFIG_DISABLE				0
#define	DMI32_MISC_STRICT_CONFIG_ENABLE					1
#define	DMI32_MISC_STRICT_CONFIG_DEFAULT				DMI32_MISC_STRICT_CONFIG_DISABLE
#define	DMI32_MISC_STRICT_CONFIG__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_MISC_STRICT_CONFIG,0,(g))
#define	DMI32_MISC_STRICT_CONFIG__SET(h,s)				DMI32_CONFIG_SET((h),DMI32_MISC_STRICT_CONFIG,0,(s))
#define	DMI32_MISC_STRICT_CONFIG__RESET(h)				DMI32_MISC_STRICT_CONFIG__SET((h),DMI32_MISC_STRICT_CONFIG_DEFAULT)
#define	DMI32_MISC_STRICT_CONFIG__NO(h)					DMI32_MISC_STRICT_CONFIG__SET((h),DMI32_MISC_STRICT_CONFIG_DISABLE)
#define	DMI32_MISC_STRICT_CONFIG__YES(h)				DMI32_MISC_STRICT_CONFIG__SET((h),DMI32_MISC_STRICT_CONFIG_ENABLE)

#define	DMI32_MISC_STRICT_DRAM_LIMITS_DISABLE			0
#define	DMI32_MISC_STRICT_DRAM_LIMITS_ENABLE			1
#define	DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT			DMI32_MISC_STRICT_DRAM_LIMITS_ENABLE
#define	DMI32_MISC_STRICT_DRAM_LIMITS__GET(h,g)			DMI32_CONFIG_GET((h),DMI32_MISC_STRICT_DRAM_LIMITS,0,(g))
#define	DMI32_MISC_STRICT_DRAM_LIMITS__SET(h,s)			DMI32_CONFIG_SET((h),DMI32_MISC_STRICT_DRAM_LIMITS,0,(s))
#define	DMI32_MISC_STRICT_DRAM_LIMITS__RESET(h)			DMI32_MISC_STRICT_DRAM_LIMITS__SET((h),DMI32_MISC_STRICT_DRAM_LIMITS_DEFAULT)
#define	DMI32_MISC_STRICT_DRAM_LIMITS__NO(h)			DMI32_MISC_STRICT_DRAM_LIMITS__SET((h),DMI32_MISC_STRICT_DRAM_LIMITS_DISABLE)
#define	DMI32_MISC_STRICT_DRAM_LIMITS__YES(h)			DMI32_MISC_STRICT_DRAM_LIMITS__SET((h),DMI32_MISC_STRICT_DRAM_LIMITS_ENABLE)



// Configuration parameters: User I/O Parameters
#define	DMI32_USER_IO_ENCODE(i)						DMI32_CONFIG_ENCODE(DMI32_CONFIG_GROUP_USER_IO, (i))

#define	DMI32_USER_IO_DIRECTION						DMI32_USER_IO_ENCODE(0)
#define	DMI32_USER_IO_INPUT							DMI32_USER_IO_ENCODE(1)
#define	DMI32_USER_IO_OUTPUT						DMI32_USER_IO_ENCODE(2)

#define	DMI32_USER_IO_DIRECTION_IN					0
#define	DMI32_USER_IO_DIRECTION_OUT					1
#define	DMI32_USER_IO_DIRECTION_DEFAULT				DMI32_USER_IO_DIRECTION_IN
#define	DMI32_USER_IO_DIRECTION__GET(h,w,g)			DMI32_CONFIG_GET((h),DMI32_USER_IO_DIRECTION,(w),(g))
#define	DMI32_USER_IO_DIRECTION__SET(h,w,s)			DMI32_CONFIG_SET((h),DMI32_USER_IO_DIRECTION,(w),(s))
#define	DMI32_USER_IO_DIRECTION__RESET(h,w)			DMI32_USER_IO_DIRECTION__SET((h),(w),DMI32_USER_IO_DIRECTION_DEFAULT)
#define	DMI32_USER_IO_DIRECTION__IN(h,w)			DMI32_USER_IO_DIRECTION__SET((h),(w),DMI32_USER_IO_DIRECTION_IN)
#define	DMI32_USER_IO_DIRECTION__OUT(h,w)			DMI32_USER_IO_DIRECTION__SET((h),(w),DMI32_USER_IO_DIRECTION_OUT)
#define	DMI32_USER_IO_DIRECTION__A_GET(h,g)			DMI32_CONFIG_GET((h),DMI32_USER_IO_DIRECTION,DMI32_WHICH_USER_IO_A,(g))
#define	DMI32_USER_IO_DIRECTION__A_SET(h,s)			DMI32_CONFIG_SET((h),DMI32_USER_IO_DIRECTION,DMI32_WHICH_USER_IO_A,(s))
#define	DMI32_USER_IO_DIRECTION__A_RESET(h)			DMI32_USER_IO_DIRECTION__A_SET((h),DMI32_USER_IO_DIRECTION_DEFAULT)
#define	DMI32_USER_IO_DIRECTION__A_IN(h)			DMI32_USER_IO_DIRECTION__A_SET((h),DMI32_USER_IO_DIRECTION_IN)
#define	DMI32_USER_IO_DIRECTION__A_OUT(h)			DMI32_USER_IO_DIRECTION__A_SET((h),DMI32_USER_IO_DIRECTION_OUT)
#define	DMI32_USER_IO_DIRECTION__B_GET(h,g)			DMI32_CONFIG_GET((h),DMI32_USER_IO_DIRECTION,DMI32_WHICH_USER_IO_B,(g))
#define	DMI32_USER_IO_DIRECTION__B_SET(h,s)			DMI32_CONFIG_SET((h),DMI32_USER_IO_DIRECTION,DMI32_WHICH_USER_IO_B,(s))
#define	DMI32_USER_IO_DIRECTION__B_RESET(h)			DMI32_USER_IO_DIRECTION__B_SET((h),DMI32_USER_IO_DIRECTION_DEFAULT)
#define	DMI32_USER_IO_DIRECTION__B_IN(h)			DMI32_USER_IO_DIRECTION__B_SET((h),DMI32_USER_IO_DIRECTION_IN)
#define	DMI32_USER_IO_DIRECTION__B_OUT(h)			DMI32_USER_IO_DIRECTION__B_SET((h),DMI32_USER_IO_DIRECTION_OUT)

#define	DMI32_USER_IO_INPUT_MASK					0x7FL
#define	DMI32_USER_IO_INPUT__GET(h,w,g)				DMI32_CONFIG_GET((h),DMI32_USER_IO_INPUT,(w),(g))
#define	DMI32_USER_IO_INPUT__A_GET(h,g)				DMI32_CONFIG_GET((h),DMI32_USER_IO_INPUT,DMI32_WHICH_USER_IO_A,(g))
#define	DMI32_USER_IO_INPUT__B_GET(h,g)				DMI32_CONFIG_GET((h),DMI32_USER_IO_INPUT,DMI32_WHICH_USER_IO_B,(g))
#define	DMI32_USER_IO_INPUT__CUR_GET(h,g)			DMI32_CONFIG_GET((h),DMI32_USER_IO_INPUT,DMI32_WHICH_USER_IO_CURRENT,(g))

#define	DMI32_USER_IO_OUTPUT_DEFAULT				0
#define	DMI32_USER_IO_OUTPUT_MASK					0x7FL
#define	DMI32_USER_IO_OUTPUT__GET(h,w,g)			DMI32_CONFIG_GET((h),DMI32_USER_IO_OUTPUT,(w),(g))
#define	DMI32_USER_IO_OUTPUT__SET(h,w,s)			DMI32_CONFIG_SET((h),DMI32_USER_IO_OUTPUT,(w),(s))
#define	DMI32_USER_IO_OUTPUT__RESET(h,w)			DMI32_USER_IO_OUTPUT__SET((h),(w),DMI32_USER_IO_OUTPUT_DEFAULT)
#define	DMI32_USER_IO_OUTPUT__A_GET(h,g)			DMI32_CONFIG_GET((h),DMI32_USER_IO_OUTPUT,DMI32_WHICH_USER_IO_A,(g))
#define	DMI32_USER_IO_OUTPUT__A_SET(h,s)			DMI32_CONFIG_SET((h),DMI32_USER_IO_OUTPUT,DMI32_WHICH_USER_IO_A,(s))
#define	DMI32_USER_IO_OUTPUT__A_RESET(h)			DMI32_USER_IO_OUTPUT__A_SET((h),DMI32_USER_IO_OUTPUT_DEFAULT)
#define	DMI32_USER_IO_OUTPUT__B_GET(h,g)			DMI32_CONFIG_GET((h),DMI32_USER_IO_OUTPUT,DMI32_WHICH_USER_IO_B,(g))
#define	DMI32_USER_IO_OUTPUT__B_SET(h,s)			DMI32_CONFIG_SET((h),DMI32_USER_IO_OUTPUT,DMI32_WHICH_USER_IO_B,(s))
#define	DMI32_USER_IO_OUTPUT__B_RESET(h)			DMI32_USER_IO_OUTPUT__B_SET((h),DMI32_USER_IO_OUTPUT_DEFAULT)
#define	DMI32_USER_IO_OUTPUT__CUR_GET(h,g)			DMI32_CONFIG_GET((h),DMI32_USER_IO_OUTPUT,DMI32_WHICH_USER_IO_CURRENT,(g))
#define	DMI32_USER_IO_OUTPUT__CUR_SET(h,s)			DMI32_CONFIG_SET((h),DMI32_USER_IO_OUTPUT,DMI32_WHICH_USER_IO_CURRENT,(s))
#define	DMI32_USER_IO_OUTPUT__CUR_RESET(h)			DMI32_USER_IO_OUTPUT__CUR_SET((h),DMI32_USER_IO_OUTPUT_DEFAULT)



// Configuration parameters: Transmitter Parameters
#define	DMI32_XFER_ENCODE(i)						DMI32_CONFIG_ENCODE(DMI32_CONFIG_GROUP_XFER, (i))

#define	DMI32_XFER_CLKIN_TEST						DMI32_XFER_ENCODE( 0)
#define	DMI32_XFER_DIRECTION						DMI32_XFER_ENCODE( 1)	// While idle only!
#define	DMI32_XFER_ENABLE							DMI32_XFER_ENCODE( 2)	// Enable, Disable
#define	DMI32_XFER_INTERNAL_CLOCK_SPEED				DMI32_XFER_ENCODE( 3)	// Hi, Low, While idle only!
#define	DMI32_XFER_MODE								DMI32_XFER_ENCODE( 4)	// Single, Multi-Buffer
#define	DMI32_XFER_OFFSET							DMI32_XFER_ENCODE( 5)
#define	DMI32_XFER_SIZE								DMI32_XFER_ENCODE( 6)	// In bytes.
#define	DMI32_XFER_START_WORD						DMI32_XFER_ENCODE( 7)
#define	DMI32_XFER_START_WORD_DIR					DMI32_XFER_ENCODE( 8)	// Rx, Tx, if available
#define	DMI32_XFER_STATE							DMI32_XFER_ENCODE( 9)	// GET only
#define	DMI32_XFER_STATE_RX							DMI32_XFER_ENCODE(10)	// Tx, Rx
#define	DMI32_XFER_STATE_TX							DMI32_XFER_ENCODE(11)	// GET only
#define	DMI32_XFER_TEST_MODE						DMI32_XFER_ENCODE(12)	// While idle only!

#define	DMI32_XFER_CLKIN_TEST_NO					0	// SET option
#define	DMI32_XFER_CLKIN_TEST_YES					1	// SET option
#define	DMI32_XFER_CLKIN_TEST_DEFAULT				DMI32_XFER_CLKIN_TEST_NO
#define	DMI32_XFER_CLKIN_TEST_BUSY					0x10000000	// GET option
#define	DMI32_XFER_CLKIN_TEST_NONE					0x20000000	// GET option
#define	DMI32_XFER_CLKIN_TEST__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_XFER_CLKIN_TEST,0,(g))
#define	DMI32_XFER_CLKIN_TEST__SET(h,s)				DMI32_CONFIG_SET((h),DMI32_XFER_CLKIN_TEST,0,(s))
#define	DMI32_XFER_CLKIN_TEST__YES(h,g)				DMI32_CONFIG_SET_GET((h),DMI32_XFER_CLKIN_TEST,0,DMI32_XFER_CLKIN_TEST_YES,(g))

#define	DMI32_XFER_DIRECTION_RX						0
#define	DMI32_XFER_DIRECTION_TX						1
#define	DMI32_XFER_DIRECTION_DEFAULT				DMI32_XFER_DIRECTION_RX
#define	DMI32_XFER_DIRECTION__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_XFER_DIRECTION,0,(g))
#define	DMI32_XFER_DIRECTION__SET(h,s)				DMI32_CONFIG_SET((h),DMI32_XFER_DIRECTION,0,(s))
#define	DMI32_XFER_DIRECTION__RESET(h)				DMI32_XFER_DIRECTION__SET((h),DMI32_XFER_DIRECTION_DEFAULT)
#define	DMI32_XFER_DIRECTION__RX(h)					DMI32_XFER_DIRECTION__SET((h),DMI32_XFER_DIRECTION_RX)
#define	DMI32_XFER_DIRECTION__TX(h)					DMI32_XFER_DIRECTION__SET((h),DMI32_XFER_DIRECTION_TX)

#define	DMI32_XFER_ENABLE_NO						0
#define	DMI32_XFER_ENABLE_YES						1
#define	DMI32_XFER_ENABLE_DEFAULT					DMI32_XFER_ENABLE_NO
#define	DMI32_XFER_ENABLE__GET(h,g)					DMI32_CONFIG_GET((h),DMI32_XFER_ENABLE,0,(g))
#define	DMI32_XFER_ENABLE__SET(h,s)					DMI32_CONFIG_SET((h),DMI32_XFER_ENABLE,0,(s))
#define	DMI32_XFER_ENABLE__RESET(h)					DMI32_XFER_ENABLE__SET((h),DMI32_XFER_ENABLE_DEFAULT)
#define	DMI32_XFER_ENABLE__NO(h)					DMI32_XFER_ENABLE__SET((h),DMI32_XFER_ENABLE_NO)
#define	DMI32_XFER_ENABLE__YES(h)					DMI32_XFER_ENABLE__SET((h),DMI32_XFER_ENABLE_YES)

#define	DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH		0
#define	DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW			1
#define	DMI32_XFER_INTERNAL_CLOCK_SPEED_DEFAULT		DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH
#define	DMI32_XFER_INTERNAL_CLOCK_SPEED__GET(h,g)	DMI32_CONFIG_GET((h),DMI32_XFER_INTERNAL_CLOCK_SPEED,0,(g))
#define	DMI32_XFER_INTERNAL_CLOCK_SPEED__SET(h,s)	DMI32_CONFIG_SET((h),DMI32_XFER_INTERNAL_CLOCK_SPEED,0,(s))
#define	DMI32_XFER_INTERNAL_CLOCK_SPEED__RESET(h)	DMI32_XFER_INTERNAL_CLOCK_SPEED__SET((h),DMI32_XFER_INTERNAL_CLOCK_SPEED_DEFAULT)
#define	DMI32_XFER_INTERNAL_CLOCK_SPEED__HIGH(h)	DMI32_XFER_INTERNAL_CLOCK_SPEED__SET((h),DMI32_XFER_INTERNAL_CLOCK_SPEED_HIGH)
#define	DMI32_XFER_INTERNAL_CLOCK_SPEED__LOW(h)		DMI32_XFER_INTERNAL_CLOCK_SPEED__SET((h),DMI32_XFER_INTERNAL_CLOCK_SPEED_LOW)

#define	DMI32_XFER_MODE_SINGLE						0
#define	DMI32_XFER_MODE_MULTI						1
#define	DMI32_XFER_MODE_DEFAULT						DMI32_XFER_MODE_SINGLE
#define	DMI32_XFER_MODE__GET(h,g)					DMI32_CONFIG_GET((h),DMI32_XFER_MODE,0,(g))
#define	DMI32_XFER_MODE__SET(h,s)					DMI32_CONFIG_SET((h),DMI32_XFER_MODE,0,(s))
#define	DMI32_XFER_MODE__RESET(h)					DMI32_XFER_MODE__SET((h),DMI32_XFER_MODE_DEFAULT)
#define	DMI32_XFER_MODE__SINGLE(h)					DMI32_XFER_MODE__SET((h),DMI32_XFER_MODE_SINGLE)
#define	DMI32_XFER_MODE__MULTI(h)					DMI32_XFER_MODE__SET((h),DMI32_XFER_MODE_MULTI)

#define	DMI32_XFER_OFFSET_MIN						0			// >= 0
#define	DMI32_XFER_OFFSET_MAX						0xFFFFFFFC	// <= (4BG - 4)
#define	DMI32_XFER_OFFSET_MASK						0xFFFFFFFC	// 32-bit boundary
#define	DMI32_XFER_OFFSET_DEFAULT					DMI32_XFER_OFFSET_MIN
#define	DMI32_XFER_OFFSET__GET(h,g)					DMI32_CONFIG_GET((h),DMI32_XFER_OFFSET,0,(g))
#define	DMI32_XFER_OFFSET__SET(h,s)					DMI32_CONFIG_SET((h),DMI32_XFER_OFFSET,0,(s))
#define	DMI32_XFER_OFFSET__RESET(h)					DMI32_XFER_OFFSET__SET((h),DMI32_XFER_OFFSET_DEFAULT)

#define	DMI32_XFER_SIZE_MIN							4
#define	DMI32_XFER_SIZE_MAX							0xFFFFFFFC	// 4GB - 4
#define	DMI32_XFER_SIZE_4GB							0			// Special case!!!
#define	DMI32_XFER_SIZE_MASK						0xFFFFFFFC	// 32-bit boundary
#define	DMI32_XFER_SIZE_DEFAULT						DMI32_XFER_SIZE_MIN
#define	DMI32_XFER_SIZE__GET(h,g)					DMI32_CONFIG_GET((h),DMI32_XFER_SIZE,0,(g))
#define	DMI32_XFER_SIZE__SET(h,s)					DMI32_CONFIG_SET((h),DMI32_XFER_SIZE,0,(s))
#define	DMI32_XFER_SIZE__RESET(h)					DMI32_XFER_SIZE__SET((h),DMI32_XFER_SIZE_DEFAULT)

#define	DMI32_XFER_START_WORD_MASK					0x7F
#define	DMI32_XFER_START_WORD_DEFAULT				0x78
#define	DMI32_XFER_START_WORD__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_XFER_START_WORD,0,(g))
#define	DMI32_XFER_START_WORD__SET(h,s)				DMI32_CONFIG_SET((h),DMI32_XFER_START_WORD,0,(s))
#define	DMI32_XFER_START_WORD__RESET(h)				DMI32_XFER_START_WORD__SET((h),DMI32_XFER_START_WORD_DEFAULT)

#define	DMI32_XFER_START_WORD_DIR_RX				0
#define	DMI32_XFER_START_WORD_DIR_TX				1
#define	DMI32_XFER_START_WORD_DIR_DEFAULT			DMI32_XFER_START_WORD_DIR_TX
#define	DMI32_XFER_START_WORD_DIR__GET(h,g)			DMI32_CONFIG_GET((h),DMI32_XFER_START_WORD_DIR,0,(g))
#define	DMI32_XFER_START_WORD_DIR__SET(h,s)			DMI32_CONFIG_SET((h),DMI32_XFER_START_WORD_DIR,0,(s))
#define	DMI32_XFER_START_WORD_DIR__RESET(h)			DMI32_XFER_START_WORD_DIR__SET((h),DMI32_XFER_START_WORD_DIR_DEFAULT)
#define	DMI32_XFER_START_WORD_DIR__RX(h)			DMI32_XFER_START_WORD_DIR__SET((h),DMI32_XFER_START_WORD_DIR_RX)
#define	DMI32_XFER_START_WORD_DIR__TX(h)			DMI32_XFER_START_WORD_DIR__SET((h),DMI32_XFER_START_WORD_DIR_TX)

#define	DMI32_XFER_STATE_INACTIVE					0	// State GET option
#define	DMI32_XFER_STATE_ACTIVE						1	// State GET option
#define	DMI32_XFER_STATE__GET(h,g)					DMI32_CONFIG_GET((h),DMI32_XFER_STATE,0,(g))

#define	DMI32_XFER_STATE_RX_INACTIVE				0	// State GET option
#define	DMI32_XFER_STATE_RX_ACTIVE					1	// State GET option
#define	DMI32_XFER_STATE_RX__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_XFER_STATE_RX,0,(g))

#define	DMI32_XFER_STATE_TX_INACTIVE				0	// State GET option
#define	DMI32_XFER_STATE_TX_ACTIVE					1	// State GET option
#define	DMI32_XFER_STATE_TX__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_XFER_STATE_TX,0,(g))

#define	DMI32_XFER_TEST_MODE_NO						0
#define	DMI32_XFER_TEST_MODE_YES					1
#define	DMI32_XFER_TEST_MODE_DEFAULT				DMI32_XFER_TEST_MODE_NO
#define	DMI32_XFER_TEST_MODE__GET(h,g)				DMI32_CONFIG_GET((h),DMI32_XFER_TEST_MODE,0,(g))
#define	DMI32_XFER_TEST_MODE__SET(h,s)				DMI32_CONFIG_SET((h),DMI32_XFER_TEST_MODE,0,(s))
#define	DMI32_XFER_TEST_MODE__RESET(h)				DMI32_XFER_TEST_MODE__SET((h),DMI32_XFER_TEST_MODE_DEFAULT)
#define	DMI32_XFER_TEST_MODE__NO(h)					DMI32_XFER_TEST_MODE__SET((h),DMI32_XFER_TEST_MODE_NO)
#define	DMI32_XFER_TEST_MODE__YES(h)				DMI32_XFER_TEST_MODE__SET((h),DMI32_XFER_TEST_MODE_YES)



// API Status
#define	DMI32_API_STATUS(s,a)						dmi32_api_status((s),(a),DMI32_API_VERSION)



// Seek Options
#define	DMI32_SEEK_TX_RX(h,s)						dmi32_seek((h),DMI32_WHICH_TX_RX,(s))
#define	DMI32_SEEK_RX(h,s)							dmi32_seek((h),DMI32_WHICH_RX,(s))
#define	DMI32_SEEK_TX(h,s)							dmi32_seek((h),DMI32_WHICH_TX,(s))



// Tell Options
#define	DMI32_TELL_RX(h,g)							dmi32_tell((h),DMI32_WHICH_RX,(g))
#define	DMI32_TELL_TX(h,g)							dmi32_tell((h),DMI32_WHICH_TX,(g))



// Version Information
#define	DMI32_VERSION_GET_LIBRARY(h,b,s)			dmi32_version_get((h),GSC_VERSION_LIBRARY,(b),(s))
#define	DMI32_VERSION_GET_DRIVER(h,b,s)				dmi32_version_get((h),GSC_VERSION_DRIVER,(b),(s))



/* typedefs	*****************************************************************/

// All callback function have this form.
// I/O: arg1 = void* handle, arg2 = I/O status, arg3 = user arg
// IRQ: arg1 = void* handle, arg2 = IRQ "which" bit, arg3 = user arg
typedef	void (*dmi32_callback_func_t)(void* arg1, unsigned long arg2, unsigned long arg3);



/* prototypes	*************************************************************/

// The return value is always the error status for the service.

U32	GSC_EXPORT dmi32_api_status(U32* stat, U32* arg, U32 api_ver);
U32	GSC_EXPORT dmi32_board_count(U8* count);
U32 GSC_EXPORT dmi32_close(void* handle);
U32	GSC_EXPORT dmi32_config(void* handle, U32 parm, U32 which, unsigned long set, unsigned long* get);
U32 GSC_EXPORT dmi32_init(void* handle);
U32	GSC_EXPORT dmi32_io_wait(void* handle, U32 which, U32 timeout_ms);
U32	GSC_EXPORT dmi32_irq_wait(void* handle, U32 which, U32 timeout_ms);
U32	GSC_EXPORT dmi32_open(U8 index, void** handle);
U32	GSC_EXPORT dmi32_read(void* handle, void* buffer, U32 bytes, U32* count);
U32	GSC_EXPORT dmi32_reg_mod(void* handle, U32 reg, U32 value, U32 mask);
U32	GSC_EXPORT dmi32_reg_read(void* handle, U32 reg, U32* value);
U32	GSC_EXPORT dmi32_reg_write(void* handle, U32 reg, U32 value);
U32	GSC_EXPORT dmi32_reset(void* handle);
U32	GSC_EXPORT dmi32_seek(void* handle, U32 which, U32 offset);
U32	GSC_EXPORT dmi32_status_text(U32 status, char* text, size_t size);
U32	GSC_EXPORT dmi32_tell(void* handle, U32 which, unsigned long* offset);
U32	GSC_EXPORT dmi32_version_get(void* handle, U8 id, char* version, size_t size);
U32	GSC_EXPORT dmi32_write(void* handle, const void* buffer, U32 bytes, U32* count);
U32	GSC_EXPORT dmi32_xfer_buf_get(void* handle, U32* offset, U32* size);
U32	GSC_EXPORT dmi32_xfer_buf_set(void* handle, U32 offset, U32 size);



#ifdef __cplusplus
}
#endif

#endif
