/* Application - DMI32 - Any OS */
/* $Workfile: clock.c $ */
/* $Revision: 2 $ */
/* $Date: 10/27/07 10:53a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/bbtest/clock.c $
 *
 * 2     10/27/07 10:53a Dnewbold
 * Ported to Linux and updated API.
 *
 * 1     8/22/05 4:08p Dnewbold
*/

#include <stdio.h>

#include "main.h"
#include "util_label.h"



/*****************************************************************************
*
*	Function:	clock_test
*
*	Purpose:
*
*		Verify the input clock from the second board.
*
*	Arguments:
*
*		i0		The index of the first board being accessed.
*
*		h0		The handle to the first device under test.
*
*		c0		Report the high rate reported by board zero here.
*
*		i1		The index of the second board being accessed.
*
*		h1		The handle to the second device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int clock_test(int i0, void* h0, U32* c0, int i1, void* h1)
{
	unsigned long	low		= 25;
	unsigned long	high	= 50;
	int				errs;
	char			label[128];
	const char*		msg	= NULL;
	U32				status;

	sprintf(label, "Clock Test (%d <- %d)", i0, i1);
	label_text(label);

	// Configure the receiving board.
	status	= dmi32_init(h0);

	if (status == GSC_SUCCESS)
		status	= DMI32_XFER_TEST_MODE__NO(h0);

	// Configure the transmitting board for the HIGH rate clock.

	if (status == GSC_SUCCESS)
		status	= dmi32_init(h1);

	if (status == GSC_SUCCESS)
		status	= DMI32_XFER_TEST_MODE__YES(h1);

	if (status == GSC_SUCCESS)
		status	= DMI32_XFER_INTERNAL_CLOCK_SPEED__HIGH(h1);

	// Check the clock received by the receiving board.

	if (status == GSC_SUCCESS)
		status	= DMI32_XFER_CLKIN_TEST__YES(h0, &high);

	// Configure the transmitting board for the LOW rate clock.

	if (status == GSC_SUCCESS)
		status	= dmi32_init(h1);

	if (status == GSC_SUCCESS)
		status	= DMI32_XFER_TEST_MODE__YES(h1);

	if (status == GSC_SUCCESS)
		status	= DMI32_XFER_INTERNAL_CLOCK_SPEED__LOW(h1);

	// Check the clock received by the receiving board.

	if (status == GSC_SUCCESS)
		status	= DMI32_XFER_CLKIN_TEST__YES(h0, &low);

	// Reset both boards.
	dmi32_init(h0);
	dmi32_init(h1);

	// Evaluate the results.

	if (status != GSC_SUCCESS)
	{
		errs	= 1;
		c0[0]	= 0;
	}
	else if (high == DMI32_XFER_CLKIN_TEST_NONE)
	{
		errs	= 1;
		c0[0]	= 0;
		msg		= "high: none";
	}
	else if (high == DMI32_XFER_CLKIN_TEST_BUSY)
	{
		errs	= 1;
		c0[0]	= 0;
		msg		= "high: not done";
	}
	else if (low == DMI32_XFER_CLKIN_TEST_NONE)
	{
		errs	= 1;
		c0[0]	= 0;
		msg		= "low: none";
	}
	else if (low == DMI32_XFER_CLKIN_TEST_BUSY)
	{
		errs	= 1;
		c0[0]	= 0;
		msg		= "low: not done";
	}
	else if (low >= high)
	{
		// This shouldn't be, but we can still use it.
		errs	= 0;
		c0[0]	= low;
		low		= high;
		high	= c0[0];
		msg		= "low >= high";
	}
	else
	{
		errs	= 0;
	}

	if (errs == 0)
		printf("PASS (%ldMHz and %ldMHz)\n", (long) high, (long) low);
	else if (msg)
		printf("FAIL <---  (%s)\n", msg);
	else
		printf("FAIL <---\n");

	return(errs);
}


