/* Application - DMI32 - Any OS */
/* $Workfile: main.c $ */
/* $Revision: 4 $ */
/* $Date: 10/27/07 10:54a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/bbtest/main.c $
 *
 * 4     10/27/07 10:54a Dnewbold
 * Additional porting updates.
 *
 * 3     4/05/07 2:36p Dnewbold
 * Ported to Linux.
 *
 * 2     10/16/06 9:57a Dnewbold
 * Added call to report the host os.
 *
 * 1     8/22/05 4:08p Dnewbold
*/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "main.h"
#include "util_close.h"
#include "util_count.h"
#include "util_host.h"
#include "util_id.h"
#include "util_label.h"
#include "util_open.h"
#include "util_select.h"
#include "util_status.h"



/* variables	*************************************************************/

static	int		_continuous		= 0;
static	int		_def_board_1	= -1;
static	int		_def_board_2	= -1;
static	int		_ignore_errors	= 0;
static	long	_minute_limit	= 0;
static	int		_quit			= 0;
static	int		_test_limit		= -1;

		int		log_data		= FALSE;
		int		verbose			= FALSE;



/*****************************************************************************
*
*	Function:	_parse_args
*
*	Purpose:
*
*		Parse the command line arguments.
*
*	Arguments:
*
*		dev		The handle of the board to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _parse_args(int argc, char** argv)
{
	char	c;
	int		i;
	int		j;
	int		k;

	printf("USAGE: bbtest <-c> <-C> <-l> <-m#> <-n#> <-v> <#1> <#2>\n");
	printf("   -c   Continuous testing, but stop on first error.\n");
	printf("   -C   Continuous testing, disregard errors.\n");
	printf("   -l   Log transfer error data to dmi-XXXX.txt).\n");
	printf("   -m#  Run for at most # minutes (a decimal number).\n");
	printf("   -n#  Repeat test at most # times (a decimal number).\n");
	printf("   -v   Be verbose with some error conditions.\n");
	printf("   #1   The index of board one in the test.\n");
	printf("   #2   The index of board two in the test.\n");
	printf(" NOTE:  Hit Ctrl-C to abort continuous testing.\n");
	printf(" NOTE:  Make sure the DMI32 board-to-board passthrough cable is attached!\n");


	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-c") == 0)
		{
			_continuous	= 1;
			_ignore_errors	= 0;
			continue;
		}

		if (strcmp(argv[i], "-C") == 0)
		{
			_continuous	= 1;
			_ignore_errors	= 1;
			continue;
		}

		if (strcmp(argv[i], "-l") == 0)
		{
			log_data	= TRUE;
			continue;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 'm') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_minute_limit	= k;
				continue;
			}

			_quit	= 1;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			break;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 'n') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_test_limit	= k;
				continue;
			}

			_quit	= 1;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			break;
		}

		if (isdigit(argv[i][0]) == 0)
		{
			_quit	= 1;
			printf("ERROR: invalid arguments: %s\n", argv[i]);
			break;
		}

		j	= sscanf(argv[i], "%d%c", &k, &c);

		if ((j == 1) && (k >= 0))
		{
			if (_def_board_1 < 0)
				_def_board_1	= k;
			else
				_def_board_2	= k;

			continue;
		}

		_quit	= 1;
		printf("ERROR: invalid argument: %s\n", argv[i]);
		break;

	}
}



/*****************************************************************************
*
*	Function:	_perform_tests
*
*	Purpose:
*
*		Perform appropriate DMI32 testing.
*
*	Arguments:
*
*		i0		The index of the first board in the test.
*
*		h0		The handle of the first board in the test.
*
*		i1		The index of the second board in the test.
*
*		h1		The handle of the second board in the test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors encounterred.
*
*****************************************************************************/

static int _perform_tests(int i0, void* h0, int i1, void* h1)
{
	int			clock;
	U32			clock0;
	U32			clock1;
	int			errs	= 0;
	const char*	psz;
	U32			size0;
	U32			size1;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= asctime(stm);
	label_text("Performing Tests");
	printf("%s", psz);
	label_level_inc();

	id_host();
	errs	+= id_software(h0, VERSION);
	errs	+= id_hardware(h0, i0);
	errs	+= id_hardware(h1, i1);

	errs	+= dram_info(i0, h0, &size0);
	errs	+= dram_info(i1, h1, &size1);
	errs	+= clock_test(i0, h0, &clock0, i1, h1);
	errs	+= clock_test(i1, h1, &clock1, i0, h0);
	clock	= clock0 && clock1;
	errs	+= user_io_tests(i0, h0, i1, h1, clock);
	errs	+= user_io_tests(i1, h1, i0, h0, clock);
	errs	+= xfer_tests(i0, h0, i1, h1, clock);
	errs	+= xfer_tests(i1, h1, i0, h0, clock);
	errs	+= irq_tests(i0, h0, i1, h1, clock);
	errs	+= irq_tests(i1, h1, i0, h0, clock);

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Program entry point.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were seen.
*		EXIT_FAILURE	Errors were seen.
*
*****************************************************************************/

int main(int argc, char** argv)
{
	void*	dev_1;
	void*	dev_2;
	int		errs		= 0;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		qty;
	long	secs;
	time_t	test;
	long	tests		= 0;
	time_t	t_limit;

	for (;;)
	{
		test	= time(NULL);
		label_init(28);
		_parse_args(argc, argv);

		if (_quit)
			break;

		printf("\n");
		t_limit	= exec + (_minute_limit * 60);
		errs	= dev_api_status();

		if (errs == 0)
		{
			qty	= dev_board_count();
			errs	= select_2_boards(qty, &_def_board_1, &_def_board_2);
		}

		if ((errs == 0) && (_def_board_1 > _def_board_2))
		{
			errs			= _def_board_1;
			_def_board_1	= _def_board_2;
			_def_board_2	= errs;
			errs			= 0;
		}

		if (errs == 0)
		{
			errs	+= dev_open((U8) _def_board_1, &dev_1);
			errs	+= dev_open((U8) _def_board_2, &dev_2);
		}

		if (errs == 0)
		{
			label_text("Testing Board Indexes");
			printf("%d and %d\n", _def_board_1, _def_board_2);
			errs	= _perform_tests(_def_board_1, dev_1, _def_board_2, dev_2);
		}

		dev_close(_def_board_1, dev_1);
		dev_close(_def_board_2, dev_2);

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf("\nRESULTS: FAIL <---");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= now - test;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= now - exec;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if ((_test_limit > 0) && (_test_limit <= tests))
			break;

		if (_continuous == 0)
			break;

		if ((_ignore_errors == 0) && (errs))
			break;

		if ((_minute_limit) && (now >= t_limit))
			break;
	}

	errs	= errs ? EXIT_FAILURE : EXIT_SUCCESS;
	return(errs);
}


