/* Application - DMI32 - Any OS */
/* $Workfile: user_io.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 10:55a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/bbtest/user_io.c $
 *
 * 3     10/27/07 10:55a Dnewbold
 * Additional porting updates.
 *
 * 2     4/05/07 2:36p Dnewbold
 * Ported to Linux.
 *
 * 1     8/22/05 4:08p Dnewbold
*/

#include <stdio.h>

#include "main.h"
#include "util_label.h"



/*****************************************************************************
*
*	Function:	user_io_tests
*
*	Purpose:
*
*		Verify the User I/O port operation for transfer from the first to the
*		second board.
*
*	Arguments:
*
*		i0		The index of the first board being accessed.
*
*		h0		The handle to the first device under test.
*
*		i1		The index of the second board being accessed.
*
*		h1		The handle to the second device under test.
*
*		clock	Is there a clock on both boards?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int user_io_tests(int i0, void* h0, int i1, void* h1, int clock)
{
	static const struct
	{
		const char*	name;
		U32			which;
	} port[2]	=
	{
		{ "A",	DMI32_WHICH_USER_IO_A	},
		{ "B",	DMI32_WHICH_USER_IO_B	}
	};

	static const U32	sw_dir[2]	=
	{
		DMI32_XFER_START_WORD_DIR_RX,
		DMI32_XFER_START_WORD_DIR_TX
	};

	unsigned long	dir;
	int				errs	= 0;
	int				i;
	unsigned long	input;
	unsigned long	input_last;
	char			label[128];
	U32				mask;
	int				port_no;
	U32				status;
	int				sw_dir0;
	int				sw_dir1;

	sprintf(label, "User I/O (%d -> %d)", i0, i1);
	label_text(label);
	printf("\n");
	label_level_inc();

	for (port_no = 0; port_no <= 1; port_no++)
	{
		label_text(port[port_no].name);

		if (clock == 0)
		{
			printf("SKIPPED (clock unavailable)\n");
			continue;
		}

		for (sw_dir0 = 0; sw_dir0 <= 1; sw_dir0++)
		{
			// Test the source board with both Start Word directions.

			for (sw_dir1 = 0; sw_dir1 <= 1; sw_dir1++)
			{
				// Test the destination board with both Start Word directions.
				// Configure the source board.

				status	= dmi32_init(h0);

				if (status == GSC_SUCCESS)
					status	= DMI32_XFER_TEST_MODE__NO(h0);

				if (status == GSC_SUCCESS)
				{
					status	= DMI32_USER_IO_DIRECTION__OUT(
								h0, port[port_no].which);
				}

				if (status == GSC_SUCCESS)
				{
					status	= DMI32_USER_IO_DIRECTION__IN(
								h0, port[(port_no + 1) & 1].which);
				}

				if (status == GSC_SUCCESS)
					status	= DMI32_XFER_START_WORD_DIR__SET(h0, sw_dir[sw_dir0]);

				if (status == GSC_SUCCESS)
					status	= DMI32_XFER_START_WORD_DIR__GET(h0, &dir);

				if ((status == GSC_SUCCESS) && (dir != sw_dir[sw_dir0]))
				{
					// The desired direction isn't available.
					continue;
				}

				// Configure the destination board.

				if (status == GSC_SUCCESS)
					status	= dmi32_init(h1);

				if (status == GSC_SUCCESS)
					status	= DMI32_XFER_TEST_MODE__YES(h1);

				if (status == GSC_SUCCESS)
				{
					status	= DMI32_USER_IO_DIRECTION__IN(
								h1, port[port_no].which);
				}

				if (status == GSC_SUCCESS)
				{
					status	= DMI32_USER_IO_DIRECTION__OUT(
								h1, port[(port_no + 1) & 1].which);
				}

				if (status == GSC_SUCCESS)
					status	= DMI32_XFER_START_WORD_DIR__SET(h1, sw_dir[sw_dir1]);

				if (status == GSC_SUCCESS)
					status	= DMI32_XFER_START_WORD_DIR__GET(h1, &dir);

				if ((status == GSC_SUCCESS) && (dir != sw_dir[sw_dir1]))
				{
					// The desired direction isn't available.
					continue;
				}

				// Perform the actual test.

				if (status == GSC_SUCCESS)
				{
					status	= DMI32_USER_IO_OUTPUT__SET(
								h0,
								port[port_no].which,
								0x00);
				}

				if (status == GSC_SUCCESS)
				{
					status	= DMI32_USER_IO_INPUT__GET(
								h1,
								port[port_no].which,
								&input_last);
				}

				if (status == GSC_SUCCESS)
				{
					// Do twice as D6 is clear for sure now.
					status	= DMI32_USER_IO_INPUT__GET(
								h1,
								port[port_no].which,
								&input_last);
				}

				for (mask = 0x00; mask <= 0x7F; mask++)
				{
					for (i = 0; (status == GSC_SUCCESS) && (i < 5); i++)
					{
						/*
						*	Do this repeatedly to reduce issues relating to
						*	cable quality.
						*/

						status	= DMI32_USER_IO_OUTPUT__SET(
									h0,
									port[port_no].which,
									mask);
					}

					if (status == GSC_SUCCESS)
					{
						status	= DMI32_USER_IO_INPUT__GET(
									h1,
									port[port_no].which,
									&input);
					}

					if (status != GSC_SUCCESS)
						break;

					if (mask & 0x40)
					{
						if (input != mask)
							status	= GSC_FAILED;
					}
					else
					{
						if (input != input_last)
						{
							status	= GSC_FAILED;
							mask	= input_last;
						}
					}

					if (status != GSC_SUCCESS)
					{
						if (verbose)
						{
							printf(	"(%s: %d %s, %d %s: D6=%s: 0x%lX: got 0x%lX) ",
									port[port_no].name,
									i0,
									sw_dir[sw_dir0] == DMI32_XFER_START_WORD_DIR_RX ? "RX" : "TX",
									i1,
									sw_dir[sw_dir1] == DMI32_XFER_START_WORD_DIR_RX ? "RX" : "TX",
									(mask & 0x40) ? "1" : "0",
									(long) mask,
									(long) input);
						}

						break;
					}

					input_last	= input;
				}

				// Reset each board.
				dmi32_init(h0);
				dmi32_init(h1);

				if (status != GSC_SUCCESS)
					break;
			}

			if (status != GSC_SUCCESS)
				break;
		}

		if (status == GSC_SUCCESS)
		{
			printf("PASS\n");
		}
		else
		{
			errs++;
			printf("FAIL <---\n");
		}
	}

	label_level_dec();
	return(errs);
}


