/* Application - DMI32 - Any OS */
/* $Workfile: xfer.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 10:55a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/bbtest/xfer.c $
 *
 * 3     10/27/07 10:55a Dnewbold
 * Additional porting updates.
 *
 * 2     4/05/07 2:36p Dnewbold
 * Ported to Linux.
 *
 * 1     8/22/05 4:08p Dnewbold
*/

#include <stdio.h>
#include <string.h>
#include <time.h>

#include "main.h"
#include "util_label.h"
#include "util_sleep.h"



/* datatypes	*************************************************************/

typedef struct
{
	U32			which;
	U32			which_alt;
	int			sw_src;
	int			sw_dest;
	U32			offset0_1;	// Transmit board's first offset.
	U32			offset0_2;	// Transmit board's second offset for Multi-Buffer Mode.
	U32			offset1_1;	// Receiving board's first offset.
	U32			offset1_2;	// Receiving board's second offset for Multi-Buffer Mode.
} xfer_t;



/* variables	*************************************************************/

static	U32		_buffer_mb[256L * 1024];



/*****************************************************************************
*
*	Function:	_log_file_setup
*
*	Purpose:
*
*		If enabled, begin the data logging process by opening and
*		initializing the log file.
*
*	Arguments:
*
*		msg		This is a message describing the buffer's meaning.
*
*	Returned:
*
*		NULL	Don't log anything or a file access error.
*		else	A valid FILE*..
*
*****************************************************************************/

static FILE* _log_file_setup(const char* msg)
{
	static	unsigned long	number	= 0;

	FILE*		file;
	char		name[64];
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	sprintf(name, "dmi-%04lu.txt", number);

	if (log_data == FALSE)
	{
		file	= NULL;
	}
	else
	{
		number++;
		file	= fopen(name, "w+b");
	}

	if (file)
	{
		printf("(logging %s ...", name);
		fprintf(file, "\r\nLog Data: %s\r\n\r\n", msg);
		time(&tt);
		stm	= localtime(&tt);
		psz	= asctime(stm);
		fprintf(file, "Time: %s", psz);
	}

	return(file);
}



/*****************************************************************************
*
*	Function:	_xfer_data_log_add
*
*	Purpose:
*
*		Append data to the given buffer for a data transfer that took place.
*
*	Arguments:
*
*		prefix		Text that is to preceed each line.
*
*		handle		The handle of the device to access.
*
*		dest		The destination buffer to receive the text.
*
*		offset		The offset for the data transfer in question.
*
*		size		The size of the transfer in question.
*
*		mode		The Transfer Mode used.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _xfer_data_log_add(	const char*	prefix,
								void*		handle,
								char*		dest,
								U32			offset,
								U32			size,
								U32			mode)
{
	#define	_WIDTH_		15

	U32				bcr;		// Board Control Register
	unsigned long	ics;		// Internal Clock Speed
	U32				status;
	unsigned long	sw;			// Start Word
	unsigned long	swd;		// Start Word Direction
	unsigned long	tm;			// Test Mode
	char			tmp[128];
	unsigned long	uioad;		// User I/O A Direction
	char			value[64];

	for (;;)	// A convenience loop.
	{
		/*
		*	BCR
		*/

		status	= dmi32_reg_read(handle, DMI32_BCR, &bcr);
		if (status != GSC_SUCCESS)	break;
		sprintf(value, "0x%08lX", (long) bcr);
		sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "BCR", value);
		strcat(dest, tmp);

		/*
		*	Test Mode
		*/

		status	= DMI32_XFER_TEST_MODE__GET(handle, &tm);
		if (status != GSC_SUCCESS)	break;
		strcpy(value, (tm == DMI32_XFER_TEST_MODE_YES) ? "Yes" : "No");
		sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "Test Mode", value);
		strcat(dest, tmp);

		/*
		*	Clock Rate
		*/

		status	= DMI32_XFER_CLKIN_TEST__YES(handle, &ics);
		if (status != GSC_SUCCESS)	break;

		if (ics == DMI32_XFER_CLKIN_TEST_BUSY)
			strcpy(value, "BUSY");
		else if (ics == DMI32_XFER_CLKIN_TEST_NONE)
			strcpy(value, "NONE");
		else
			sprintf(value, "%luMHz", (long) ics);

		sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "Clock Speed", value);
		strcat(dest, tmp);

		/*
		*	Transfer Mode
		*/

		strcpy(value, (mode == DMI32_XFER_MODE_SINGLE) ? "Single Buffer" : "Multi-Buffer");
		sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "Transfer Mode", value);
		strcat(dest, tmp);

		/*
		*	Offset
		*/

		sprintf(value, "0x%lX", (long) offset);
		sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "Offset", value);
		strcat(dest, tmp);

		/*
		*	Offset
		*/

		sprintf(value, "0x%lX", (long) size);
		sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "Size", value);
		strcat(dest, tmp);

		/*
		*	Start Word
		*/

		status	= DMI32_XFER_START_WORD__GET(handle, &sw);
		if (status != GSC_SUCCESS)	break;
		sprintf(value, "0x%02lX", (long) sw);
		sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "Start Word", value);
		strcat(dest, tmp);

		/*
		*	Start Word Direction
		*/

		status	= DMI32_XFER_START_WORD_DIR__GET(handle, &swd);
		if (status != GSC_SUCCESS)	break;
		strcpy(value, (swd == DMI32_XFER_START_WORD_DIR_RX) ? "Rx" : "Tx");
		sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "Start Word Dir", value);
		strcat(dest, tmp);

		/*
		*	User I/O Port
		*/

		status	= DMI32_USER_IO_DIRECTION__A_GET(handle, &uioad);
		if (status != GSC_SUCCESS)	break;

		if (swd == DMI32_XFER_START_WORD_DIR_RX)
			strcpy(value, (uioad == DMI32_USER_IO_DIRECTION_IN) ? "A" : "B");
		else
			strcpy(value, (uioad == DMI32_USER_IO_DIRECTION_IN) ? "B" : "A");

		sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "User I/O Port", value);
		strcat(dest, tmp);

		break;
	}

	return(status);

	#undef	_WIDTH_
}



/*****************************************************************************
*
*	Function:	_dram_data_log_add
*
*	Purpose:
*
*		Append data to the given buffer for a data transfer that took place.
*
*	Arguments:
*
*		prefix		Text that is to preceed each line.
*
*		dest		The destination buffer to receive the text.
*
*		offset		The offset for the data transfer in question.
*
*		size		The size of the transfer in question.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _dram_data_log_add(	const char*	prefix,
								char*		dest,
								U32			offset,
								U32			size)
{
	#define	_WIDTH_		7

	char		tmp[128];
	char		value[64];

	/*
	*	Offset
	*/

	sprintf(value, "0x%lX", (long) offset);
	sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "Offset", value);
	strcat(dest, tmp);

	/*
	*	Size
	*/

	sprintf(value, "0x%lX", (long) size);
	sprintf(tmp, "%s: %-*s = %s\r\n", prefix, _WIDTH_, "Size", value);
	strcat(dest, tmp);

	#undef	_WIDTH_
}



/*****************************************************************************
*
*	Function:	_fill_pattern_verify
*
*	Purpose:
*
*		Verify the content of data read from a DMI32 and held in the 1MB
*		buffer. The content should be the value given.
*
*	Arguments:
*
*		msg			This is a message describing the buffer's meaning.
*
*		text		Additional text to appear after the log file header.
*
*		sqew		The amount of sqew to accommodate.
*
*		value		The value expected in the buffer.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _fill_pattern_verify(const char* msg, const char* text, long sqew, U32 value)
{
	long	begin	= sqew;
	long	end		= (sizeof(_buffer_mb) / 4) - sqew;
	FILE*	file	= NULL;
	long	index;
	long	l;
	U32		status	= GSC_SUCCESS;

	for (l = begin; l < end; l++)
	{
		if (_buffer_mb[l] != value)
		{
			status	= GSC_FAILED;
			file	= _log_file_setup(msg);
			break;
		}
	}

	if (file)
	{
		index	= l;
		fprintf(file, text);
		fprintf(file, "pattern = fill (0x%lX)\r\n", (long) value);
		fprintf(file, "status  = 0x%08lX (%lu)\r\n", (long) status, (long) status);
		fprintf(file, "index   = 0x%08lX (%lu)\r\n", index, index);
		fprintf(file, "sqew    = 0x%08lX (%lu)\r\n", sqew, sqew);
		fprintf(file, "begin   = 0x%08lX (%lu)\r\n", begin, begin);
		fprintf(file, "end     = 0x%08lX (%lu)\r\n", end, end);
		fprintf(file, "\r\n");
		fprintf(file, "Index       Actual      Expected    Comments\r\n");
		fprintf(file, "==========  ==========  ==========  =========\r\n");

		for (l = 0; l < (int) (sizeof(_buffer_mb) / 4); l++)
		{
			fprintf(file, "0x%08lX", l);
			fprintf(file, "  0x%08lX", (long) _buffer_mb[l]);
			fprintf(file, "  0x%08lX", (long) value);

			if ((l < begin) || (l >= end))
				fprintf(file, "  SKEW");
			else if (_buffer_mb[l] != value)
				fprintf(file, "  ERROR");

			if (l == index)
				fprintf(file, "  <--- index");

			fprintf(file, "\r\n");
		}

		fclose(file);
		printf("done) ");
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_inc_pattern_verify
*
*	Purpose:
*
*		Verify the content of the data captured by the receiving board and
*		held in the 1MB buffer. The transmission source was an incrementing
*		pattern beginning from zero.
*
*		The TRANSMIT and RECEIVE processes each take a few clock cycled to
*		get going, so there is going to be some sqew as to where in out
*		buffer valid data starts and ends, as well as the content of the
*		first and last valid data values.
*
*	Arguments:
*
*		msg			Text describing the current buffer's content.
*
*		text		Additional text that is to appear after the log header.
*
*		sqew		The amount of sqew to accommodate.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _inc_pattern_verify(const char* msg, const char* text, long sqew)
{
	long	begin	= sqew;
	long	delta	= _buffer_mb[1024] - 1024;
	long	end		= (sizeof(_buffer_mb) / 4) - sqew;
	FILE*	file	= NULL;
	long	index;
	long	l;
	U32		status	= GSC_SUCCESS;

	for (l = begin; l < end; l++)
	{
		if ((long) _buffer_mb[l] != (l + delta))
		{
			status	= GSC_FAILED;
			file	= _log_file_setup(msg);
			break;
		}
	}

	if (file)
	{
		index	= l;
		fprintf(file, text);
		fprintf(file, "pattern         = incrementing (0, 1, 2, ...)\r\n");
		fprintf(file, "status          = 0x%08lX (%lu)\r\n", (long) status, (long) status);
		fprintf(file, "index           = 0x%08lX (%lu)\r\n", index, index);
		fprintf(file, "sqew            = 0x%08lX (%lu)\r\n", sqew, sqew);
		fprintf(file, "begin           = 0x%08lX (%lu)\r\n", begin, begin);
		fprintf(file, "end             = 0x%08lX (%lu)\r\n", end, end);
		fprintf(file, "delta           = 0x%08lX (%ld)\r\n", delta, delta);
		fprintf(file, "data[1024]      = 0x%08lX (%lu) (for delta)\r\n",
								(long) _buffer_mb[1024],
								(long) _buffer_mb[1024]);
		fprintf(file, "\r\n");
		fprintf(file, "Index       Actual      Expected    Comments\r\n");
		fprintf(file, "==========  ==========  ==========  =========\r\n");

		for (l = 0; l < (int) (sizeof(_buffer_mb) / 4); l++)
		{
			fprintf(file, "0x%08lX", l);
			fprintf(file, "  0x%08lX", (long) _buffer_mb[l]);
			fprintf(file, "  0x%08lX", (long) l + delta);

			if ((l < begin) || (l >= end))
				fprintf(file, "  SKEW");
			else if ((long) _buffer_mb[l] != (l + delta))
				fprintf(file, "  ERROR");

			if (l == index)
				fprintf(file, "  <--- index");

			fprintf(file, "\r\n");
		}

		fclose(file);
		printf("done) ");
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_xfer_data_test
*
*	Purpose:
*
*		Verify DRAM transfer from the first to the second board using the
*		parameters provided.
*
*	Arguments:
*
*		i0		The index of the first board being accessed.
*
*		h0		The handle to the first device under test.
*
*		i1		The index of the second board being accessed.
*
*		h1		The handle to the second device under test.
*
*		mode	Either Single Buffer or Multi-Buffer.
*
*		data	Data controlling board configurations.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_data_test(int i0, void* h0, int i1, void* h1, U32 mode, const xfer_t* data)
{
	#define	_SW_	0x4F

	int				errs;
	void*			h[2];
	int				i;
	const char*		msg;
	U32				status;
	char			text[2048];
	char			tmp[128];
	U32				u32;
	unsigned long	ul;
	U32				xfer;

	h[0]	= h0;
    h[1]    = h1;

	for (;;)	// A convenience loop.
	{
		// Configure the board transmitting DRAM data.
		msg	= "src: configure";
		status	= dmi32_init(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_DIRECTION__TX(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__OUT(h0, data->which);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__IN(h0, data->which_alt);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_OFFSET__SET(h0, data->offset0_1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_SIZE__SET(h0, _1M);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_START_WORD__SET(h0, _SW_);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_MODE__SET(h0, mode);
		if (status != GSC_SUCCESS)	break;

		// Initialize the transmitting board's first DRAM buffer.
		msg	= "src: buffer 1: init";
		status	= DMI32_IO_OFFSET__TX_SET(h0, data->offset0_1);
		if (status != GSC_SUCCESS)	break;

		for (u32 = 0; u32 < (int) (sizeof(_buffer_mb) / 4); u32++)
			_buffer_mb[u32]	= u32;

		status	= dmi32_write(h0, _buffer_mb, sizeof(_buffer_mb), &xfer);
		if (status != GSC_SUCCESS)	break;
		if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Initialize the transmitting board's second DRAM buffer.
		msg	= "src: buffer 2: init";
		status	= DMI32_IO_OFFSET__TX_SET(h0, data->offset0_2);
		if (status != GSC_SUCCESS)	break;

		memset(_buffer_mb, 0, sizeof(_buffer_mb));

		status	= dmi32_write(h0, _buffer_mb, sizeof(_buffer_mb), &xfer);
		if (status != GSC_SUCCESS)	break;
		if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Verify the transmitting board's first DRAM buffer.
		msg	= "src: buffer 1: verify";
		status	= DMI32_IO_OFFSET__RX_SET(h0, data->offset0_1);
		if (status != GSC_SUCCESS)	break;

		status	= dmi32_read(h0, _buffer_mb, sizeof(_buffer_mb), &xfer);
		if (status != GSC_SUCCESS)	break;
		if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		strcpy(text, "Description     = verify the source's first DRAM buffer\r\n");
		_dram_data_log_add(	"SRC 1st",
							text,
							data->offset0_1,
							sizeof(_buffer_mb));

		status	= _inc_pattern_verify(msg, text, 0);
		if (status != GSC_SUCCESS)	break;

		// Verify the transmitting board's second DRAM buffer.
		msg	= "src: buffer 2: init";
		status	= DMI32_IO_OFFSET__RX_SET(h0, data->offset0_2);
		if (status != GSC_SUCCESS)	break;

		status	= dmi32_read(h0, _buffer_mb, sizeof(_buffer_mb), &xfer);
		if (status != GSC_SUCCESS)	break;
		if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		strcpy(text, "Description     = verify the source's second DRAM buffer\r\n");
		_dram_data_log_add(	"SRC 2nd",
							text,
							data->offset0_2,
							sizeof(_buffer_mb));

		status	= _fill_pattern_verify(msg, text, 0, 0);
		if (status != GSC_SUCCESS)	break;

		// Configure the board receiving DRAM data.
		msg	= "dest: configure";
		status	= dmi32_init(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_TEST_MODE__YES(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_INTERNAL_CLOCK_SPEED__HIGH(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_DIRECTION__RX(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__IN(h1, data->which);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__OUT(h1, data->which_alt);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_OFFSET__SET(h1, data->offset1_1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_SIZE__SET(h1, _1M);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_START_WORD__SET(h1, _SW_);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_MODE__SET(h1, mode);
		if (status != GSC_SUCCESS)	break;

		// Initialize the receiving board's first DRAM buffer.
		msg	= "dest: buffer 1: init";
		status	= DMI32_IO_OFFSET__TX_SET(h1, data->offset1_1);
		if (status != GSC_SUCCESS)	break;

		memset(_buffer_mb, 0x11, sizeof(_buffer_mb));

		status	= dmi32_write(h1, _buffer_mb, sizeof(_buffer_mb), &xfer);
		if (status != GSC_SUCCESS)	break;
		if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Initialize the receiving board's second DRAM buffer.
		msg	= "dest: buffer 2: init";
		status	= DMI32_IO_OFFSET__TX_SET(h1, data->offset1_2);
		if (status != GSC_SUCCESS)	break;

		memset(_buffer_mb, 0x22, sizeof(_buffer_mb));

		status	= dmi32_write(h1, _buffer_mb, sizeof(_buffer_mb), &xfer);
		if (status != GSC_SUCCESS)	break;
		if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Verify the receiving board's first DRAM buffer.
		msg	= "dest: buffer 1: verify";
		status	= DMI32_IO_OFFSET__RX_SET(h1, data->offset1_1);
		if (status != GSC_SUCCESS)	break;

		status	= dmi32_read(h1, _buffer_mb, sizeof(_buffer_mb), &xfer);
		if (status != GSC_SUCCESS)	break;
		if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		strcpy(text, "Description     = verify the destinations's first DRAM buffer\r\n");
		_dram_data_log_add(	"DST 1st",
							text,
							data->offset1_1,
							sizeof(_buffer_mb));

		status	= _fill_pattern_verify(msg, text, 0, 0x11111111);
		if (status != GSC_SUCCESS)	break;

		// Verify the receiving board's second DRAM buffer.
		msg	= "dest: buffer 2: verify";
		status	= DMI32_IO_OFFSET__RX_SET(h1, data->offset1_2);
		if (status != GSC_SUCCESS)	break;

		status	= dmi32_read(h1, _buffer_mb, sizeof(_buffer_mb), &xfer);
		if (status != GSC_SUCCESS)	break;
		if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		strcpy(text, "Description     = verify the destinations's second DRAM buffer\r\n");
		_dram_data_log_add(	"DST 2nd",
							text,
							data->offset1_2,
							sizeof(_buffer_mb));

		status	= _fill_pattern_verify(msg, text, 0, 0x22222222);
		if (status != GSC_SUCCESS)	break;

		// Configure the Start Word Direction for the transmitting board.
		msg	= "Start Word Direction: Tx";
		status	= DMI32_XFER_START_WORD_DIR__TX(h[data->sw_src]);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_START_WORD_DIR__GET(h[data->sw_src], &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_START_WORD_DIR_TX)	break;	// Not in firmware.

		msg	= "Start Word Direction: Rx";
		// Configure the Start Word Direction for the receiving board.
		status	= DMI32_XFER_START_WORD_DIR__RX(h[data->sw_dest]);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_START_WORD_DIR__GET(h[data->sw_dest], &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_START_WORD_DIR_RX)	break;	// Not in firmware.

		// Prepare for data reception.
		msg	= "dest: Rx prepare";
		status	= DMI32_USER_IO_INPUT__CUR_GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_ENABLE__YES(h[data->sw_dest]);
		if (status != GSC_SUCCESS)	break;

		// Initiate data transmission.

		msg	= "src: Tx start";
		status	= DMI32_XFER_ENABLE__YES(h[data->sw_src]);
		if (status != GSC_SUCCESS)	break;

		for (i = 0;; i++)
		{
			/*
			*	Do this repeatedly to reduce issues relating to
			*	cable quality.
			*/

			status	= DMI32_USER_IO_INPUT__CUR_GET(h[data->sw_dest], &ul);
			if (status != GSC_SUCCESS)	break;
			if (ul == _SW_)			break;

			if (i >= 5)
			{
				// The Start Word wasn't seen by the receiving device.
				status	= GSC_FAILED;
				break;
			}

			status	= DMI32_USER_IO_OUTPUT__CUR_SET(h[data->sw_src], _SW_);
			if (status != GSC_SUCCESS)	break;
		}

		if (status != GSC_SUCCESS)	break;

		if (mode == DMI32_XFER_MODE_MULTI)
		{
			// Update the receiving board's offset.
			msg	= "dest: use 2nd buffer";
			status	= DMI32_XFER_OFFSET__SET(h1, data->offset1_2);
			if (status != GSC_SUCCESS)	break;

			// Update the transmitting board's offset.
			msg	= "src: use 2nd buffer";
			status	= DMI32_XFER_OFFSET__SET(h0, data->offset0_2);
			if (status != GSC_SUCCESS)	break;

			sleep_ms(50);	// Wait for data transfer.

			// Switch both boards to Single Buffer mode and wait for the end.
			msg	= "dest: Single Buffer mode";
			status	= DMI32_XFER_MODE__SINGLE(h1);
			if (status != GSC_SUCCESS)	break;
			sleep_ms(50);	// Let destination board finish first.

			msg	= "src: Single Buffer mode";
			status	= DMI32_XFER_MODE__SINGLE(h0);
			if (status != GSC_SUCCESS)	break;
		}

		sleep_ms(100);	// Wait for data transfer to complete.

		// Read the first destination data buffer.
		msg	= "dest: buffer 1: verify Rx data";
		status	= DMI32_IO_OFFSET__RX_SET(h1, data->offset1_1);
		if (status != GSC_SUCCESS)	break;

		status	= dmi32_read(h1, _buffer_mb, sizeof(_buffer_mb), &xfer);
		if (status != GSC_SUCCESS)	break;
		if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		strcpy(text, "Description     = verify the first DRAM buffer received over the cable\r\n");
		sprintf(tmp, "Transfer From   = %d -> %d\r\n", i0, i1);
		strcat(text, tmp);

		status	= _xfer_data_log_add(	"SRC 1st",
										h0,
										text,
										data->offset0_1,
										sizeof(_buffer_mb),
										mode);
		if (status != GSC_SUCCESS)	break;

		status	= _xfer_data_log_add(	"DST 1st",
										h1,
										text,
										data->offset1_1,
										sizeof(_buffer_mb),
										mode);
		if (status != GSC_SUCCESS)	break;

		status	= _inc_pattern_verify(msg, text, SKEW);
		if (status != GSC_SUCCESS)	break;

		if (mode == DMI32_XFER_MODE_MULTI)
		{
			// Read and validate the second destination data buffer.
			msg	= "dest: buffer 2: verify Rx data";
			status	= DMI32_IO_OFFSET__RX_SET(h1, data->offset1_2);
			if (status != GSC_SUCCESS)	break;

			status	= dmi32_read(h1, _buffer_mb, sizeof(_buffer_mb), &xfer);
			if (status != GSC_SUCCESS)	break;
			if (xfer != sizeof(_buffer_mb))	status	= GSC_FAILED;
			if (status != GSC_SUCCESS)	break;

			strcpy(text, "Description         = verify the second DRAM buffer received over the cable\r\n");
			sprintf(tmp, "Transfer From       = %d -> %d\r\n", i0, i1);
			strcat(text, tmp);

			status	= _xfer_data_log_add(	"SRC 2nd",
											h0,
											text,
											data->offset0_2,
											sizeof(_buffer_mb),
											mode);
			if (status != GSC_SUCCESS)	break;

			status	= _xfer_data_log_add(	"DST 2nd",
											h1,
											text,
											data->offset1_2,
											sizeof(_buffer_mb),
											mode);
			if (status != GSC_SUCCESS)	break;

			status	= _fill_pattern_verify(msg, text, SKEW, 0);
		}

		break;
	}

	if ((verbose) && (status != GSC_SUCCESS))
		printf("(%s) ", msg);

	errs	= (status == GSC_SUCCESS) ? 0 : 1;
	return(errs);
}



/*****************************************************************************
*
*	Function:	_bitblt_pattern_verify
*
*	Purpose:
*
*		Verify that the destination pattern matched the source.
*
*	Arguments:
*
*		msg				Text describing the portion of the test being
*						validated.
*
*		text			Additional text to appear after the log file header.
*
*		src				The source pattern.
*
*		dest			The captured pattern.
*
*		size			The size of the captured patterns.
*
*	Returned:
*
*		GSC_SUCCESS		All went well.
*		else			An appropriate
*
*****************************************************************************/

static U32 _bitblt_pattern_verify(	const char*	msg,
									const char*	text,
									const U32*	src,
									const U32*	dest,
									size_t		size)
{
	FILE*	file	= NULL;
	int		i;
	long	index;
	long	l;
	U32		status	= GSC_SUCCESS;

	for (i = 0; i < (int) size; i++)
	{
		if (src[i] != dest[i])
		{
			status	= GSC_FAILED;
			file	= _log_file_setup(msg);
			break;
		}
	}

	if (file)
	{
		index	= i;
		fprintf(file, text);
		fprintf(file, "pattern = exact image\r\n");
		fprintf(file, "status  = 0x%08lX (%lu)\r\n", (long) status, (long) status);
		fprintf(file, "index   = 0x%08lX (%lu)\r\n", index, index);
		fprintf(file, "\r\n");
		fprintf(file, "Index       Actual      Expected    Comments\r\n");
		fprintf(file, "==========  ==========  ==========  =========\r\n");

		for (l = 0; l < (long) size; l++)
		{
			fprintf(file, "0x%08lX", l);
			fprintf(file, "  0x%08lX", (long) src[l]);
			fprintf(file, "  0x%08lX", (long) dest[l]);

			if (src[l] != dest[l])
				fprintf(file, "  ERROR");

			if (l == index)
				fprintf(file, "  <--- index");

			fprintf(file, "\r\n");
		}

		fclose(file);
		printf("done) ");
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_bit_pattern_verify
*
*	Purpose:
*
*		Verify the bit pattern captured by the receive board.
*
*	Arguments:
*
*		msg				Text describing the portion of the test being
*						validated.
*
*		text			Additional text to appear after the log file header.
*
*		sqew			The amount of sqew to accommodate.
*
*		src				The source pattern.
*
*		s_size			The size of the source pattern.
*
*		dest			The captured pattern.
*
*		d_size			The size of the captured pattern.
*
*	Returned:
*
*		GSC_SUCCESS		All went well.
*		else			An appropriate
*
*****************************************************************************/

static U32 _bit_pattern_verify(	const char*	msg,
								const char*	text,
								long		sqew,
								const U32*	src,
								size_t		s_size,
								const U32*	dest,
								size_t		d_size)
{
	long		begin	= 0;
	long		end;
	FILE*		file	= NULL;
	int			half	= (int) d_size / 2;
	long		index;
	int			i		= 0;
	long		l;
	U32			status	= GSC_SUCCESS;
	const char*	txt;

	for (;;)	// A convenience loop.
	{
		txt	= "size check";

		if ((int) s_size > half)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Locate the starting pattern.
		txt	= "starting position";

		for (begin = sqew; begin < half; begin++)
		{
			if (dest[begin] == src[0])
				break;
		}

		if (begin >= half)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Scan for pattern differences.
		txt	= "pattern check";

		for (i = 0; i < (int) s_size; i++)
		{
			if (dest[begin + i] != src[i])
			{
				status	= GSC_FAILED;
				break;
			}
		}

		break;
	}

	if ((verbose) && (status != GSC_SUCCESS))
		printf("(%s)", txt);

	if (status != GSC_SUCCESS)
	{
		char	buf[1024];

		sprintf(buf, "%s, %s", msg, txt);
		file	= _log_file_setup(buf);
	}

	if (file)
	{
		index	= begin + i;
		end		= begin + s_size;
		fprintf(file, text);
		fprintf(file, "pattern = walking and inverted bits (starts with 0xDEADBEEF)\r\n");
		fprintf(file, "status  = 0x%08lX (%lu)\r\n", (long) status, (long) status);
		fprintf(file, "index   = 0x%08lX (%lu)\r\n", index, index);
		fprintf(file, "sqew    = 0x%08lX (%lu)\r\n", sqew, sqew);
		fprintf(file, "begin   = 0x%08lX (%lu)\r\n", begin, begin);
		fprintf(file, "end     = 0x%08lX (%lu)\r\n", end, end);
		fprintf(file, "d_size  = 0x%08lX (%lu)\r\n", (long) d_size, (long) d_size);
		fprintf(file, "s_size  = 0x%08lX (%lu)\r\n", (long) s_size, (long) s_size);
		fprintf(file, "half    = 0x%08lX (%lu)\r\n", (long) half, (long) half);
		fprintf(file, "\r\n");
		fprintf(file, "Index       Actual      Expected    Comments\r\n");
		fprintf(file, "==========  ==========  ==========  =========\r\n");

		for (l = 0, i = 0; l < (long) d_size; l++)
		{
			fprintf(file, "0x%08lX", l);
			fprintf(file, "  0x%08lX", (long) dest[l]);

			if (l < sqew)
			{
				fprintf(file, "              SKEW");
			}
			else if ((l < begin) || (l >= end))
			{
			}
			else
			{
				fprintf(file, "  0x%08lX", (long) src[i]);

				if (dest[l] != src[i])
					fprintf(file, "  ERROR");

				i++;
			}

			if (l == index)
				fprintf(file, "  <--- index");

			if (l == half)
				fprintf(file, "  <--- half");

			fprintf(file, "\r\n");
		}

		fclose(file);
		printf("done) ");
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_dram_bit_pattern_test
*
*	Purpose:
*
*		Send a varying bit pattern to DRAM, transfer it across the cable and
*		verify what is read back.
*
*	Arguments:
*
*		i0		The index of the first board.
*
*		h0		The handle to the first device under test.
*
*		i1		The index of the second board.
*
*		h1		The handle to the second device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _dram_bit_pattern_test(int i0, void* h0, int i1, void* h1)
{
	#define	SAMPLES	(33L * 32 * 4 + 2)
	#define	BYTES	(SAMPLES * 4)
	#define	LOOPS	4

	int				bits;
	int				errs;
	int				i;
	U32				mask;
	const char*		msg;
	unsigned long	offset;
	U32				pattern;
	int				pos;
	U32				rx[SAMPLES * 3];
	U32				status	= GSC_SUCCESS;
	char			text[2048];
	char			tmp[256];
	U32				tx[SAMPLES];
	unsigned long	ul;
	U32				xfer;

	mask	= 0;
	tx[0]	= 0xDEADBEEF;
	tx[1]	= 0xDEADBEEF;
	i		= 2;

	for (bits = 0; bits <= 32; bits++)
	{
		for (pos = 0; pos <= 31; pos++)
		{
			pattern	= (mask << pos) | (mask >> (32 - pos));
			tx[i]	= pattern;
			i++;
			tx[i]	= pattern;
			i++;
			tx[i]	= ~pattern;
			i++;
			tx[i]	= ~pattern;
			i++;
		}

		mask	= (mask << 1) | 1;
	}

	for (;;)	// A convenience loop.
	{
		// Configure the board transmitting DRAM data.
		msg	= "bits: src: configuration";

		status	= dmi32_init(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_DIRECTION__TX(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__A_OUT(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_OFFSET__SET(h0, 0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_SIZE__SET(h0, BYTES * LOOPS);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_START_WORD_DIR__TX(h0);
		if (status != GSC_SUCCESS)	break;

		// Configure the board receiving DRAM data.
		msg	= "bits: dest: configuration";

		status	= dmi32_init(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_TEST_MODE__YES(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_INTERNAL_CLOCK_SPEED__HIGH(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_DIRECTION__RX(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_USER_IO_DIRECTION__A_IN(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_OFFSET__SET(h1, BYTES);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_SIZE__SET(h1, BYTES * LOOPS);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_START_WORD_DIR__RX(h1);
		if (status != GSC_SUCCESS)	break;

		// Write the data to DRAM on the first board.
		msg	= "bits: src: pattern: write";

		for (i = 0; i < LOOPS; i++)
		{
			status	= dmi32_write(h0, tx, sizeof(tx), &xfer);
			if (status != GSC_SUCCESS)	break;
			if (xfer != sizeof(tx))		status	= GSC_FAILED;
			if (status != GSC_SUCCESS)	break;
		}

		if (status != GSC_SUCCESS)	break;

		// Verify the pattern held in the source board's DRAM.
		msg	= "bits: src: pattern: verify";

		for (i = 0; i < LOOPS; i++)
		{
			status	= DMI32_IO_OFFSET__RX_GET(h0, &offset);
			if (status != GSC_SUCCESS)	break;
			status	= dmi32_read(h0, rx, sizeof(tx), &xfer);
			if (status != GSC_SUCCESS)	break;
			if (xfer != sizeof(tx))		status	= GSC_FAILED;
			if (status != GSC_SUCCESS)	break;

			strcpy(text, "Description  = verify the source DRAM buffer\r\n");
			sprintf(tmp, "Portion      = %d of %d\r\n", i + 1, LOOPS);
			strcat(text, tmp);
			_dram_data_log_add("SRC", text, offset, sizeof(tx));
			status	= _bitblt_pattern_verify(msg, text, tx, rx, sizeof(tx) / 4);
			if (status != GSC_SUCCESS)	break;
		}

		if (status != GSC_SUCCESS)	break;

		// Prepare for data reception.
		msg	= "bits: dest: Rx prepare";
		status	= DMI32_USER_IO_INPUT__CUR_GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_ENABLE__YES(h1);
		if (status != GSC_SUCCESS)	break;

		// Transmit the data.
		msg	= "bits: src: Tx start";
		status	= DMI32_XFER_ENABLE__YES(h0);
		if (status != GSC_SUCCESS)	break;

		for (i = 0;; i++)
		{
			/*
			*	Do this repeatedly to reduce issues relating to
			*	cable quality.
			*/

			status	= DMI32_USER_IO_INPUT__CUR_GET(h1, &ul);
			if (status != GSC_SUCCESS)	break;
			if (ul == DMI32_XFER_START_WORD_DEFAULT)	break;

			if (i >= 5)
			{
				// The Start Word wasn't seen by the receiving device.
				status	= GSC_FAILED;
				break;
			}


			status	= DMI32_USER_IO_OUTPUT__CUR_SET(h0, DMI32_XFER_START_WORD_DEFAULT);
			if (status != GSC_SUCCESS)	break;
		}

		if (status != GSC_SUCCESS)	break;

		// Wait for data transfer.
		sleep_ms(50);

		// Read in the data from the second card.
		msg	= "bits: dest: Rx data verify";
		status	= DMI32_IO_OFFSET__RX_SET(h1, BYTES);
		if (status != GSC_SUCCESS)	break;

		status	= dmi32_read(h1, rx, sizeof(rx), &xfer);
		if (status != GSC_SUCCESS)	break;

		strcpy(text, "Description     = verify the DRAM buffer received over the cable\r\n");
		sprintf(tmp, "Transfer From   = %d -> %d\r\n", i0, i1);
		strcat(text, tmp);

		status	= _xfer_data_log_add(	"SRC",
										h0,
										text,
										0,
										BYTES * LOOPS,
										DMI32_XFER_MODE_SINGLE);
		if (status != GSC_SUCCESS)	break;

		status	= _xfer_data_log_add(	"DST",
										h1,
										text,
										0,
										BYTES * LOOPS,
										DMI32_XFER_MODE_SINGLE);
		if (status != GSC_SUCCESS)	break;

		status	= _bit_pattern_verify(msg, text, SKEW, tx, sizeof(tx) / 4, rx, sizeof(rx) / 4);
		break;
	}

	if ((verbose) && (status != GSC_SUCCESS))
		printf("(%s)", msg);

	dmi32_init(h0);
	dmi32_init(h1);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;
	return(errs);

	#undef	SAMPLES
	#undef	BYTES
}



/*****************************************************************************
*
*	Function:	_xfer_enable_test
*
*	Purpose:
*
*		Verify the Transfer Enable option.
*
*	Arguments:
*
*		h0		The handle to the first device under test.
*
*		h1		The handle to the second device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_enable_test(void* h0, void* h1)
{
	int				err		= 0;
	int				errs	= 0;
	U32				status;
	unsigned long	ul;

	for (;;)	// A convenience loop.
	{
		label_text("Transfer Enable");

		// Initial source device state.
		status	= dmi32_init(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_STATE__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Initial destination device state.
		status	= dmi32_init(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_STATE__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_INACTIVE)		status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Start data transfer.Initial source device state.
		err	= xfer_activity_config(h0, h1);
		if (err)					break;

		// Active source device state.
		status	= DMI32_XFER_STATE__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_ACTIVE)			status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Active destination device state.
		status	= DMI32_XFER_STATE__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_ACTIVE)			status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Stop data reception.
		status	= DMI32_XFER_ENABLE__NO(h1);
		if (status != GSC_SUCCESS)	break;

		// Active source device state.
		status	= DMI32_XFER_STATE__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_ACTIVE)			status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Inactive destination device state.
		status	= DMI32_XFER_STATE__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_INACTIVE)			status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Stop data transmission.
		status	= DMI32_XFER_ENABLE__NO(h0);
		if (status != GSC_SUCCESS)	break;

		// Inactive source device state.
		status	= DMI32_XFER_STATE__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_INACTIVE)			status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Inactive destination device state.
		status	= DMI32_XFER_STATE__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_INACTIVE)			status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// We're done.
		break;
	}

	dmi32_init(h0);
	dmi32_init(h1);
	errs	= (status == GSC_SUCCESS) ? err : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_tx_progress_test
*
*	Purpose:
*
*		Verify the Transmit In Progress option.
*
*	Arguments:
*
*		h0		The handle to the first device under test.
*
*		h1		The handle to the second device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_tx_progress_test(void* h0, void* h1)
{
	int				err		= 0;
	int				errs	= 0;
	U32				status;
	unsigned long	ul;

	for (;;)	// A convenience loop.
	{
		label_text("Transmit In Progress");

		// Initial source device state.
		status	= dmi32_init(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_STATE_TX__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_TX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Initial destination device state.
		status	= dmi32_init(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_STATE_TX__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_TX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Start data transfer.Initial source device state.
		err	= xfer_activity_config(h0, h1);
		if (err)					break;

		// Active source device state.
		status	= DMI32_XFER_STATE_TX__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_TX_ACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Active destination device state.
		status	= DMI32_XFER_STATE_TX__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_TX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Stop data reception.
		status	= DMI32_XFER_ENABLE__NO(h1);
		if (status != GSC_SUCCESS)	break;

		// Active source device state.
		status	= DMI32_XFER_STATE_TX__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_TX_ACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Active destination device state.
		status	= DMI32_XFER_STATE_TX__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_TX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Stop data transmission.
		status	= DMI32_XFER_ENABLE__NO(h0);
		if (status != GSC_SUCCESS)	break;

		// Active source device state.
		status	= DMI32_XFER_STATE_TX__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_TX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Active destination device state.
		status	= DMI32_XFER_STATE_TX__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_TX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// We're done.
		break;
	}

	dmi32_init(h0);
	dmi32_init(h1);
	errs	= (status == GSC_SUCCESS) ? err : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_xfer_rx_progress_test
*
*	Purpose:
*
*		Verify the Receive In Progress option.
*
*	Arguments:
*
*		h0		The handle to the first device under test.
*
*		h1		The handle to the second device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _xfer_rx_progress_test(void* h0, void* h1)
{
	int				err		= 0;
	int				errs	= 0;
	U32				status;
	unsigned long	ul;

	for (;;)	// A convenience loop.
	{
		label_text("Receive In Progress");

		// Initial source device state.
		status	= dmi32_init(h0);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_STATE_RX__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_RX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Initial destination device state.
		status	= dmi32_init(h1);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_STATE_RX__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_RX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Start data transfer.Initial source device state.
		err	= xfer_activity_config(h0, h1);
		if (err)					break;

		// Active source device state.
		status	= DMI32_XFER_STATE_RX__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_RX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Active destination device state.
		status	= DMI32_XFER_STATE_RX__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_RX_ACTIVE)		status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Stop data reception.
		status	= DMI32_XFER_ENABLE__NO(h1);
		if (status != GSC_SUCCESS)	break;
		sleep_ms(50);	// Wait for it to stop.

		// Active source device state.
		status	= DMI32_XFER_STATE_RX__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_RX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Inactive destination device state.
		status	= DMI32_XFER_STATE_RX__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_RX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Stop data transmission.
		status	= DMI32_XFER_ENABLE__NO(h0);
		if (status != GSC_SUCCESS)	break;

		// Active source device state.
		status	= DMI32_XFER_STATE_RX__GET(h0, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_RX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// Inactive destination device state.
		status	= DMI32_XFER_STATE_RX__GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;
		if (ul != DMI32_XFER_STATE_RX_INACTIVE)	status	= GSC_FAILED;
		if (status != GSC_SUCCESS)	break;

		// We're done.
		break;
	}

	dmi32_init(h0);
	dmi32_init(h1);
	errs	= (status == GSC_SUCCESS) ? err : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	xfer_activity_config
*
*	Purpose:
*
*		Configure the boards for activity tests.
*
*	Arguments:
*
*		h0		The handle to the first device under test.
*
*		h1		The handle to the second device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int xfer_activity_config(void* h0, void* h1)
{
	int				errs;
	void*			h[2];
	int				i;
	const char*		msg;
	U32				status;
	unsigned long	ul;

	h[0]	= h0;
    h[1]    = h1;

	for (;;)	// A convenience loop.
	{
		// Configure the board transmitting DRAM data.
		msg		= "0: dmi32_init";
		status	= dmi32_init(h0);
		if (status != GSC_SUCCESS)	break;

		msg		= "0: Xfer Tx";
		status	= DMI32_XFER_DIRECTION__TX(h0);
		if (status != GSC_SUCCESS)	break;

		msg		= "0: User I/O A Out";
		status	= DMI32_USER_IO_DIRECTION__A_OUT(h0);
		if (status != GSC_SUCCESS)	break;

		msg		= "0: Xfer Offset 0";
		status	= DMI32_XFER_OFFSET__SET(h0, 0);
		if (status != GSC_SUCCESS)	break;

		msg		= "0: Xfer Size 1MB";
		status	= DMI32_XFER_SIZE__SET(h0, _1M);
		if (status != GSC_SUCCESS)	break;

		msg		= "0: Xfer Start Word";
		status	= DMI32_XFER_START_WORD__SET(h0, _SW_);
		if (status != GSC_SUCCESS)	break;

		msg		= "0: Xfer Mode multi-Buffer";
		status	= DMI32_XFER_MODE__MULTI(h0);
		if (status != GSC_SUCCESS)	break;

		msg		= "0: Xfer Start Word Dir Tx";
		status	= DMI32_XFER_START_WORD_DIR__TX(h0);
		if (status != GSC_SUCCESS)	break;

		// Configure the board receiving DRAM data.
		msg		= "1: dmi32_init";
		status	= dmi32_init(h1);
		if (status != GSC_SUCCESS)	break;

		msg		= "1: Test Mode";
		status	= DMI32_XFER_TEST_MODE__YES(h1);
		if (status != GSC_SUCCESS)	break;

		msg		= "1: Xfer Rx";
		status	= DMI32_XFER_DIRECTION__RX(h1);
		if (status != GSC_SUCCESS)	break;

		msg		= "1: User I/O A In";
		status	= DMI32_USER_IO_DIRECTION__A_IN(h1);
		if (status != GSC_SUCCESS)	break;

		msg		= "1: Xfer Offset 0";
		status	= DMI32_XFER_OFFSET__SET(h1, 0);
		if (status != GSC_SUCCESS)	break;

		msg		= "1: Xfer Size 1MB";
		status	= DMI32_XFER_SIZE__SET(h1, _1M);
		if (status != GSC_SUCCESS)	break;

		msg		= "1: Xfer Start Word";
		status	= DMI32_XFER_START_WORD__SET(h1, _SW_);
		if (status != GSC_SUCCESS)	break;

		msg		= "1: Xfer Mode multi-Buffer";
		status	= DMI32_XFER_MODE__MULTI(h1);
		if (status != GSC_SUCCESS)	break;

		msg		= "1: Xfer Start Word Dir Rx";
		status	= DMI32_XFER_START_WORD_DIR__RX(h1);
		if (status != GSC_SUCCESS)	break;

		// Prepare for data reception.
		msg		= "1: Rx setup";
		status	= DMI32_USER_IO_INPUT__CUR_GET(h1, &ul);
		if (status != GSC_SUCCESS)	break;

		status	= DMI32_XFER_ENABLE__YES(h1);
		if (status != GSC_SUCCESS)	break;

		// Initiate data transmission.
		msg		= "0: Tx start";
		status	= DMI32_XFER_ENABLE__YES(h0);
		if (status != GSC_SUCCESS)	break;

		for (i = 0;; i++)
		{
			/*
			*	Do this repeatedly to reduce issues relating to
			*	cable quality.
			*/

			status	= DMI32_USER_IO_INPUT__CUR_GET(h1, &ul);
			if (status != GSC_SUCCESS)	break;
			if (ul == _SW_)				break;

			if (i >= 5)
			{
				// The Start Word wasn't seen by the receiving device.
				status	= GSC_FAILED;
				break;
			}

			status	= DMI32_USER_IO_OUTPUT__A_SET(h0, _SW_);
			if (status != GSC_SUCCESS)	break;
		}

		// Data is now moving, so we're done.
		break;
	}

	if ((verbose) && (status != GSC_SUCCESS))
		printf("(%s) ", msg);

	errs	= (status == GSC_SUCCESS) ? 0 : 1;
	return(errs);
}



/*****************************************************************************
*
*	Function:	xfer_tests
*
*	Purpose:
*
*		Verify DRAM transfer from the first to the second board.
*
*	Arguments:
*
*		i0		The index of the first board being accessed.
*
*		h0		The handle to the first device under test.
*
*		i1		The index of the second board being accessed.
*
*		h1		The handle to the second device under test.
*
*		clock	Is there a clock on both boards?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int xfer_tests(int i0, void* h0, int i1, void* h1, int clock)
{
	static const xfer_t	list[]	=
	{
		{
			/* which		*/	DMI32_WHICH_USER_IO_A,
			/* which_alt	*/	DMI32_WHICH_USER_IO_B,
			/* sw_src		*/	0,
			/* sw_dest		*/	1,
			/* offset0_1	*/	_1K,
			/* offset0_2	*/	_256M,
			/* offset1_1	*/	_500K,
			/* offset1_2	*/	_500M
		},
		{
			/* which		*/	DMI32_WHICH_USER_IO_A,
			/* which_alt	*/	DMI32_WHICH_USER_IO_B,
			/* sw_src		*/	1,
			/* sw_dest		*/	0,
			/* offset0_1	*/	_500K,
			/* offset0_2	*/	_256M,
			/* offset1_1	*/	_1K,
			/* offset1_2	*/	_500M
		},
		{
			/* which		*/	DMI32_WHICH_USER_IO_B,
			/* which_alt	*/	DMI32_WHICH_USER_IO_A,
			/* sw_src		*/	0,
			/* sw_dest		*/	1,
			/* offset0_1	*/	_1K,
			/* offset0_2	*/	_256M,
			/* offset1_1	*/	_500K,
			/* offset1_2	*/	_500M
		},
		{
			/* which		*/	DMI32_WHICH_USER_IO_B,
			/* which_alt	*/	DMI32_WHICH_USER_IO_A,
			/* sw_src		*/	1,
			/* sw_dest		*/	0,
			/* offset0_1	*/	_500K,
			/* offset0_2	*/	_256M,
			/* offset1_1	*/	_1K,
			/* offset1_2	*/	_500M
		},
		{ 0, 0, 0, 0, 0 }
	};

	static const struct
	{
		const char*	name;
		U32			mode;
	} mode[2] =
	{
		/* name				mode					*/
		{ "Single Buffer",	DMI32_XFER_MODE_SINGLE	},
		{ "Multi-Buffer",	DMI32_XFER_MODE_MULTI	}
	};

	int		err		= 0;
	int		errs	= 0;
	int		i;
	int		j;
	char	label[128];

	sprintf(label, "Data Transfer (%d -> %d)", i0, i1);
	label_text(label);
	printf("\n");

	if (clock == 0)
	{
		printf("SKIPPED (clock unavailable)\n");
	}
	else
	{
		label_level_inc();

		for (j = 0; j <= 1; j++)
		{
			label_text(mode[j].name);
			err	= 0;

			for (i = 0; list[i].offset0_1; i++)
			{
				if (err == 0)
					err	+= _xfer_data_test(i0, h0, i1, h1, mode[j].mode, &list[i]);
			}

			if (err == 0)
				err	= _dram_bit_pattern_test(i0, h0, i1, h1);

			errs	+= err;

			if (err)
				printf("FAIL <---\n");
			else
				printf("PASS\n");
		}

		errs	+= _xfer_enable_test(h0, h1);
		errs	+= _xfer_tx_progress_test(h0, h1);
		errs	+= _xfer_rx_progress_test(h0, h1);
		label_level_dec();
	}

	return(errs);
}


