/* Application - DMI32 - Any OS */
/* $Workfile: io.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 11:01a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/sbtest/io.c $
 *
 * 3     10/27/07 11:01a Dnewbold
 * Additional porting updates.
 *
 * 2     4/05/07 2:56p Dnewbold
 * Ported to Linux.
 *
 * 1     8/22/05 4:10p Dnewbold
*/

#include <stdio.h>

#include "main.h"
#include "util_label.h"
#include "util_sleep.h"



/* typedefs	*****************************************************************/

typedef struct
{
	unsigned long	set;
	unsigned long	get;
} set_get_t;

typedef struct
{
	const char*			name;
	U32					parm;
	const set_get_t*	list;
	size_t				size;
} set_get_list_t;



/* variables	*************************************************************/

static	U32	_io_callback_status;



/*****************************************************************************
*
*	Function:	_io_parm_list_test
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	Which I/O side are these tests for?
*
*		set		The data set to act on.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_parm_list_test(	void*					handle,
								U32						which,
								const set_get_list_t*	set,
								int						verbose)
{
	int				errs	= 0;
	unsigned long	get;
	int				i;
	char			name[128];
	U32				status;

	sprintf(name, "%s %s", (which & DMI32_WHICH_TX) ? "Tx" : "Rx", set->name);

	if (verbose)
		label_text(name);

	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
		status	= DMI32_MISC_STRICT_DRAM_LIMITS__NO(handle);

	for (i = 0; i < (int) set->size; i++)
	{
		status	= dmi32_config(	handle,
								set->parm,
								which,
								set->list[i].set,
								&get);

		if ((status == GSC_SUCCESS) && (get != set->list[i].get))
			status	= GSC_FAILED;

		if (status != GSC_SUCCESS)
			break;
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_buf_size_test
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	Which I/O side are these tests for?
*
*		set		The data set to act on.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_buf_size_test(	void*					handle,
								U32						which,
								const set_get_list_t*	set,
								int						verbose)
{
	int				errs	= 0;
	unsigned long	get;
	int				i;
	char			name[128];
	U32				status;

	sprintf(name, "%s %s", (which & DMI32_WHICH_TX) ? "Tx" : "Rx", set->name);

	if (verbose)
		label_text(name);

	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
		status	= DMI32_MISC_STRICT_DRAM_LIMITS__NO(handle);

	for (i = 0; i < (int) set->size; i++)
	{
		status	= dmi32_config(	handle,
								set->parm,
								which,
								set->list[i].set,
								&get);

		if ((status == GSC_SUCCESS) && (get < set->list[i].get))
			status	= GSC_FAILED;

		if (status != GSC_SUCCESS)
			break;
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_abort_tests
*
*	Purpose:
*
*		Perform I/O Abort tests. This test doesn't actually verify that the
*		abort request actually works. This test simply verifies that the API
*		service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_abort_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set						get						*/
		{ DMI32_IO_ABORT_NO,		DMI32_IO_ABORT_NO		},
		{ GSC_NO_CHANGE,			DMI32_IO_ABORT_NO		},
		{ GSC_DEFAULT,				DMI32_IO_ABORT_DEFAULT	},
		{ DMI32_IO_ABORT_YES,		DMI32_IO_ABORT_NO		},	// It auto clears.
		{ GSC_NO_CHANGE,			DMI32_IO_ABORT_NO		},	// It auto clears.
		{ GSC_DEFAULT,				DMI32_IO_ABORT_DEFAULT	},
		{ DMI32_IO_ABORT_DEFAULT,	DMI32_IO_ABORT_DEFAULT	},
		{ GSC_NO_CHANGE,			DMI32_IO_ABORT_DEFAULT	},
		{ GSC_DEFAULT,				DMI32_IO_ABORT_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Abort",
		/* parm	*/	DMI32_IO_ABORT,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, DMI32_WHICH_TX, &set, verbose);
	errs	+= _io_parm_list_test(handle, DMI32_WHICH_RX, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_buffer_pointer_test
*
*	Purpose:
*
*		Perform a I/O Buffer Pointer test.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	transmitter or receiver side?
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_buffer_pointer_test(void* handle, U32 which, int verbose)
{
	int				errs	= 0;
	unsigned long	get;
	int				i;
	char			label[128];
	U8*				ptr;
	U32				status;

	sprintf(label,
			"%s Buffer pointer",
			(which & DMI32_WHICH_TX) ? "Tx" : "Rx");

	if (verbose)
		label_text(label);

	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
	{
		status	= dmi32_config(	handle,
								DMI32_IO_BUFFER_SIZE,
								which,
								0,
								&get);

		if ((status == GSC_SUCCESS) && (get != 0))
			status	= GSC_FAILED;	// The size should match.
	}

	if (status == GSC_SUCCESS)
	{
		status	= dmi32_config(	handle,
								DMI32_IO_BUFFER_POINTER,
								which,
								0,
								&get);

		if ((status == GSC_SUCCESS) && (get))
			status	= GSC_FAILED;	// The pointer should be NULL.
	}

	if (status == GSC_SUCCESS)
	{
		status	= dmi32_config(	handle,
								DMI32_IO_BUFFER_SIZE,
								which,
								1024,
								&get);

		if ((status == GSC_SUCCESS) && (get < 1024))
			status	= GSC_FAILED;	// The size should not be less.
	}

	if (status == GSC_SUCCESS)
	{
		status	= dmi32_config(	handle,
								DMI32_IO_BUFFER_POINTER,
								which,
								0,
								&get);

		if ((status == GSC_SUCCESS) && (get == (U32) NULL))
			status	= GSC_FAILED;	// The pointer should NOT be NULL.
	}

	if (status == GSC_SUCCESS)
	{
		ptr	= (U8*) get;

		for (i = 0; i < (int) 1024; i++)
		{
			ptr[0]	= (U8) i;

			if (ptr[0] != (U8) i)
			{
				status	= GSC_FAILED;
				break;
			}
		}
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_buffer_pointer_tests
*
*	Purpose:
*
*		Perform I/O Buffer Pointer tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_buffer_pointer_tests(void* handle, int verbose)
{
	int	errs	= 0;

	errs	+= _io_buffer_pointer_test(handle, DMI32_WHICH_TX, verbose);
	errs	+= _io_buffer_pointer_test(handle, DMI32_WHICH_RX, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_buffer_size_tests
*
*	Purpose:
*
*		Perform I/O Buffer Size tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_buffer_size_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set							get								*/
		{ DMI32_IO_BUFFER_SIZE_DEFAULT,	DMI32_IO_BUFFER_SIZE_DEFAULT	},
		{ GSC_NO_CHANGE,				DMI32_IO_BUFFER_SIZE_DEFAULT	},
		{ GSC_DEFAULT,					DMI32_IO_BUFFER_SIZE_DEFAULT	},
		{ 1024,							1024							},
		{ GSC_NO_CHANGE,				1024							},
		{ GSC_DEFAULT,					DMI32_IO_BUFFER_SIZE_DEFAULT	},
		{ 2048,							2048							},
		{ GSC_NO_CHANGE,				2048							},
		{ GSC_DEFAULT,					DMI32_IO_BUFFER_SIZE_DEFAULT	},
		{ 0,							0								},
		{ GSC_NO_CHANGE,				DMI32_IO_BUFFER_SIZE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Buffer Size",
		/* parm	*/	DMI32_IO_BUFFER_SIZE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_buf_size_test(handle, DMI32_WHICH_TX, &set, verbose);
	errs	+= _io_buf_size_test(handle, DMI32_WHICH_RX, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_callback_arg_tests
*
*	Purpose:
*
*		Perform I/O Callback Argument tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_callback_arg_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ 1024,								1024							},
		{ GSC_NO_CHANGE,					1024							},
		{ GSC_DEFAULT,						DMI32_IO_CALLBACK_ARG_DEFAULT	},
		{ 0,								0								},
		{ GSC_NO_CHANGE,					0								},
		{ GSC_DEFAULT,						DMI32_IO_CALLBACK_ARG_DEFAULT	},
		{ 2048,								2048							},
		{ GSC_NO_CHANGE,					2048							},
		{ GSC_DEFAULT,						DMI32_IO_CALLBACK_ARG_DEFAULT	},
		{ DMI32_IO_CALLBACK_ARG_DEFAULT,	DMI32_IO_CALLBACK_ARG_DEFAULT	},
		{ GSC_NO_CHANGE,					DMI32_IO_CALLBACK_ARG_DEFAULT	},
		{ GSC_DEFAULT,						DMI32_IO_CALLBACK_ARG_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Callback Argument",
		/* parm	*/	DMI32_IO_CALLBACK_ARG,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, DMI32_WHICH_TX, &set, verbose);
	errs	+= _io_parm_list_test(handle, DMI32_WHICH_RX, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_callback_func_tests
*
*	Purpose:
*
*		Perform I/O Callback Function tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static void _callback(U32 arg1, U32 arg2, U32 arg3) {}

static int _io_callback_func_tests(void* handle, int verbose)
{
	static set_get_t	list[]	=
	{
		/* set								get								*/
		{ (unsigned long) _callback,		(unsigned long) _callback		},
		{ GSC_NO_CHANGE,					(unsigned long) _callback		},
		{ GSC_DEFAULT,						DMI32_IO_CALLBACK_FUNC_DEFAULT	},
		{ (unsigned long) NULL,				(unsigned long) NULL			},
		{ GSC_NO_CHANGE,					(unsigned long) NULL			},
		{ GSC_DEFAULT,						DMI32_IO_CALLBACK_FUNC_DEFAULT	},
		{ DMI32_IO_CALLBACK_FUNC_DEFAULT,	DMI32_IO_CALLBACK_FUNC_DEFAULT	},
		{ GSC_NO_CHANGE,					DMI32_IO_CALLBACK_FUNC_DEFAULT	},
		{ GSC_DEFAULT,						DMI32_IO_CALLBACK_FUNC_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Callback Function",
		/* parm	*/	DMI32_IO_CALLBACK_ARG,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, DMI32_WHICH_TX, &set, verbose);
	errs	+= _io_parm_list_test(handle, DMI32_WHICH_RX, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_mode_tests
*
*	Purpose:
*
*		Perform I/O Mode tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_mode_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set						get						*/
		{ DMI32_IO_MODE_PIO,		DMI32_IO_MODE_PIO		},
		{ GSC_NO_CHANGE,			DMI32_IO_MODE_PIO		},
		{ GSC_DEFAULT,				DMI32_IO_MODE_DEFAULT	},
		{ DMI32_IO_MODE_DMA,		DMI32_IO_MODE_DMA		},
		{ GSC_NO_CHANGE,			DMI32_IO_MODE_DMA		},
		{ GSC_DEFAULT,				DMI32_IO_MODE_DEFAULT	},
		{ DMI32_IO_MODE_DEFAULT,	DMI32_IO_MODE_DEFAULT	},
		{ GSC_NO_CHANGE,			DMI32_IO_MODE_DEFAULT	},
		{ GSC_DEFAULT,				DMI32_IO_MODE_DEFAULT	}
	};

	static const set_get_list_t	set	=
{
		/* name	*/	"Mode",
		/* parm	*/	DMI32_IO_MODE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, DMI32_WHICH_TX, &set, verbose);
	errs	+= _io_parm_list_test(handle, DMI32_WHICH_RX, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_offset_tests
*
*	Purpose:
*
*		Perform I/O Offset tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_offset_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set						get						*/
		{ DMI32_IO_OFFSET_MIN,		DMI32_IO_OFFSET_MIN		},
		{ GSC_NO_CHANGE,			DMI32_IO_OFFSET_MIN		},
		{ GSC_DEFAULT,				DMI32_IO_OFFSET_DEFAULT	},
		{ DMI32_IO_OFFSET_MAX,		DMI32_IO_OFFSET_MAX		},
		{ GSC_NO_CHANGE,			DMI32_IO_OFFSET_MAX		},
		{ GSC_DEFAULT,				DMI32_IO_OFFSET_DEFAULT	},
		{ DMI32_IO_OFFSET_MASK,		DMI32_IO_OFFSET_MASK	},
		{ GSC_NO_CHANGE,			DMI32_IO_OFFSET_MASK	},
		{ GSC_DEFAULT,				DMI32_IO_OFFSET_DEFAULT	},
		{ DMI32_IO_OFFSET_DEFAULT,	DMI32_IO_OFFSET_DEFAULT	},
		{ GSC_NO_CHANGE,			DMI32_IO_OFFSET_DEFAULT	},
		{ GSC_DEFAULT,				DMI32_IO_OFFSET_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Offset",
		/* parm	*/	DMI32_IO_OFFSET,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, DMI32_WHICH_TX, &set, verbose);
	errs	+= _io_parm_list_test(handle, DMI32_WHICH_RX, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_overlap_enable_tests
*
*	Purpose:
*
*		Perform I/O Overlap Enable tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_overlap_enable_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ DMI32_IO_OVERLAP_ENABLE_NO,		DMI32_IO_OVERLAP_ENABLE_NO		},
		{ GSC_NO_CHANGE,					DMI32_IO_OVERLAP_ENABLE_NO		},
		{ GSC_DEFAULT,						DMI32_IO_OVERLAP_ENABLE_DEFAULT	},
		{ DMI32_IO_OVERLAP_ENABLE_YES,		DMI32_IO_OVERLAP_ENABLE_YES		},
		{ GSC_NO_CHANGE,					DMI32_IO_OVERLAP_ENABLE_YES		},
		{ GSC_DEFAULT,						DMI32_IO_OVERLAP_ENABLE_DEFAULT	},
		{ DMI32_IO_OVERLAP_ENABLE_DEFAULT,	DMI32_IO_OVERLAP_ENABLE_DEFAULT	},
		{ GSC_NO_CHANGE,					DMI32_IO_OVERLAP_ENABLE_DEFAULT	},
		{ GSC_DEFAULT,						DMI32_IO_OVERLAP_ENABLE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Overlap Enable",
		/* parm	*/	DMI32_IO_OVERLAP_ENABLE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, DMI32_WHICH_TX, &set, verbose);
	errs	+= _io_parm_list_test(handle, DMI32_WHICH_RX, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_status_which_tests
*
*	Purpose:
*
*		Perform I/O Status tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_status_which_tests(void* handle, U32 which, int verbose)
{
	U32				buffer[1024];
	U32				count;
	int				errs	= 0;
	unsigned long	get;
	char			label[128];
	U32				status;
	U32				xfer;

	sprintf(label, "%s I/O Status", (which & DMI32_WHICH_TX) ? "Tx" : "Rx");

	if (verbose)
		label_text(label);

	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_OFFSET__SET(handle, which, 0);

	if (status != GSC_SUCCESS)
		;
	else if (which & DMI32_WHICH_TX)
		status	= dmi32_write(handle, buffer, sizeof(buffer), &xfer);
	else
		status	= dmi32_read(handle, buffer, sizeof(buffer), &xfer);

	if (status == GSC_SUCCESS)
	{
		status	= dmi32_config(	handle,
								DMI32_IO_STATUS,
								which,
								GSC_NO_CHANGE,
								&get);
	}

	if (status == GSC_SUCCESS)
	{
		if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_SUCCESS)
		{
			// This is what is expected.
			count	= get & GSC_IO_STATUS_COUNT_MASK;

			if (count != sizeof(buffer))
			{
				printf(	"(xfer: expected %ld, was %ld) ",
						(long) sizeof(buffer),
						(long) count);
				status	= GSC_FAILED;
			}
		}
		else if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ACTIVE)
		{
			printf("(still active) ");
			status	= GSC_FAILED;
		}
		else if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ERROR)
		{
			printf("(unexpected error) ");
			status	= GSC_FAILED;
		}
		else if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ABORTED)
		{
			printf("(unexpected abort) ");
			status	= GSC_FAILED;
		}
		else if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_TIMEOUT)
		{
			printf("(unexpected timeout) ");
			status	= GSC_FAILED;
		}
		else
		{
			printf("(unknown status 0x%lX) ", (long) get);
			status	= GSC_FAILED;
		}
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (verbose)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_status_tests
*
*	Purpose:
*
*		Perform I/O Status tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_status_tests(void* handle, int verbose)
{
	int	errs	= 0;

	errs	+= _io_status_which_tests(handle, DMI32_WHICH_TX, verbose);
	errs	+= _io_status_which_tests(handle, DMI32_WHICH_RX, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_timeout_tests
*
*	Purpose:
*
*		Perform I/O Timeout tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_timeout_tests(void* handle, int verbose)
{
	static const set_get_t	list[]	=
	{
		/* set						get							*/
		{ DMI32_IO_TIMEOUT_NO_WAIT,	DMI32_IO_TIMEOUT_NO_WAIT	},
		{ GSC_NO_CHANGE,			DMI32_IO_TIMEOUT_NO_WAIT	},
		{ GSC_DEFAULT,				DMI32_IO_TIMEOUT_DEFAULT	},
		{ DMI32_IO_TIMEOUT_MAX,		DMI32_IO_TIMEOUT_MAX		},
		{ GSC_NO_CHANGE,			DMI32_IO_TIMEOUT_MAX		},
		{ GSC_DEFAULT,				DMI32_IO_TIMEOUT_DEFAULT	},
		{ DMI32_IO_TIMEOUT_DEFAULT,	DMI32_IO_TIMEOUT_DEFAULT	},
		{ GSC_NO_CHANGE,			DMI32_IO_TIMEOUT_DEFAULT	},
		{ GSC_DEFAULT,				DMI32_IO_TIMEOUT_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Timeout",
		/* parm	*/	DMI32_IO_TIMEOUT,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, DMI32_WHICH_TX, &set, verbose);
	errs	+= _io_parm_list_test(handle, DMI32_WHICH_RX, &set, verbose);
	return(errs);
}



/*****************************************************************************
*
*	Function:	io_config_tests
*
*	Purpose:
*
*		Perform I/O configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int io_config_tests(void* handle)
{
	int	errs	= 0;
	int	verbose	= 1;

	label_text("I/O Configuration");

	if (verbose)
		printf("\n");

	label_level_inc();
	errs	+= _io_abort_tests(handle, verbose);
	errs	+= _io_buffer_pointer_tests(handle, verbose);
	errs	+= _io_buffer_size_tests(handle, verbose);
	errs	+= _io_callback_arg_tests(handle, verbose);
	errs	+= _io_callback_func_tests(handle, verbose);
	errs	+= _io_mode_tests(handle, verbose);
	errs	+= _io_offset_tests(handle, verbose);
	errs	+= _io_overlap_enable_tests(handle, verbose);
	errs	+= _io_status_tests(handle, verbose);
	errs	+= _io_timeout_tests(handle, verbose);
	label_level_dec();

	if (verbose == 0)
	{
		if (errs)
			printf("FAIL <---\n");
		else
			printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_callback
*
*	Purpose:
*
*		Respond to an I/O completion callback.
*
*	Arguments:
*
*		arg1	This is a device handle.
*
*		arg2	A user supplied argument. We ignore this.
*
*		arg3	The status of the transfer.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static void _io_callback(U32 arg1, U32 arg2, U32 arg3)
{
	_io_callback_status	= 1;
}



/*****************************************************************************
*
*	Function:	io_notify_tests
*
*	Purpose:
*
*		Perform I/O notification tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int io_notify_tests(void* handle)
{
	U32				buffer[1024];
	int				errs;
	int				i;
	U32				status;
	unsigned long	ul;
	U32				xfer;

	_io_callback_status	= 0;
	label_text("I/O Complete Notification");
	status	= dmi32_init(handle);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_CALLBACK_ARG__TX_SET(handle, 0);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_CALLBACK_FUNC__TX_SET(handle, (U32) _io_callback);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_MODE__TX_PIO(handle);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_TIMEOUT__TX_SET(handle, 1);

	if (status == GSC_SUCCESS)
		status	= dmi32_write(handle, buffer, sizeof(buffer), &xfer);

	if (status == GSC_SUCCESS)
		sleep_ms(250);

	if (status == GSC_SUCCESS)
		status	= DMI32_IO_STATUS__TX_GET(handle, &ul);

	if ((status == GSC_SUCCESS) && (_io_callback_status == 0))
	{
		printf("(no callback) ");
		status	= GSC_FAILED;
	}

	// 1. Test status from DMI32_IO_STATUS__TX_GET().
	// 2. Test status passed to the callback.

	for (i = 1; (status == GSC_SUCCESS) && (i <= 2); i++)
	{
		if ((status == GSC_SUCCESS) &&
			((ul & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ACTIVE))
		{
			printf("(%d. still active) ", i);
			status	= GSC_FAILED;
		}

		if ((status == GSC_SUCCESS) &&
			((ul & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_TIMEOUT))
		{
			printf("(%d. timeout) ", i);
			status	= GSC_FAILED;
		}

		if ((status == GSC_SUCCESS) &&
			((ul & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ERROR))
		{
			printf("(%d. transfer error) ", i);
			status	= GSC_FAILED;
		}

		if ((status == GSC_SUCCESS) &&
			((ul & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ABORTED))
		{
			printf("(%d. transfer aborted) ", i);
			status	= GSC_FAILED;
		}
		if ((status == GSC_SUCCESS) &&
			((ul & GSC_IO_STATUS_MASK) != GSC_IO_STATUS_SUCCESS))
		{
			printf("(%d. unsuccessful) ", i);
			status	= GSC_FAILED;
		}

		ul	= _io_callback_status;
	}

	dmi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}


