/* Application - DMI32 - Any OS */
/* $Workfile: tx.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 11:04a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/tx/tx.c $
 *
 * 3     10/27/07 11:04a Dnewbold
 * Additional porting updates.
 *
 * 2     4/05/07 3:08p Dnewbold
 * Ported to Linux.
 *
 * 1     8/22/05 4:11p Dnewbold
*/

#include <stdio.h>

#include "main.h"
#include "util_label.h"
#include "util_sleep.h"



/* variables	*************************************************************/

static	U32	buffer[256L * 1024];	// 1MB of data.



/*****************************************************************************
*
*	Function:	tx_data
*
*	Purpose:
*
*		Perform a simple data transmit operation.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int	tx_data(void* handle)
{
	unsigned long	clock;
	int				errs	= 0;
	U32				sent	= 0;
	U32				status;

	label_text("Transmitting Data");
	printf("\n");
	label_level_inc();

	// Initialize the board.
	label_text("Initializing");
	status	= dmi32_init(handle);

	if (status != GSC_SUCCESS)
		errs	+= label_fail_func(1, "dmi32_init", status);
	else
		printf("PASS\n");

	// See if there is a clock.
	label_text("Checking Clock");
	status	= DMI32_XFER_CLKIN_TEST__YES(handle, &clock);

	if (status != GSC_SUCCESS)
		errs	+= label_fail_func(1, "DMI32_XFER_CLKIN_TEST__YES", status);
	else if (clock == DMI32_XFER_CLKIN_TEST_NONE)
		errs	+= label_fail_text(1, "NO INPUT CLOCK");
	else if (clock == DMI32_XFER_CLKIN_TEST_BUSY)
		errs	+= label_fail_text(1, "INPUT CLOCK PROBLEM");
	else
		printf("PASS\n");

	// Write data to DRAM: 1MB (each I/O offset initializes to zero).
	label_text("Writing to DRAM");
	status	= dmi32_write(handle, buffer, sizeof(buffer), &sent);

	if (status != GSC_SUCCESS)
		errs	+= label_fail_func(1, "dmi32_write", status);
	else
		printf("PASS\n");

	// Set the data transfer direction to Tx.
	label_text("Setting Direction to Tx");
	status	= DMI32_XFER_DIRECTION__TX(handle);

	if (status != GSC_SUCCESS)
		errs	+= label_fail_func(1, "DMI32_XFER_DIRECTION__TX", status);
	else
		printf("PASS\n");

	// Set the transfer offset and size.
	label_text("Setting Offset & Size");
	status	= dmi32_xfer_buf_set(handle, 0, 1024L * 1024);

	if (status != GSC_SUCCESS)
		errs	+= label_fail_func(1, "dmi32_xfer_buf_set", status);
	else
		printf("PASS\n");

	// Transmit the data (the mode defaults to Single Buffer).
	label_text("Initiating Transmit");
	status	= DMI32_XFER_ENABLE__YES(handle);
	// Wait for the data to be sent.
	sleep_ms(50);

	if (status != GSC_SUCCESS)
		errs	+= label_fail_func(1, "DMI32_XFER_ENABLE__YES", status);
	else
		printf("PASS\n");

	return(errs);
}


