/* Application - DMI32 - Linux */
/* $Workfile: util_sleep.c $ */
/* $Revision: 2 $ */
/* $Date: 10/27/07 11:08a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/utils/linux/util_sleep.c $
 *
 * 2     10/27/07 11:08a Dnewbold
 * Modified the implementation.
 *
 * 1     4/05/07 3:55p Dnewbold
*/

#include "util_sleep.h"
#include "gsc_common.h"

#include <pthread.h>
#include <unistd.h>
#include <sys/time.h>



/*****************************************************************************
*
*	Function:	sleep_ms
*
*	Purpose:
*
*		Sleep for the given number of milliseconds. The OS may sleep longer
*		than requested, much longer, due to factors outside our control.
*
*	Arguments:
*
*		ms	The number of milleseconds to sleep. If zero or less is given we
*			use one.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void sleep_ms(long ms)
{
	// This implementation puts only the calling thread to sleep.

	pthread_cond_t	cond;
	pthread_mutex_t	mutex;
	U32				sec;
	struct timespec	ts;
	struct timeval	tv;
	struct timezone	tz;
	U32				us;
	U32				x;

	pthread_cond_init(&cond, NULL);
	pthread_mutex_init(&mutex, NULL);
	pthread_mutex_lock(&mutex);

	gettimeofday(&tv, &tz);
	sec			= ms / 1000;
	x			= ms % 1000;
	us			= tv.tv_usec + (x * 1000);
	ts.tv_sec	= tv.tv_sec + sec + (us / 1000000L);
	ts.tv_nsec	= (tv.tv_usec * 1000) + (us % 1000000);;

	pthread_cond_timedwait(&cond, &mutex, &ts);

	pthread_mutex_unlock(&mutex);
	pthread_cond_destroy(&cond);
	pthread_mutex_destroy(&mutex);
}


