/* Application - DMI32 - Any OS */
/* $Workfile: util_id.c $ */
/* $Revision: 3 $ */
/* $Date: 10/27/07 11:06a $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/utils/util_id.c $
 *
 * 3     10/27/07 11:06a Dnewbold
 * Additional porting updates.
 *
 * 2     4/05/07 3:54p Dnewbold
 * Ported to Linux.
 *
 * 1     8/22/05 4:02p Dnewbold
*/

#include <stdio.h>

#include "util_id.h"

#include "dmi32_api.h"
#include "util_label.h"
#include "util_reg.h"



/*****************************************************************************
*
*	Function:	_id_api
*
*	Purpose:
*
*		Report the API version number the application was programmed for.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

static int _id_api(void)
{
	label_text("API Interface Version");
	printf("%d\n", DMI32_API_VERSION);
	return(0);
}



/*****************************************************************************
*
*	Function:	_id_api_lib
*
*	Purpose:
*
*		Report the version number for the API Library.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

static int _id_api_lib(void)
{
	char	buf[64];
	int		errs;
	U32		status;

	label_text("SDK Library Version");
	status	= dmi32_version_get(NULL, GSC_VERSION_LIBRARY, buf, sizeof(buf));

	if (status == GSC_SUCCESS)
	{
		errs	= 0;
		printf("%s\n", buf);
	}
	else
	{
		errs	= label_fail_func(1, "dmi32_version_get", status);
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_id_app
*
*	Purpose:
*
*		Identify the application.
*
*	Arguments:
*
*		dev		The handle of the board to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static int _id_app(const char* version)
{
	if (version)
	{
		label_text("Application Version");
		printf("%s\n", version);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_id_driver
*
*	Purpose:
*
*		Report the version number for the driver.
*
*	Arguments:
*
*		dev		The handle for the device.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

static int _id_driver(void* dev)
{
	char	buf[64];
	int		errs;
	U32		status;

	label_text("Device Driver Version");
	status	= dmi32_version_get(dev, GSC_VERSION_DRIVER, buf, sizeof(buf));

	if (status == GSC_SUCCESS)
	{
		errs	= 0;
		printf("%s\n", buf);
	}
	else
	{
		errs	= label_fail_func(1, "dmi32_version_get", status);
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_id_fr
*
*	Purpose:
*
*		Identify the board using the Features Register.
*
*	Arguments:
*
*		dev		The handle of the device to access.
*
*		frr		This is the FRR value.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

static int _id_fr(void* dev, U32 frr)
{
	int	errs	= 0;
	U32	field;
	U32	fr;

	label_text("Features Register");

	if ((frr & DMI32_FRR_FR) == 0)
	{
		printf("            (UNSUPPORTED)\n");
	}
	else
	{
		errs	+= dev_reg_read(dev, DMI32_FR, &fr, 1);

		if (errs == 0)
		{
			printf("0x%08lX\n", (long) fr);

			label_level_inc();

			/*
			*	Capacity
			*/

			label_text("4GB Capacity");
			field	= fr & DMI32_FR_4GB;
			fr		&= ~DMI32_FR_4GB;
			printf("0x-------%01lX  ", (long) field);
			printf("(%s)\n", field ? "Yes, 4GB max" : "No, 2GB max");

			/*
			*	Reserved
			*/

			label_text("Reserved");
			printf("0x%08lX  ", (long) fr);

			if (fr)
				errs	+= label_fail_text(1, "UNRECOGNIZED");
			else
				printf("\n");

			label_level_dec();
		}
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_id_frr
*
*	Purpose:
*
*		Identify the board using the Firmware Revision Register.
*
*	Arguments:
*
*		dev		The handle of the device to access.
*
*		frr		The value read from the FRR is put here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

static int _id_frr(void* dev, U32* frr)
{
	int	errs	= 0;
	U32	field;
	U32	tmp;

	label_text("Firmware Revision Reg");
	errs	+= dev_reg_read(dev, DMI32_FRR, frr, 1);

	if (errs == 0)
	{
		printf("0x%08lX\n", (long) frr[0]);

		label_level_inc();

		/*
		*	Firmware Revision.
		*/

		label_text("Firmware Revision");
		field	= DMI32_FRR_FR_DECODE(frr[0]);
		printf("0x------%02lX\n", (long) field);

		/*
		*	Device ID
		*/

		label_text("Device ID");
		field	= DMI32_FRR_DID_DECODE(frr[0]);
		printf("0x----%02lX--  \n", (long) field);

		/*
		*	Board ID
		*/

		label_text("Board ID");
		field	= DMI32_FRR_BID_DECODE(frr[0]);
		printf("0x--%02lX----  \n", (long) field);

		/*
		*	Miscellaneous
		*/

		label_text("Miscellaneous");
		field	= DMI32_FRR_MISC_DECODE(frr[0]);
		printf("0x%02lX------  \n", (long) field);

		label_level_inc();

		/*
		*	Misc: Features Register
		*/

		label_text("Features Register");
		tmp		= field & DMI32_FRR_MISC_FR;
		field	&= ~DMI32_FRR_MISC_FR;
		printf("0x%02lX------  (%s)\n", (long) tmp, tmp ? "yes" : "no");

		/*
		*	Misc: Reserved
		*/

		label_text("Reserved");
		printf("0x%02lX------  ", (long) field);

		if (field)
			errs	+= label_fail_text(1, "UNRECOGNIZED");
		else
			printf("\n");

		label_level_dec();
		label_level_dec();
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_id_pci
*
*	Purpose:
*
*		Identify the board using the PCI register information.
*
*	Arguments:
*
*		dev		The handle of the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

static int _id_pci(void* dev)
{
	U32	didr;
	int	err;
	int	errs	= 0;
	U32	sid;
	U32	svid;
	U32	vidr;

	/*
	*	Vendor ID
	*/

	label_text("Vendor ID");
	err		= dev_reg_read(dev, GSC_PCI_9080_VIDR, &vidr, 1);
	errs	+= err;

	if (err == 0)
	{
		printf("0x%04lX      ", (long) vidr);

		if (vidr == DMI32_VENDOR_ID)
			printf("(PLX)\n");
		else
			errs	+= label_fail_text(1, "NOT RECOGNIZED");
	}

	/*
	*	Device ID
	*/

	label_text("Device ID");
	err		= dev_reg_read(dev, GSC_PCI_9080_DIDR, &didr, 1);
	errs	+= err;

	if (err == 0)
	{
		printf("0x%04lX      ", (long) didr);

		if (didr == DMI32_DEVICE_ID)
			printf("(PCI9080)\n");
		else
			errs	+= label_fail_text(1, "NOT RECOGNIZED");
	}

	/*
	*	Subsystem Vendor ID
	*/

	label_text("Subsystem Vendor ID");
	err		= dev_reg_read(dev, GSC_PCI_9080_SVID, &svid, 1);
	errs	+= err;

	if (err == 0)
	{
		printf("0x%04lX      ", (long) svid);

		if (svid == DMI32_SUBVENDOR_ID)
			printf("(Subsystem ID is PLX assigned)\n");
		else
			errs	+= label_fail_text(1, "NOT RECOGNIZED");
	}

	/*
	*	Subsystem ID
	*/

	label_text("Subsystem ID");
	err		= dev_reg_read(dev, GSC_PCI_9080_SID, &sid, 1);
	errs	+= err;

	if (err == 0)
	{
		printf("0x%04lX      ", (long) sid);

		if (sid == DMI32_SUBSYSTEM_ID)
			printf("(DMI32)\n");
		else
			errs	+= label_fail_text(1, "NOT RECOGNIZED");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_id_user_jumpers
*
*	Purpose:
*
*		Identify the board using the User Jumpers.
*
*	Arguments:
*
*		dev		The handle of the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

static int _id_user_jumpers(void* dev)
{
	unsigned long	bj;
	int				errs;
	U32				status;

	label_text("User Jumpers");
	status	= DMI32_MISC_BOARD_JUMPERS__GET(dev, &bj);

	if (status != GSC_SUCCESS)
	{
		errs	= label_fail_func(	1,
									"DMI32_MISC_BOARD_JUMPERS__GET",
									status);
	}
	else
	{
		errs	= 0;
		printf(	"0x%01lX         (UJ1 %s, UJ0 %s)\n",
				(long) bj,
				(bj & 0x2) ? "On" : "Off",
				(bj & 0x1) ? "On" : "Off");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	id_hardware
*
*	Purpose:
*
*		Identify the hardware being used.
*
*	Arguments:
*
*		dev		The handle for the device.
*
*		index	The index of the device.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

int id_hardware(void* dev, int index)
{
	char	buf[128];
	int		errs	= 0;
	U32		frr;

	sprintf(buf, "Hardware Identification (#%d)", index);
	label_text(buf);
	printf("\n");
	label_level_inc();

	errs	+= _id_pci(dev);
	errs	+= _id_frr(dev, &frr);
	errs	+= _id_fr(dev, frr);
	errs	+= _id_user_jumpers(dev);

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	id_software
*
*	Purpose:
*
*		Identify the software being used.
*
*	Arguments:
*
*		dev		The handle for the device.
*
*		app_ver	The version for the application.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
*****************************************************************************/

int id_software(void* dev, const char* app_ver)
{
	int		errs	= 0;

	label_text("Software Identification");
	printf("\n");
	label_level_inc();

	errs	+= _id_app(app_ver);
	errs	+= _id_api();
	errs	+= _id_api_lib();
	errs	+= _id_driver(dev);

	label_level_dec();
	return(errs);
}


