/* Application - DMI32 - Any OS */
/* $Workfile: util_label.c $ */
/* $Revision: 1 $ */
/* $Date: 8/22/05 4:01p $ */
/* $Log: /NEW SOURCESAFE DATABASE/SDK/PLX/gsc/dmi32/utils/util_label.c $
 *
 * 1     8/22/05 4:01p Dnewbold
*/

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "util_label.h"



/* variables	*************************************************************/

static	int	_width	= 0;
static	int	_level	= 0;



/*****************************************************************************
*
*	Function:	label_text
*
*	Purpose:
*
*		Print a message introducing a test.
*
*	Arguments:
*
*		label	The test name.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void label_text(const char* label)
{
	int	i;
	int	spaces;

	spaces	= _width - strlen(label);
	spaces	= (spaces < 0) ? 0 : spaces;

	for (i = 0; i < _level; i++)
	{
		printf("  ");
		spaces	-= 2;
	}

	printf("%s: %*s", label, spaces, "");
	fflush(stdout);
}



/*****************************************************************************
*
*	Function:	label_init
*
*	Purpose:
*
*		Initialize the label code.
*
*	Arguments:
*
*		width	The default width for the label field.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void label_init(int width)
{
	_width	= width;
}



/*****************************************************************************
*
*	Function:	label_level_dec
*
*	Purpose:
*
*		Decrease the label indenting.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void label_level_dec(void)
{
	_level--;
}



/*****************************************************************************
*
*	Function:	label_level_inc
*
*	Purpose:
*
*		Increase the label indenting.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void label_level_inc(void)
{
	_level++;
}



/*****************************************************************************
*s
*	Function:	label_fail_func
*
*	Purpose:
*
*		Report a failure for the given function if verbose is enabled, and if
*		the status is an error.
*
*	Arguments:
*
*		verbose	Work verbosely?
*
*		func	The name of the function that returned the error.
*
*		status	The status returned by the function.
*
*	Returned:
*
*		1		The status indicated an error.
*		0		The satus did not indicate an error.
*
*****************************************************************************/

int label_fail_func(int verbose, const char* func, U32 status)
{
	char	buf[256];
	int		errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if ((errs) && (func) && (verbose))
	{
		dmi32_status_text(status, buf, sizeof(buf));
		printf("FAIL <---  (%s(), %s)\n", func, buf);
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	label_fail_text
*
*	Purpose:
*
*		Report a failure with the given text if verbose is enabled.
*
*	Arguments:
*
*		verbose	Work verbosely?
*
*		text	The text to report.
*
*	Returned:
*
*		1		This is an error condition.
*
*****************************************************************************/

int label_fail_text(int verbose, const char* text)
{
	if ((verbose) && (text))
		printf("FAIL <---  (%s)\n", text);

	return(1);
}



/*****************************************************************************
*
*	Function:	label_fail_value
*
*	Purpose:
*
*		Report a failure with the given value if verbose is enabled.
*
*	Arguments:
*
*		verbose	Work verbosely?
*
*		value	The value to report.
*
*	Returned:
*
*		1		This is an error condition.
*
*****************************************************************************/

int label_fail_value(int verbose, long value)
{
	if (verbose)
	{
		printf(	"FAIL <---  (invalid value: %ld, 0x%lX)\n",
			(long) value,
			(long) value);
	}

	return(1);
}


